@@ -25,25 +25,30 @@
 #include "artistview.h"
 #include "albumview.h"
 #include "songview.h"
-#include "song.h"
-#include "utils.h"
-#include "covers.h"
-#include "networkaccessmanager.h"
-#include "settings.h"
+#ifdef ENABLE_ONLINE_SERVICES
+#include "onlineview.h"
+#endif
+#include "mpd/song.h"
+#include "support/utils.h"
+#include "gui/covers.h"
+#include "network/networkaccessmanager.h"
+#include "gui/settings.h"
 #include "wikipediaengine.h"
-#include "localize.h"
+#include "support/localize.h"
 #include "backdropcreator.h"
-#include "sizewidget.h"
-#include "gtkstyle.h"
-#include "qjson/parser.h"
-#include "playqueueview.h"
-#include "treeview.h"
+#include "support/gtkstyle.h"
+#include "widgets/playqueueview.h"
+#include "widgets/treeview.h"
+#include "widgets/thinsplitterhandle.h"
 #include <QHBoxLayout>
 #include <QGridLayout>
 #include <QSpacerItem>
 #include <QStylePainter>
 #if QT_VERSION >= 0x050000
 #include <QUrlQuery>
+#include <QJsonDocument>
+#else
+#include "qjson/parser.h"
 #endif
 #include <QXmlStreamReader>
 #include <QFile>
@@ -57,7 +62,6 @@
 #include <QPair>
 #include <QImage>
 #include <QToolButton>
-#include <QStyleOptionToolButton>
 #include <QButtonGroup>
 #include <QWheelEvent>
 #include <qglobal.h>
@@ -77,15 +81,12 @@ const QLatin1String ContextWidget::constBackdropFileName("backdrop");
 //const QLatin1String ContextWidget::constHtbApiKey(0); // API key required
 const QLatin1String ContextWidget::constFanArtApiKey("ee86404cb429fa27ac32a1a3c117b006");
 const QLatin1String ContextWidget::constCacheDir("backdrops/");
-static const double constBgndOpacity=0.15;
 
 static QString cacheFileName(const QString &artist, bool createDir)
 {
     return Utils::cacheDir(ContextWidget::constCacheDir, createDir)+Covers::encodeName(artist)+".jpg";
 }
 
-#define SIMPLE_VSB
-
 class ViewSelectorButton : public QToolButton
 {
 public:
@@ -93,56 +94,22 @@ public:
     void paintEvent(QPaintEvent *ev)
     {
         Q_UNUSED(ev)
-        QStylePainter painter(this);
-        QStyleOptionToolButton opt;
-        initStyleOption(&opt);
-        bool isOn=opt.state&QStyle::State_On;
-        bool isMo=opt.state&QStyle::State_MouseOver;
-        if (isOn || isMo) {
-            #ifdef SIMPLE_VSB
-            QColor col=palette().highlight().color();
-            col.setAlphaF(isMo && !isOn ? 0.15 : 0.35);
-            painter.fillRect(rect().adjusted(0, 1, 0, 0), col);
-            #else
-            QStyleOptionViewItemV4 styleOpt;
-            styleOpt.palette=opt.palette;
-            styleOpt.rect=rect().adjusted(0, 1, 0, 0);
-            styleOpt.state=opt.state;
-            styleOpt.state&=~(QStyle::State_Selected|QStyle::State_MouseOver);
-            styleOpt.state|=QStyle::State_Selected|QStyle::State_Enabled;
-            styleOpt.viewItemPosition = QStyleOptionViewItemV4::OnlyOne;
-            styleOpt.showDecorationSelected=true;
+        QPainter painter(this);
+        QString txt=text();
 
-            if (GtkStyle::isActive()) {
-                GtkStyle::drawSelection(styleOpt, &painter, isMo && !isOn ? 0.15 : 1.0);
-            } else {
-                if (isMo && !isOn) {
-                    QColor col(styleOpt.palette.highlight().color());
-                    col.setAlphaF(0.15);
-                    styleOpt.palette.setColor(styleOpt.palette.currentColorGroup(), QPalette::Highlight, col);
-                }
-                style()->drawPrimitive(QStyle::PE_PanelItemViewItem, &styleOpt, &painter, 0);
-            }
-            #endif
+        txt.replace("&", "");
+        QFont f(font());
+        if (isChecked()) {
+            f.setBold(true);
+        }
+        QFontMetrics fm(f);
+        if (fm.width(txt)>rect().width()) {
+            txt=fm.elidedText(txt, isRightToLeft() ? Qt::ElideLeft : Qt::ElideRight, rect().width());
         }
 
-        int alignment = Qt::AlignCenter | Qt::TextShowMnemonic;
-        if (!style()->styleHint(QStyle::SH_UnderlineShortcut, &opt, this)) {
-            alignment |= Qt::TextHideMnemonic;
-        }
-
-        QString text=opt.text;
-        if (fontMetrics().width(text)>rect().width()) {
-            text=fontMetrics().elidedText(text, Qt::RightToLeft==layoutDirection() ? Qt::ElideLeft : Qt::ElideRight, rect().width());
-        }
-        #ifdef SIMPLE_VSB
-        painter.drawItemText(rect(), alignment, opt.palette, true, text, QPalette::WindowText);
-        #else
-        if (isOn) {
-            opt.state|=QStyle::State_Selected;
-        }
-        painter.drawItemText(rect(), alignment, opt.palette, true, text, isOn ? QPalette::HighlightedText : QPalette::WindowText);
-        #endif
+        painter.setFont(f);
+        painter.setPen(palette().color(!isChecked() && underMouse() ? QPalette::Highlight : QPalette::Text));
+        painter.drawText(rect(), Qt::AlignCenter, txt);
     }
 };
 
@@ -152,7 +119,6 @@ ViewSelector::ViewSelector(QWidget *p)
     : QWidget(p)
 {
     group=new QButtonGroup(this);
-    setFixedHeight(SizeWidget::standardHeight());
 }
 
 void ViewSelector::addItem(const QString &label, const QVariant &data)
@@ -180,17 +146,8 @@ void ViewSelector::addItem(const QString &label, const QVariant &data)
 void ViewSelector::buttonActivated()
 {
     QToolButton *button=qobject_cast<QToolButton *>(sender());
-    if (!button) {
-        return;
-    }
-
-    if (button->isChecked()) {
-        QFont f(font());
-        f.setBold(true);
-        button->setFont(f);
+    if (button && button->isChecked()) {
         emit activated(buttons.indexOf(button));
-    } else {
-        button->setFont(font());
     }
 }
 
@@ -217,9 +174,6 @@ void ViewSelector::setCurrentIndex(int index)
         bool wasChecked=btn->isChecked();
         btn->setChecked(i==index);
         if (i==index) {
-            QFont bf(f);
-            bf.setBold(true);
-            btn->setFont(bf);
             emit activated(i);
         } else if (wasChecked) {
             btn->setFont(f);
@@ -244,122 +198,28 @@ void ViewSelector::wheelEvent(QWheelEvent *ev)
     }
 }
 
-static void drawFadedLine(QPainter *p, const QRect &r, const QColor &col)
-{
-    QPoint start(r.x(), r.y());
-    QPoint end(r.x()+r.width()-1, r.y()+r.height()-1);
-    QLinearGradient grad(start, end);
-    QColor c(col);
-    c.setAlphaF(0.45);
-    QColor fade(c);
-    const int fadeSize=Utils::isHighDpi() ? 64 : 32;
-    double fadePos=1.0-((r.width()-(fadeSize*2))/(r.width()*1.0));
-
-    fade.setAlphaF(0.0);
-    if(fadePos>=0 && fadePos<=1.0) {
-        grad.setColorAt(0, fade);
-        grad.setColorAt(fadePos, c);
-    } else {
-        grad.setColorAt(0, c);
-    }
-    if(fadePos>=0 && fadePos<=1.0) {
-        grad.setColorAt(1.0-fadePos, c);
-        grad.setColorAt(1, fade);
-    } else {
-        grad.setColorAt(1, c);
-    }
-    p->setPen(QPen(QBrush(grad), 1));
-    p->drawLine(start, end);
-}
-
 void ViewSelector::paintEvent(QPaintEvent *ev)
 {
-    QWidget::paintEvent(ev);
-    QPainter p(this);
-    QRect r=rect();
-    r.setHeight(1);
-    drawFadedLine(&p, r, palette().foreground().color());
+    Q_UNUSED(ev)
 }
 
-static QColor splitterColor;
-
-class ThinSplitterHandle : public QSplitterHandle
-{
-public:
-    ThinSplitterHandle(Qt::Orientation orientation, ThinSplitter *parent)
-        : QSplitterHandle(orientation, parent)
-        , underMouse(false)
-    {
-        setMask(QRegion(contentsRect()));
-        setAttribute(Qt::WA_MouseNoMask, true);
-        setAttribute(Qt::WA_OpaquePaintEvent, false);
-        setAttribute(Qt::WA_MouseTracking, true);
-        QAction *act=new QAction(i18n("Reset Spacing"), this);
-        addAction(act);
-        connect(act, SIGNAL(triggered(bool)), parent, SLOT(reset()));
-        setContextMenuPolicy(Qt::ActionsContextMenu);
-        size=Utils::isHighDpi() ? 4 : 2;
-    }
-
-    void resizeEvent(QResizeEvent *event)
-    {
-        if (Qt::Horizontal==orientation()) {
-            setContentsMargins(size, 0, size, 0);
-        } else {
-            setContentsMargins(0, size, 0, size);
-        }
-        setMask(QRegion(contentsRect()));
-        QSplitterHandle::resizeEvent(event);
-    }
-
-    void paintEvent(QPaintEvent *event)
-    {
-        if (underMouse) {
-            QColor col(splitterColor);
-            QPainter p(this);
-            col.setAlphaF(0.75);
-            p.fillRect(event->rect().adjusted(1, 0, -1, 0), col);
-            col.setAlphaF(0.25);
-            p.fillRect(event->rect(), col);
-        }
-    }
-
-    bool event(QEvent *event)
-    {
-        switch(event->type()) {
-        case QEvent::Enter:
-        case QEvent::HoverEnter:
-            underMouse = true;
-            update();
-            break;
-        case QEvent::ContextMenu:
-        case QEvent::Leave:
-        case QEvent::HoverLeave:
-            underMouse = false;
-            update();
-            break;
-        default:
-            break;
-        }
-
-        return QWidget::event(event);
-    }
-
-    bool underMouse;
-    int size;
-};
-
 ThinSplitter::ThinSplitter(QWidget *parent)
     : QSplitter(parent)
 {
-    setHandleWidth(3);
-    setChildrenCollapsible(false);
+    setChildrenCollapsible(true);
     setOrientation(Qt::Horizontal);
+    resetAct=new QAction(i18n("Reset Spacing"), this);
+    connect(resetAct, SIGNAL(triggered(bool)), this, SLOT(reset()));
+    setHandleWidth(0);
 }
 
 QSplitterHandle * ThinSplitter::createHandle()
 {
-    return new ThinSplitterHandle(orientation(), this);
+    ThinSplitterHandle *handle=new ThinSplitterHandle(orientation(), this);
+    handle->addAction(resetAct);
+    handle->setContextMenuPolicy(Qt::ActionsContextMenu);
+    handle->setHighlightUnderMouse();
+    return handle;
 }
 
 void ThinSplitter::reset()
@@ -381,26 +241,34 @@ ContextWidget::ContextWidget(QWidget *parent)
     : QWidget(parent)
     , job(0)
     , alwaysCollapsed(false)
-    , backdropType(true)
+    , backdropType(PlayQueueView::BI_Cover)
     , darkBackground(false)
-//    , useHtBackdrops(0!=constHtbApiKey.latin1())
     , useFanArt(0!=constFanArtApiKey.latin1())
     , albumCoverBackdrop(false)
     , oldIsAlbumCoverBackdrop(false)
-    , fadeValue(0)
+    , fadeValue(1.0)
     , isWide(false)
     , stack(0)
+    #ifdef ENABLE_ONLINE_SERVICES
+    , onlineContext(0)
+    #endif
     , splitter(0)
     , viewSelector(0)
     , creator(0)
 {
+    QHBoxLayout *layout=new QHBoxLayout(this);
+    mainStack=new QStackedWidget(this);
+    standardContext=new QWidget(mainStack);
+    mainStack->addWidget(standardContext);
+    layout->setMargin(0);
+    layout->addWidget(mainStack);
     animator.setPropertyName("fade");
     animator.setTargetObject(this);
 
     appLinkColor=QApplication::palette().color(QPalette::Link);
-    artist = new ArtistView(this);
-    album = new AlbumView(this);
-    song = new SongView(this);
+    artist = new ArtistView(standardContext);
+    album = new AlbumView(standardContext);
+    song = new SongView(standardContext);
     minWidth=album->picSize().width()*2.5;
 
     artist->addEventFilter(this);
@@ -413,7 +281,6 @@ ContextWidget::ContextWidget(QWidget *parent)
     readConfig();
     setZoom();
     setWide(true);
-    splitterColor=palette().text().color();
 
     #ifndef SCALE_CONTEXT_BGND
     QDesktopWidget *dw=QApplication::desktop();
@@ -423,11 +290,8 @@ ContextWidget::ContextWidget(QWidget *parent)
         minBackdropSize.setWidth(((int)(minBackdropSize.width()/32))*32);
         minBackdropSize.setHeight(((int)(minBackdropSize.height()/32))*32);
         maxBackdropSize=QSize(geo.width()*1.25, geo.height()*1.25);
-    } else if (Utils::isHighDpi()) {
-        minBackdropSize=QSize(1024*3, 768*3);
-        maxBackdropSize=QSize(minBackdropSize.width()*2, minBackdropSize.height()*2);
     } else {
-        minBackdropSize=QSize(1024, 768);
+        minBackdropSize=QSize(Utils::scaleForDpi(1024*3), Utils::scaleForDpi(768*3);
         maxBackdropSize=QSize(minBackdropSize.width()*2, minBackdropSize.height()*2);
     }
     #endif
@@ -451,11 +315,11 @@ void ContextWidget::setWide(bool w)
 
     isWide=w;
     if (w) {
-        if (layout()) {
-            delete layout();
+        if (standardContext->layout()) {
+            delete standardContext->layout();
         }
-        QHBoxLayout *l=new QHBoxLayout(this);
-        setLayout(l);
+        QHBoxLayout *l=new QHBoxLayout(standardContext);
+        standardContext->setLayout(l);
         int m=l->margin()/2;
         l->setMargin(0);
         if (stack) {
@@ -472,7 +336,7 @@ void ContextWidget::setWide(bool w)
         QByteArray state;
         bool resetSplitter=splitter;
         if (!splitter) {
-            splitter=new ThinSplitter(this);
+            splitter=new ThinSplitter(standardContext);
             state=Settings::self()->contextSplitterState();
         }
         l->addWidget(splitter);
@@ -488,29 +352,24 @@ void ContextWidget::setWide(bool w)
         } else if (!state.isEmpty()) {
             splitter->restoreState(state);
         }
-//        l->addWidget(album);
-//        l->addWidget(song);
-        //    layout->addItem(new QSpacerItem(m, m, QSizePolicy::Fixed, QSizePolicy::Fixed));
-//        l->setStretch(1, 1);
-//        l->setStretch(2, 1);
-//        l->setStretch(3, 1);
     } else {
-        if (layout()) {
-            delete layout();
+        if (standardContext->layout()) {
+            delete standardContext->layout();
         }
-        QGridLayout *l=new QGridLayout(this);
-        setLayout(l);
+        QGridLayout *l=new QGridLayout(standardContext);
+        standardContext->setLayout(l);
         int m=l->margin()/2;
         l->setMargin(0);
         l->setSpacing(0);
         if (!stack) {
-            stack=new QStackedWidget(this);
+            stack=new QStackedWidget(standardContext);
         }
         if (!viewSelector) {
-            viewSelector=new ViewSelector(this);
+            viewSelector=new ViewSelector(standardContext);
             viewSelector->addItem(i18n("&Artist"), "artist");
             viewSelector->addItem(i18n("Al&bum"), "album");
-            viewSelector->addItem(i18n("&Lyrics"), "song");
+            viewSelector->addItem(i18n("&Track"), "song");
+            viewSelector->setPalette(palette());
             connect(viewSelector, SIGNAL(activated(int)), stack, SLOT(setCurrentIndex(int)));
         }
         if (splitter) {
@@ -564,25 +423,21 @@ void ContextWidget::readConfig()
     switch (backdropType) {
     case PlayQueueView::BI_None:
         if (origType!=backdropType && isVisible() && !currentArtist.isEmpty()) {
-            updateArtist=currentArtist;
-            currentArtist.clear();
-            updateBackdrop();
+            updateBackdrop(true);
             QWidget::update();
         }
         break;
     case PlayQueueView::BI_Cover:
         if (origType!=backdropType || backdropOpacity!=origOpacity || backdropBlur!=origBlur) {
             if (isVisible() && !currentArtist.isEmpty()) {
-                updateArtist=currentArtist;
-                currentArtist.clear();
-                updateBackdrop();
+                updateBackdrop(true);
                 QWidget::update();
             }
         }
         break;
    case PlayQueueView::BI_Custom:
         if (origType!=backdropType || backdropOpacity!=origOpacity || backdropBlur!=origBlur || origCustomBackdropFile!=customBackdropFile) {            
-            updateImage(QImage(customBackdropFile), true);
+            updateImage(QImage(customBackdropFile), false);
             artistsCreatedBackdropsFor.clear();
         }
         break;
@@ -606,6 +461,7 @@ void ContextWidget::saveConfig()
     if (splitter) {
         Settings::self()->saveContextSplitterState(splitter->saveState());
     }
+    song->saveConfig();
 }
 
 void ContextWidget::useDarkBackground(bool u)
@@ -631,16 +487,17 @@ void ContextWidget::useDarkBackground(bool u)
             pal.setColor(QPalette::LinkVisited, linkVisited);
             prevLinkColor=appLinkColor;
             linkCol=pal.color(QPalette::Link);
-            splitterColor=light;
         } else {
             linkCol=appLinkColor;
             prevLinkColor=QColor(240, 240, 240);
-            splitterColor=pal.text().color();
         }
         setPalette(pal);
         artist->setPal(pal, linkCol, prevLinkColor);
         album->setPal(pal, linkCol, prevLinkColor);
         song->setPal(pal, linkCol, prevLinkColor);
+        if (viewSelector) {
+            viewSelector->setPalette(pal);
+        }
         QWidget::update();
     }
 }
@@ -659,10 +516,6 @@ void ContextWidget::paintEvent(QPaintEvent *e)
     QPainter p(this);
     QRect r(rect());
 
-    if (!isWide && viewSelector) {
-        int space=2; // fontMetrics().height()/4;
-        r.adjust(0, 0, 0, -(viewSelector->rect().height()+space));
-    }
     if (darkBackground) {
         p.fillRect(r, palette().background().color());
     }
@@ -687,16 +540,6 @@ void ContextWidget::paintEvent(QPaintEvent *e)
             #endif
             p.fillRect(r, QBrush(currentBackdrop));
         }
-//        if (!backdropText.isEmpty() && isWide) {
-//            int pad=fontMetrics().height()*2;
-//            QFont f("Sans", font().pointSize()*12);
-//            f.setBold(true);
-//            p.setFont(f);
-//            p.setOpacity(0.15);
-//            QTextOption textOpt(Qt::AlignBottom|(Qt::RightToLeft==layoutDirection() ? Qt::AlignRight : Qt::AlignLeft));
-//            textOpt.setWrapMode(QTextOption::NoWrap);
-//            p.drawText(QRect(pad, pad, width(), height()-(2*pad)), backdropText, textOpt);
-//        }
     }
     if (!darkBackground) {
         QWidget::paintEvent(e);
@@ -718,7 +561,6 @@ void ContextWidget::setFade(float value)
 void ContextWidget::updateImage(QImage img, bool created)
 {
     DBUG << img.isNull() << currentBackdrop.isNull();
-//    backdropText=currentArtist;
     oldBackdrop=currentBackdrop;
     oldIsAlbumCoverBackdrop=albumCoverBackdrop;
     currentBackdrop=QPixmap();
@@ -750,10 +592,15 @@ void ContextWidget::updateImage(QImage img, bool created)
     albumCoverBackdrop=created;
     resizeBackdrop();
 
-    fadeValue=0.0;
-    animator.setDuration(250);
-    animator.setEndValue(1.0);
-    animator.start();
+    animator.stop();
+    if (PlayQueueView::BI_Custom==backdropType || !isVisible()) {
+        setFade(1.0);
+    } else {
+        fadeValue=0.0;
+        animator.setDuration(250);
+        animator.setEndValue(1.0);
+        animator.start();
+    }
 }
 
 void ContextWidget::search()
@@ -770,7 +617,7 @@ void ContextWidget::update(const Song &s)
         sng.revertVariousArtists();
     }
 
-    if (sng.isStream() && !sng.isCantataStream() && !sng.isCdda() && sng.artist.isEmpty() && sng.albumartist.isEmpty() && sng.album.isEmpty()) {
+    if (sng.isStandardStream() && sng.artist.isEmpty() && sng.albumartist.isEmpty() && sng.album.isEmpty()) {
         int pos=sng.title.indexOf(QLatin1String(" - "));
         if (pos>3) {
             sng.artist=sng.title.left(pos);
@@ -781,6 +628,29 @@ void ContextWidget::update(const Song &s)
     if (s.albumArtist()!=currentSong.albumArtist()) {
         cancel();
     }
+
+    #ifdef ENABLE_ONLINE_SERVICES
+    if (Song::OnlineSvrTrack==sng.type) {
+        if (!onlineContext) {
+            QWidget *onlinePage=new QWidget(mainStack);
+            QHBoxLayout *onlineLayout=new QHBoxLayout(onlinePage);
+            int m=onlineLayout->margin()/2;
+            onlineLayout->setMargin(0);
+            onlineLayout->addItem(new QSpacerItem(m, m, QSizePolicy::Fixed, QSizePolicy::Fixed));
+            onlineContext=new OnlineView(onlinePage);
+            onlineLayout->addWidget(onlineContext);
+            mainStack->addWidget(onlinePage);
+        }
+        onlineContext->update(sng);
+        mainStack->setCurrentIndex(1);
+        updateArtist=QString();
+        if (isVisible() && PlayQueueView::BI_Cover==backdropType) {
+            updateBackdrop();
+        }
+        return;
+    }
+    mainStack->setCurrentIndex(0);
+    #endif
     artist->update(sng);
     album->update(sng);
     song->update(sng);
@@ -816,10 +686,10 @@ void ContextWidget::cancel()
     }
 }
 
-void ContextWidget::updateBackdrop()
+void ContextWidget::updateBackdrop(bool force)
 {
-    DBUG << updateArtist << currentArtist << currentSong.file;
-    if (updateArtist==currentArtist) {
+    DBUG << updateArtist << currentArtist << currentSong.file << force;
+    if (!force && updateArtist==currentArtist) {
         return;
     }
     currentArtist=updateArtist;
@@ -1024,13 +894,15 @@ void ContextWidget::fanArtResponse()
     QString url;
 
     if (reply->ok()) {
-        QJson::Parser parser;
-        bool ok=false;
-        #ifdef Q_OS_WIN
-        QVariantMap parsed=parser.parse(reply->readAll(), &ok).toMap();
+        #if QT_VERSION >= 0x050000
+        QJsonParseError jsonParseError;
+        QVariantMap parsed=QJsonDocument::fromJson(reply->readAll(), &jsonParseError).toVariant().toMap();
+        bool ok=QJsonParseError::NoError==jsonParseError.error;
         #else
-        QVariantMap parsed=parser.parse(reply->actualJob(), &ok).toMap();
+        bool ok=false;
+        QVariantMap parsed = QJson::Parser().parse(reply->readAll(), &ok).toMap();
         #endif
+
         if (ok && !parsed.isEmpty()) {
             QVariantMap artist=parsed[parsed.keys().first()].toMap();
             if (artist.contains("artistbackground")) {
@@ -1104,13 +976,15 @@ void ContextWidget::discoGsResponse()
     QString url;
 
     if (reply->ok()) {
-        QJson::Parser parser;
-        bool ok=false;
-        #ifdef Q_OS_WIN
-        QVariantMap parsed=parser.parse(reply->readAll(), &ok).toMap();
+        #if QT_VERSION >= 0x050000
+        QJsonParseError jsonParseError;
+        QVariantMap parsed=QJsonDocument::fromJson(reply->readAll(), &jsonParseError).toVariant().toMap();
+        bool ok=QJsonParseError::NoError==jsonParseError.error;
         #else
-        QVariantMap parsed=parser.parse(reply->actualJob(), &ok).toMap();
+        bool ok=false;
+        QVariantMap parsed = QJson::Parser().parse(reply->readAll(), &ok).toMap();
         #endif
+
         if (ok && parsed.contains("resp")) {
             QVariantMap response=parsed["resp"].toMap();
             if (response.contains("search")) {
