@@ -22,32 +22,32 @@
  */
 
 #include "actiondialog.h"
-#include "devicesmodel.h"
+#include "models/devicesmodel.h"
 #include "device.h"
-#include "utils.h"
+#include "support/utils.h"
 #include "devicepropertiesdialog.h"
 #include "devicepropertieswidget.h"
-#include "settings.h"
-#include "musiclibrarymodel.h"
-#include "musiclibraryproxymodel.h"
-#include "dirviewmodel.h"
-#include "albumsmodel.h"
-#include "mpdparseutils.h"
-#include "mpdconnection.h"
+#include "gui/settings.h"
+#include "models/musiclibrarymodel.h"
+#include "models/musiclibraryproxymodel.h"
+#include "models/dirviewmodel.h"
+#include "models/albumsmodel.h"
+#include "mpd/mpdparseutils.h"
+#include "mpd/mpdconnection.h"
 #include "encoders.h"
-#include "localize.h"
-#include "messagebox.h"
+#include "support/localize.h"
+#include "support/messagebox.h"
 #include "filejob.h"
 #include "freespaceinfo.h"
-#include "icons.h"
+#include "widgets/icons.h"
 #include "config.h"
-#include "tags.h"
-#include "treeview.h"
+#include "tags/tags.h"
+#include "widgets/treeview.h"
 #ifdef ENABLE_ONLINE_SERVICES
-#include "onlineservicesmodel.h"
+#include "models/onlineservicesmodel.h"
 #endif
 #ifdef ENABLE_REPLAYGAIN_SUPPORT
-#include "rgdialog.h"
+#include "replaygain/rgdialog.h"
 #endif
 #include <QFile>
 #include <QDebug>
@@ -172,6 +172,20 @@ void ActionDialog::controlInfoLabel(Device *dev)
     }
 }
 
+void ActionDialog::deviceRenamed()
+{
+    Device *dev=getDevice(sourceUdi.isEmpty() ? destUdi : sourceUdi);
+    if (!dev) {
+        return;
+    }
+
+    if (Remove==mode || (Copy==mode && !sourceUdi.isEmpty())) {
+        sourceLabel->setText(QLatin1String("<b>")+dev->data()+QLatin1String("</b>"));
+    } else {
+        destinationLabel->setText(QLatin1String("<b>")+dev->data()+QLatin1String("</b>"));
+    }
+}
+
 void ActionDialog::showSongs()
 {
     if (!songDialog) {
@@ -742,6 +756,7 @@ void ActionDialog::configure(const QString &udi)
         if (dev) {
             dev->configure(this);
             connect(dev, SIGNAL(configurationChanged()), SLOT(controlInfoLabel()));
+            connect(dev, SIGNAL(renamed()), SLOT(deviceRenamed()));
         }
     }
 }
@@ -801,7 +816,7 @@ ActionDialog::StringPairList ActionDialog::formatSong(const Song &s, bool showFi
     StringPairList str;
     str.append(StringPair(i18n("Artist:"), s.albumArtist()));
     str.append(StringPair(i18n("Album:"), s.album));
-    str.append(StringPair(i18n("Track:"), s.trackAndTitleStr(Song::isVariousArtists(s.albumArtist()) && !Song::isVariousArtists(s.artist))));
+    str.append(StringPair(i18n("Track:"), s.trackAndTitleStr()));
 
     if (showFiles) {
         if (Copy==mode) {
@@ -838,6 +853,7 @@ bool ActionDialog::refreshLibrary()
     if (!actionedSongs.isEmpty()) {
         if ( (Copy==mode && !sourceUdi.isEmpty()) ||
              (Remove==mode && sourceUdi.isEmpty()) ) {
+            MusicLibraryModel::self()->checkForNewSongs();
             AlbumsModel::self()->update(MusicLibraryModel::self()->root());
             emit update();
         } else if ( (Copy==mode && sourceUdi.isEmpty()) ||
@@ -845,9 +861,9 @@ bool ActionDialog::refreshLibrary()
             Device *dev=DevicesModel::self()->device(sourceUdi.isEmpty() ? destUdi : sourceUdi);
 
             if (dev) {
+                connect(dev, SIGNAL(cacheSaved()), this, SLOT(cacheSaved()));
                 dev->saveCache();
                 progressLabel->setText(i18n("Saving cache"));
-                connect(dev, SIGNAL(cacheSaved()), this, SLOT(cacheSaved()));
                 setButtons(Close);
                 return true;
             }
