@@ -22,41 +22,38 @@
  */
 
 #include "devicespage.h"
-#include "musiclibraryitemroot.h"
-#include "musiclibraryitemartist.h"
-#include "musiclibraryitemalbum.h"
-#include "musiclibraryitemsong.h"
-#include "mainwindow.h"
-#include "devicesmodel.h"
-#include "settings.h"
-#include "messagebox.h"
-#include "localize.h"
-#include "icons.h"
-#include "action.h"
-#include "actioncollection.h"
-#include "stdactions.h"
-#ifdef ENABLE_KDE_SUPPORT
-#include <KDE/KGlobalSettings>
-#endif
+#include "models/musiclibraryitemroot.h"
+#include "models/musiclibraryitemartist.h"
+#include "models/musiclibraryitemalbum.h"
+#include "models/musiclibraryitemsong.h"
+#include "models/devicesmodel.h"
+#include "gui/settings.h"
+#include "support/messagebox.h"
+#include "support/localize.h"
+#include "widgets/icons.h"
+#include "support/action.h"
+#include "support/actioncollection.h"
+#include "gui/stdactions.h"
 #ifdef ENABLE_REMOTE_DEVICES
 #include "remotedevicepropertiesdialog.h"
 #include "devicepropertieswidget.h"
 #endif
 #include "syncdialog.h"
 #ifdef ENABLE_REPLAYGAIN_SUPPORT
-#include "rgdialog.h"
+#include "replaygain/rgdialog.h"
 #endif
-#include "tageditor.h"
+#include "tags/tageditor.h"
 #include "actiondialog.h"
-#include "trackorganiser.h"
-#include "preferencesdialog.h"
-#include "coverdialog.h"
-#include "mpdconnection.h"
+#include "tags/trackorganiser.h"
+#include "gui/preferencesdialog.h"
+#include "gui/coverdialog.h"
+#include "mpd/mpdconnection.h"
 #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
 #include "audiocddevice.h"
 #include "albumdetailsdialog.h"
 #include "cddbselectiondialog.h"
 #endif
+#include <QMenu>
 
 DevicesPage::DevicesPage(QWidget *p)
     : QWidget(p)
@@ -129,10 +126,12 @@ DevicesPage::DevicesPage(QWidget *p)
     view->setModel(&proxy);
     view->setRootIsDecorated(false);
     view->setSearchResetLevel(1);
+    view->load(metaObject()->className());
 }
 
 DevicesPage::~DevicesPage()
 {
+    view->save(metaObject()->className());
 }
 
 void DevicesPage::showEvent(QShowEvent *e)
@@ -212,13 +211,8 @@ QList<Song> DevicesPage::selectedSongs(bool allowPlaylists) const
     }
 
     // Ensure all songs are from UMS/Remote devices...
-    QString udi;
-    QModelIndexList mapped;
     foreach (const QModelIndex &idx, selected) {
-        QModelIndex index = proxy.mapToSource(idx);
-        mapped.append(index);
-        MusicLibraryItem *item=static_cast<MusicLibraryItem *>(index.internalPointer());
-
+        MusicLibraryItem *item=static_cast<MusicLibraryItem *>(proxy.mapToSource(idx).internalPointer());
         if (item && MusicLibraryItem::Type_Root!=item->itemType()) {
             while(item->parentItem()) {
                 item=item->parentItem();
@@ -233,7 +227,7 @@ QList<Song> DevicesPage::selectedSongs(bool allowPlaylists) const
         }
     }
 
-    return DevicesModel::self()->songs(mapped);
+    return DevicesModel::self()->songs(proxy.mapToSource(selected, proxy.enabled() && Settings::self()->filteredOnly()));
 }
 
 void DevicesPage::addSelectionToPlaylist(const QString &name, bool replace, quint8 priorty, bool randomAlbums)
