@@ -22,30 +22,29 @@
  */
 
 #include "umsdevice.h"
-#include "tags.h"
-#include "musiclibrarymodel.h"
-#include "musiclibraryitemsong.h"
-#include "musiclibraryitemalbum.h"
-#include "musiclibraryitemartist.h"
-#include "musiclibraryitemroot.h"
-#include "dirviewmodel.h"
+#include "tags/tags.h"
+#include "models/musiclibrarymodel.h"
+#include "models/musiclibraryitemsong.h"
+#include "models/musiclibraryitemalbum.h"
+#include "models/musiclibraryitemartist.h"
+#include "models/musiclibraryitemroot.h"
+#include "models/dirviewmodel.h"
 #include "devicepropertiesdialog.h"
 #include "devicepropertieswidget.h"
-#include "utils.h"
-#include "mpdparseutils.h"
-#include "mpdconnection.h"
+#include "support/utils.h"
+#include "mpd/mpdparseutils.h"
+#include "mpd/mpdconnection.h"
 #include "encoders.h"
 #include "transcodingjob.h"
 #include "actiondialog.h"
-#include "localize.h"
-#include "covers.h"
-#include "thread.h"
+#include "support/localize.h"
+#include "gui/covers.h"
+#include "support/thread.h"
 #include <QDir>
 #include <QFile>
 #include <QFileInfo>
 #include <QTextStream>
 #include <QTimer>
-#include <time.h>
 
 const QLatin1String FsDevice::constCantataCacheFile("/.cache");
 const QLatin1String FsDevice::constCantataSettingsFile("/.cantata");
@@ -66,8 +65,6 @@ MusicScanner::MusicScanner()
     : QObject(0)
     , stopRequested(false)
     , count(0)
-    , lastUpdate(0)
-    , lastCacheProg(-1)
 {
     thread=new Thread(metaObject()->className());
     moveToThread(thread);
@@ -81,7 +78,6 @@ MusicScanner::~MusicScanner()
 
 void MusicScanner::scan(const QString &folder, const QString &cacheFile, bool readCache, const QSet<FileOnlySong> &existingSongs)
 {
-    lastCacheProg=-1;
     if (!cacheFile.isEmpty() && readCache) {
         MusicLibraryItemRoot *lib=new MusicLibraryItemRoot;
         MusicLibraryModel::convertCache(cacheFile);
@@ -103,17 +99,17 @@ void MusicScanner::scan(const QString &folder, const QString &cacheFile, bool re
         return;
     }
     count=0;
-    lastUpdate=0;
     MusicLibraryItemRoot *library = new MusicLibraryItemRoot;
     QString topLevel=Utils::fixPath(QDir(folder).absolutePath());
     QSet<FileOnlySong> existing=existingSongs;
+    timer.start();
     scanFolder(library, topLevel, topLevel, existing, 0);
 
     if (!stopRequested) {
         library->applyGrouping();
         if (!cacheFile.isEmpty()) {
             writeProgress(0.0);
-            library->toXML(cacheFile, QDateTime(), this);
+            library->toXML(cacheFile, QDateTime(), false, this);
         }
         emit libraryUpdated(library);
     } else {
@@ -123,9 +119,8 @@ void MusicScanner::scan(const QString &folder, const QString &cacheFile, bool re
 
 void MusicScanner::saveCache(const QString &cache, MusicLibraryItemRoot *lib)
 {
-    lastCacheProg=-1;
     writeProgress(0.0);
-    lib->toXML(cache, QDateTime(), this);
+    lib->toXML(cache, QDateTime(), false, this);
     emit cacheSaved();
 }
 
@@ -174,9 +169,8 @@ void MusicScanner::scanFolder(MusicLibraryItemRoot *library, const QString &topL
                     continue;
                 }
                 count++;
-                int t=time(NULL);
-                if ((t-lastUpdate)>=2 || 0==(count%5)) {
-                    lastUpdate=t;
+                if (timer.elapsed()>=1500 || 0==(count%5)) {
+                    timer.restart();
                     emit songCount(count);
                 }
 
@@ -189,10 +183,11 @@ void MusicScanner::scanFolder(MusicLibraryItemRoot *library, const QString &topL
                     albumItem = artistItem->album(song);
                 }
                 MusicLibraryItemSong *songItem = new MusicLibraryItemSong(song, albumItem);
+                const QSet<QString> &songGenres=songItem->allGenres();
                 albumItem->append(songItem);
-                albumItem->addGenre(song.genre);
-                artistItem->addGenre(song.genre);
-                library->addGenre(song.genre);
+                albumItem->addGenres(songGenres);
+                artistItem->addGenres(songGenres);
+                library->addGenres(songGenres);
             }
         }
     }
@@ -200,22 +195,12 @@ void MusicScanner::scanFolder(MusicLibraryItemRoot *library, const QString &topL
 
 void MusicScanner::readProgress(double pc)
 {
-    int prog=(int)pc;
-
-    if (prog!=lastCacheProg) {
-        lastCacheProg=prog;
-        emit readingCache(lastCacheProg);
-    }
+    emit readingCache(pc);
 }
 
 void MusicScanner::writeProgress(double pc)
 {
-    int prog=(int)pc;
-
-    if (prog!=lastCacheProg) {
-        lastCacheProg=prog;
-        emit savingCache(lastCacheProg);
-    }
+    emit savingCache(pc);
 }
 
 bool FsDevice::readOpts(const QString &fileName, DeviceOptions &opts, bool readAll)
@@ -725,6 +710,7 @@ void FsDevice::saveCache()
 {
     if (opts.useCache) {
         state=SavingCache;
+        initScaner();
         emit saveCache(cacheFileName(), this);
     }
 }
