@@ -22,24 +22,25 @@
  */
 
 #include "mtpdevice.h"
-#include "musiclibrarymodel.h"
-#include "musiclibraryitemsong.h"
-#include "musiclibraryitemalbum.h"
-#include "musiclibraryitemartist.h"
-#include "musiclibraryitemroot.h"
-#include "dirviewmodel.h"
+#include "models/musiclibrarymodel.h"
+#include "models/musiclibraryitemsong.h"
+#include "models/musiclibraryitemalbum.h"
+#include "models/musiclibraryitemartist.h"
+#include "models/musiclibraryitemroot.h"
+#include "models/dirviewmodel.h"
 #include "devicepropertiesdialog.h"
 #include "devicepropertieswidget.h"
-#include "covers.h"
-#include "song.h"
+#include "gui/covers.h"
+#include "mpd/song.h"
 #include "encoders.h"
 #include "transcodingjob.h"
-#include "utils.h"
-#include "mpdparseutils.h"
-#include "localize.h"
+#include "support/utils.h"
+#include "mpd/mpdparseutils.h"
+#include "mpd/mpdconnection.h"
+#include "support/localize.h"
 #include "filejob.h"
-#include "settings.h"
-#include "thread.h"
+#include "support/configuration.h"
+#include "support/thread.h"
 #include <QTimer>
 #include <QDir>
 #include <QTemporaryFile>
@@ -58,7 +59,12 @@
 #include <QElapsedTimer>
 #endif
 
+// Enable the following #define to have Cantata attempt to ascertain the AlbumArtist tag by
+// looking at the file path
 #define MTP_FAKE_ALBUMARTIST_SUPPORT
+
+// Enable the following #define to have Cantata attempt to ascertain the tracks's track number
+// from its filename. This will only be done if the device returns '0' as the track number.
 #define MTP_TRACKNUMBER_FROM_FILENAME
 
 static const QLatin1String constMtpDefaultCover("AlbumArt.jpg");
@@ -69,14 +75,14 @@ static int progressMonitor(uint64_t const processed, uint64_t const total, void
 {
     const MtpConnection *con=static_cast<const MtpConnection *>(data);
     const_cast<MtpConnection *>(con)->emitProgress((int)(((processed*1.0)/(total*1.0)*100.0)+0.5));
-    return con->abortRequested() ? -1 : 0;
+    return con->abortWasRequested() ? -1 : 0;
 }
 
 static int trackListMonitor(uint64_t const processed, uint64_t const total, void const * const data)
 {
     const MtpConnection *con=static_cast<const MtpConnection *>(data);
     const_cast<MtpConnection *>(con)->trackListProgress(((processed*100.0)/(total*1.0))+0.5);
-    return con->abortRequested() ? -1 : 0;
+    return con->abortWasRequested() ? -1 : 0;
 }
 
 static uint16_t fileReceiver(void *params, void *priv, uint32_t sendlen, unsigned char *data, uint32_t *putlen)
@@ -88,14 +94,17 @@ static uint16_t fileReceiver(void *params, void *priv, uint32_t sendlen, unsigne
     return LIBMTP_HANDLER_RETURN_OK;
 }
 
-MtpConnection::MtpConnection(MtpDevice *p)
+MtpConnection::MtpConnection(unsigned int bus, unsigned int dev, bool aaSupport)
     : device(0)
     #ifdef MTP_CLEAN_ALBUMS
     , albums(0)
     #endif
     , library(0)
-    , dev(p)
     , lastListPercent(-1)
+    , abortRequested(false)
+    , busNum(bus)
+    , devNum(dev)
+    , supprtAlbumArtistTag(aaSupport)
 {
     size=0;
     used=0;
@@ -107,8 +116,7 @@ MtpConnection::MtpConnection(MtpDevice *p)
 
 MtpConnection::~MtpConnection()
 {
-    disconnectFromDevice(false);
-    thread->stop();
+    stop();
 }
 
 MusicLibraryItemRoot * MtpConnection::takeLibrary()
@@ -118,11 +126,6 @@ MusicLibraryItemRoot * MtpConnection::takeLibrary()
     return lib;
 }
 
-bool MtpConnection::abortRequested() const
-{
-    return dev->abortRequested();
-}
-
 void MtpConnection::emitProgress(int percent)
 {
     emit progress(percent);
@@ -162,8 +165,8 @@ void MtpConnection::connectToDevice()
 
     LIBMTP_mtpdevice_t *mptDev=0;
     for (int i = 0; i < numDev; i++) {
-        if (0!=dev->busNum && 0!=dev->devNum) {
-            if (rawDevices[i].bus_location==dev->busNum && rawDevices[i].devnum==dev->devNum) {
+        if (0!=busNum && 0!=devNum) {
+            if (rawDevices[i].bus_location==busNum && rawDevices[i].devnum==devNum) {
                 #ifdef ENABLE_UNCACHED_MTP
                 mptDev = LIBMTP_Open_Raw_Device_Uncached(&rawDevices[i]);
                 #else
@@ -243,7 +246,6 @@ struct MtpFolder {
     QString artist;
     QString album;
 };
-
 #endif
 
 struct Path {
@@ -280,7 +282,7 @@ static Path decodePath(const QString &path)
     return p;
 }
 
-void MtpConnection::updateLibrary()
+void MtpConnection::updateLibrary(const DeviceOptions &opts)
 {
     if (!isConnected()) {
         connectToDevice();
@@ -304,6 +306,9 @@ void MtpConnection::updateLibrary()
     emit statusMessage(i18n("Updating folders..."));
     #ifdef ENABLE_UNCACHED_MTP
     updateFilesAndFolders();
+    if (abortRequested) {
+        return;
+    }
     #else
     updateFolders();
     #endif
@@ -342,19 +347,25 @@ void MtpConnection::updateLibrary()
         setMusicFolder(*store);
         storageNames[(*store).id]=(*store).description;
     }
+    if (abortRequested) {
+        return;
+    }
 
     MusicLibraryItemArtist *artistItem = 0;
     MusicLibraryItemAlbum *albumItem = 0;
     #ifdef MTP_FAKE_ALBUMARTIST_SUPPORT
     QMap<QString, MtpAlbum> albumMap;
     QMap<uint32_t, MtpFolder> folders;
-    bool getAlbumArtistFromPath=dev->options().scheme.startsWith(DeviceOptions::constAlbumArtist+QChar('/')+DeviceOptions::constAlbumTitle+QChar('/'));
+    bool getAlbumArtistFromPath=opts.scheme.startsWith(DeviceOptions::constAlbumArtist+QChar('/')+DeviceOptions::constAlbumTitle+QChar('/'));
     #endif
     #ifdef TIME_MTP_OPERATIONS
     qWarning() << "Tracks update:" << timer.elapsed();
     timer.restart();
     #endif
-    while (tracks && !abortRequested()) {
+    while (tracks) {
+        if (abortRequested) {
+            return;
+        }
         LIBMTP_track_t *track=tracks;
         tracks=tracks->next;
 
@@ -371,7 +382,10 @@ void MtpConnection::updateLibrary()
         s.album=QString::fromUtf8(track->album);
         s.artist=QString::fromUtf8(track->artist);
         s.albumartist=s.artist; // TODO: ALBUMARTIST: Read from 'track' when libMTP supports album artist!
-        s.composer=QString::fromUtf8(track->composer);
+        QString composer=QString::fromUtf8(track->composer);
+        if (!composer.isEmpty()) {
+            s.setComposer(composer);
+        }
         s.year=QString::fromUtf8(track->date).mid(0, 4).toUInt();
         s.title=QString::fromUtf8(track->title);
         s.genre=QString::fromUtf8(track->genre);
@@ -383,7 +397,17 @@ void MtpConnection::updateLibrary()
         if (getAlbumArtistFromPath) {
             QStringList folderParts=(*folder).path.split('/', QString::SkipEmptyParts);
             if (folderParts.length()>=3) {
-                MtpFolder folder(folderParts.at(1), folderParts.at(2));
+                // Path should be "Music/${AlbumArtist}/${Album}"
+                int artistPath=1;
+                int albumPath=2;
+                // BubbleUPNP will download to "Music/Artist/${AlbumArtist}/${Album}" if it is configured to
+                // download to "Music" and "Preserve folder structure" (At least this is the folder structure
+                // from MiniDLNA...)
+                if (folderParts.length()>=4 && QLatin1String("Artist")==folderParts.at(1)) {
+                    artistPath++;
+                    albumPath++;
+                }
+                MtpFolder folder(folderParts.at(artistPath), folderParts.at(albumPath));
                 folders.insert(track->parent_id, folder);
                 if (folder.album==s.album && Song::isVariousArtists(folder.artist)) {
                     s.albumartist=folder.artist;
@@ -394,22 +418,23 @@ void MtpConnection::updateLibrary()
         #ifdef MTP_TRACKNUMBER_FROM_FILENAME
         if (0==s.track) {
             int space=trackFilename.indexOf(' ');
-            if (space>0) {
+            if (space>0 && space<=3) {
                 s.track=trackFilename.mid(0, space).toInt();
             }
         }
         #endif
-        if (!artistItem || (dev->supportsAlbumArtistTag() ? s.artistOrComposer()!=artistItem->data() : s.artist!=artistItem->data())) {
+        if (!artistItem || (supprtAlbumArtistTag ? s.artistOrComposer()!=artistItem->data() : s.artist!=artistItem->data())) {
             artistItem = library->artist(s);
         }
         if (!albumItem || albumItem->parentItem()!=artistItem || s.albumName()!=albumItem->data()) {
             albumItem = artistItem->album(s);
         }
         MusicLibraryItemSong *songItem = new MusicLibraryItemSong(s, albumItem);
+        const QSet<QString> &songGenres=songItem->allGenres();
         albumItem->append(songItem);
-        albumItem->addGenre(s.genre);
-        artistItem->addGenre(s.genre);
-        library->addGenre(s.genre);
+        albumItem->addGenres(songGenres);
+        artistItem->addGenres(songGenres);
+        library->addGenres(songGenres);
 
         #ifdef MTP_FAKE_ALBUMARTIST_SUPPORT
         // Store AlbumName->Artists/Songs mapping
@@ -431,84 +456,87 @@ void MtpConnection::updateLibrary()
     qWarning() << "Tracks parse:" << timer.elapsed();
     timer.restart();
     #endif
-    if (!abortRequested()) {
-        #ifdef MTP_FAKE_ALBUMARTIST_SUPPORT
-        // Use Album map to determine 'AlbumArtist' tag for various artist albums, and
-        // albums that have tracks where artist is set to '${artist} and somebodyelse'
-        QMap<QString, MtpAlbum>::ConstIterator it=albumMap.constBegin();
-        QMap<QString, MtpAlbum>::ConstIterator end=albumMap.constEnd();
-        for (; it!=end; ++it) {
-            if ((*it).artists.count()>1) {
-                QSet<quint16> tracks;
-                QString shortestArtist;
-                bool duplicateTrackNumbers=false;
-                foreach (MusicLibraryItemSong *s, (*it).songs) {
-                    if (tracks.contains(s->track())) {
-                        duplicateTrackNumbers=true;
+    #ifdef MTP_FAKE_ALBUMARTIST_SUPPORT
+    // Use Album map to determine 'AlbumArtist' tag for various artist albums, and
+    // albums that have tracks where artist is set to '${artist} and somebodyelse'
+    QMap<QString, MtpAlbum>::ConstIterator it=albumMap.constBegin();
+    QMap<QString, MtpAlbum>::ConstIterator end=albumMap.constEnd();
+    for (; it!=end; ++it) {
+        if (abortRequested) {
+            return;
+        }
+        if ((*it).artists.count()>1) {
+            QSet<quint16> tracks;
+            QString shortestArtist;
+            bool duplicateTrackNumbers=false;
+            foreach (MusicLibraryItemSong *s, (*it).songs) {
+                if (tracks.contains(s->track())) {
+                    duplicateTrackNumbers=true;
+                    break;
+                } else {
+                    if (shortestArtist.isEmpty() || s->song().artist.length()<shortestArtist.length()) {
+                        shortestArtist=s->song().artist;
+                    }
+                    tracks.insert(s->track());
+                }
+            }
+            // If an album has mutiple tracks with the same track number, then we probably have X albums
+            // by X artists - in which case we proceeed no further.
+            if (!duplicateTrackNumbers) {
+                MtpFolder &f=folders[(*it).folder];
+                // Now, check to see if all artists contain 'shortestArtist'. If so then use 'shortestArtist' as the album
+                // artist. This is probably due to songs which have artist set to '${artist} and somebodyelse'
+                QString albumArtist=shortestArtist;
+                foreach (const QString &artist, (*it).artists) {
+                    if (!artist.contains(shortestArtist)) {
+                        // Found an artist that did not contain 'shortestArtist', so use 'Various Artists' for album artist
+                        albumArtist=!f.artist.isEmpty() && f.album==it.key() ? f.artist : Song::variousArtists();
                         break;
-                    } else {
-                        if (shortestArtist.isEmpty() || s->song().artist.length()<shortestArtist.length()) {
-                            shortestArtist=s->song().artist;
-                        }
-                        tracks.insert(s->track());
                     }
                 }
 
-                // If an album has mutiple tracks with the same track number, then we probably have X albums
-                // by X artists - in which case we proceeed no further.
-                if (!duplicateTrackNumbers) {
-                    MtpFolder &f=folders[(*it).folder];
-                    // Now, check to see if all artists contain 'shortesArtist'. If so then use 'shortesArtist' as the album
-                    // artist. This ir probably due to songs which have artist set to '${artist} and somebodyelse'
-                    QString albumArtist=shortestArtist;
-                    foreach (const QString &artist, (*it).artists) {
-                        if (!artist.contains(shortestArtist)) {
-                            // Found an artist that did not contain 'shortestArtist', so use 'Various Artists' for album artist
-                            albumArtist=!f.artist.isEmpty() && f.album==it.key() ? f.artist : Song::variousArtists();
-                            break;
-                        }
+                // Now move songs to correct artist/album...
+                foreach (MusicLibraryItemSong *s, (*it).songs) {
+                    if (s->song().albumartist==albumArtist) {
+                        continue;
                     }
-
-                    // Now move songs to correct artist/album...
-                    foreach (MusicLibraryItemSong *s, (*it).songs) {
-                        if (s->song().albumartist==albumArtist) {
-                            continue;
-                        }
-                        Song song=s->song();
-                        song.albumartist=albumArtist;
-                        artistItem=library->artist(song);
-                        albumItem=artistItem->album(song);
-                        MusicLibraryItemSong *songItem = new MusicLibraryItemSong(song, albumItem);
-                        albumItem->append(songItem);
-                        albumItem->updateGenres();
-                        artistItem->updateGenres();
-                        MusicLibraryItemAlbum *prevAlbum=(MusicLibraryItemAlbum *)s->parentItem();
-                        prevAlbum->remove(s);
-                        if (0==prevAlbum->childCount()) {
-                            // Album no longer has any songs, so remove!
-                            MusicLibraryItemArtist *prevArtist=(MusicLibraryItemArtist *)prevAlbum->parentItem();
-                            prevArtist->remove(prevAlbum);
-                            if (0==prevArtist->childCount()) {
-                                // Artist no longer has any albums, so remove!
-                                library->remove(prevArtist);
-                            }
+                    Song song=s->song();
+                    song.albumartist=albumArtist;
+                    artistItem=library->artist(song);
+                    albumItem=artistItem->album(song);
+                    MusicLibraryItemSong *songItem = new MusicLibraryItemSong(song, albumItem);
+                    albumItem->append(songItem);
+                    albumItem->updateGenres();
+                    artistItem->updateGenres();
+                    MusicLibraryItemAlbum *prevAlbum=(MusicLibraryItemAlbum *)s->parentItem();
+                    prevAlbum->remove(s);
+                    if (0==prevAlbum->childCount()) {
+                        // Album no longer has any songs, so remove!
+                        MusicLibraryItemArtist *prevArtist=(MusicLibraryItemArtist *)prevAlbum->parentItem();
+                        prevArtist->remove(prevAlbum);
+                        if (0==prevArtist->childCount()) {
+                            // Artist no longer has any albums, so remove!
+                            library->remove(prevArtist);
                         }
                     }
                 }
             }
         }
-        #ifdef TIME_MTP_OPERATIONS
-        qWarning() << "AlbumArtist detection:" << timer.elapsed();
-        timer.restart();
-        #endif
-        #endif
-        library->applyGrouping();
-        #ifdef TIME_MTP_OPERATIONS
-        qWarning() << "Grouping:" << timer.elapsed();
-        qWarning() << "TOTAL update:" <<totalTimer.elapsed();
-        #endif
-        emit libraryUpdated();
     }
+    #ifdef TIME_MTP_OPERATIONS
+    qWarning() << "AlbumArtist detection:" << timer.elapsed();
+    timer.restart();
+    #endif
+    #endif
+    if (abortRequested) {
+        return;
+    }
+    library->applyGrouping();
+    #ifdef TIME_MTP_OPERATIONS
+    qWarning() << "Grouping:" << timer.elapsed();
+    qWarning() << "TOTAL update:" <<totalTimer.elapsed();
+    #endif
+    emit libraryUpdated();
 }
 
 void MtpConnection::setMusicFolder(Storage &store)
@@ -529,6 +557,9 @@ void MtpConnection::updateFilesAndFolders()
 {
     folderMap.clear();
     foreach (const Storage st, storage) {
+        if (abortRequested) {
+            return;
+        }
         listFolder(st.id, constRootFolder, 0);
     }
 }
@@ -537,7 +568,7 @@ void MtpConnection::listFolder(uint32_t storage, uint32_t parentDir, Folder *f)
 {
     LIBMTP_file_t *files=LIBMTP_Get_Files_And_Folders(device, storage, parentDir);
 
-    while (files) {
+    while (files && !abortRequested) {
         LIBMTP_file_t *file=files;
         files = files->next;
 
@@ -913,7 +944,7 @@ void MtpConnection::putSong(const Song &s, bool fixVa, const DeviceOptions &opts
 
         meta->parent_id=folderId=store.musicFolderId;
         meta->storage_id=store.id;
-        destName=store.musicPath+dev->opts.createFilename(song);
+        destName=store.musicPath+opts.createFilename(song);
         QStringList dirs=destName.split('/', QString::SkipEmptyParts);
         if (dirs.count()>1) {
             destName=dirs.takeLast();
@@ -934,7 +965,7 @@ void MtpConnection::putSong(const Song &s, bool fixVa, const DeviceOptions &opts
         if (status!=Device::SongExists) {
             meta->title=createString(song.title);
             meta->artist=createString(song.artist);
-            meta->composer=createString(song.composer);
+            meta->composer=createString(song.composer());
             meta->genre=createString(song.genre);
             meta->album=createString(song.album);
             meta->date=createString(QString().sprintf("%4d0101T0000.0", song.year));
@@ -1051,7 +1082,7 @@ void MtpConnection::getSong(const Song &song, const QString &dest, bool fixVa, b
     bool copiedSong=device && 0==LIBMTP_Get_File_To_File(device, song.id, dest.toUtf8(), &progressMonitor, this);
     bool copiedCover=false;
 
-    if (copiedSong && !abortRequested() && copyCover) {
+    if (copiedSong && !abortRequested && copyCover) {
         QString destDir=Utils::getDir(dest);
 
         if (QDir(destDir).entryList(QStringList() << QLatin1String("*.jpg") << QLatin1String("*.png"), QDir::Files|QDir::Readable).isEmpty()) {
@@ -1072,7 +1103,7 @@ void MtpConnection::getSong(const Song &song, const QString &dest, bool fixVa, b
         }
     }
 
-    if (copiedSong && fixVa && !abortRequested()) {
+    if (copiedSong && fixVa && !abortRequested) {
         Song s(song);
         Device::fixVariousArtists(dest, s, false);
     }
@@ -1188,6 +1219,15 @@ void MtpConnection::getCover(const Song &song)
     }
 }
 
+void MtpConnection::stop()
+{
+    if (thread) {
+        disconnectFromDevice(false);
+        thread->stop();
+        thread=0;
+    }
+}
+
 #ifdef MTP_CLEAN_ALBUMS
 void MtpConnection::updateAlbums()
 {
@@ -1263,8 +1303,6 @@ MtpDevice::MtpDevice(MusicModel *m, Solid::Device &dev)
     , pmp(dev.as<Solid::PortableMediaPlayer>())
     , tempFile(0)
     , mtpUpdating(false)
-    , busNum(0)
-    , devNum(0)
 {
     static bool registeredTypes=false;
     if (!registeredTypes) {
@@ -1274,14 +1312,16 @@ MtpDevice::MtpDevice(MusicModel *m, Solid::Device &dev)
     }
 
     Solid::GenericInterface *iface = dev.as<Solid::GenericInterface>();
+    unsigned int busNum(0);
+    unsigned int devNum(0);
     if (iface) {
         QMap<QString, QVariant> properties = iface->allProperties();
         busNum = properties.value(QLatin1String("BUSNUM")).toInt();
         devNum = properties.value(QLatin1String("DEVNUM")).toInt();
     }
 
-    connection=new MtpConnection(this);
-    connect(this, SIGNAL(updateLibrary()), connection, SLOT(updateLibrary()));
+    connection=new MtpConnection(busNum, devNum, supportsAlbumArtistTag());
+    connect(this, SIGNAL(updateLibrary(const DeviceOptions &)), connection, SLOT(updateLibrary(const DeviceOptions &)));
     connect(connection, SIGNAL(libraryUpdated()), this, SLOT(libraryUpdated()));
     connect(connection, SIGNAL(progress(int)), this, SLOT(emitProgress(int)));
     connect(this, SIGNAL(putSong(const Song &, bool, const DeviceOptions &, bool, bool)), connection, SLOT(putSong(const Song &, bool, const DeviceOptions &, bool, bool)));
@@ -1300,6 +1340,10 @@ MtpDevice::MtpDevice(MusicModel *m, Solid::Device &dev)
     opts.fixVariousArtists=false;
     opts.coverName=constMtpDefaultCover;
     QTimer::singleShot(0, this, SLOT(rescan(bool)));
+    defaultName=data();
+    if (!opts.name.isEmpty()) {
+        setData(opts.name);
+    }
 }
 
 MtpDevice::~MtpDevice()
@@ -1313,10 +1357,11 @@ void MtpDevice::deviceDetails(const QString &s)
         serial=s;
         QString configKey=cfgKey(solidDev, serial);
         opts.load(configKey);
-        #ifndef ENABLE_KDE_SUPPORT
-        QSettings cfg;
-        #endif
-        configured=HAS_GROUP(configKey);
+        configured=Configuration().hasGroup(configKey);
+        if (!opts.name.isEmpty() && opts.name!=defaultName) {
+            setData(opts.name);
+            emit renamed();
+        }
     }
 }
 
@@ -1327,10 +1372,23 @@ bool MtpDevice::isConnected() const
 
 void MtpDevice::stop()
 {
-    jobAbortRequested=true;
+    abortJob();
     deleteTemp();
-    connection->deleteLater();
-    connection=0;
+    if (connection) {
+        disconnect(connection, SIGNAL(libraryUpdated()), this, SLOT(libraryUpdated()));
+        disconnect(connection, SIGNAL(progress(int)), this, SLOT(emitProgress(int)));
+        disconnect(connection, SIGNAL(putSongStatus(int, const QString &, bool, bool)), this, SLOT(putSongStatus(int, const QString &, bool, bool)));
+        disconnect(connection, SIGNAL(getSongStatus(bool, bool)), this, SLOT(getSongStatus(bool, bool)));
+        disconnect(connection, SIGNAL(delSongStatus(bool)), this, SLOT(delSongStatus(bool)));
+        disconnect(connection, SIGNAL(cleanDirsStatus(bool)), this, SLOT(cleanDirsStatus(bool)));
+        disconnect(connection, SIGNAL(statusMessage(const QString &)), this, SLOT(setStatusMessage(const QString &)));
+        disconnect(connection, SIGNAL(deviceDetails(const QString &)), this, SLOT(deviceDetails(const QString &)));
+        disconnect(connection, SIGNAL(updatePercentage(int)), this, SLOT(updatePercentage(int)));
+        disconnect(connection, SIGNAL(cover(const Song &, const QImage &)), this, SIGNAL(cover(const Song &, const QImage &)));
+        metaObject()->invokeMethod(connection, "stop", Qt::QueuedConnection);
+        connection->deleteLater();
+        connection=0;
+    }
 }
 
 void MtpDevice::configure(QWidget *parent)
@@ -1344,7 +1402,11 @@ void MtpDevice::configure(QWidget *parent)
     if (!configured) {
         connect(dlg, SIGNAL(cancelled()), SLOT(saveProperties()));
     }
-    dlg->show(QString(), opts, connection->getStorageList(), DevicePropertiesWidget::Prop_CoversAll|DevicePropertiesWidget::Prop_Va|DevicePropertiesWidget::Prop_Transcoder);
+    DeviceOptions o=opts;
+    if (o.name.isEmpty()) {
+        o.name=data();
+    }
+    dlg->show(QString(), o, connection->getStorageList(), DevicePropertiesWidget::Prop_Name|DevicePropertiesWidget::Prop_CoversAll|DevicePropertiesWidget::Prop_Va|DevicePropertiesWidget::Prop_Transcoder);
 }
 
 void MtpDevice::rescan(bool full)
@@ -1359,12 +1421,12 @@ void MtpDevice::rescan(bool full)
     }
     mtpUpdating=true;
     emit updating(solidDev.udi(), true);
-    emit updateLibrary();
+    emit updateLibrary(opts);
 }
 
 void MtpDevice::addSong(const Song &s, bool overwrite, bool copyCover)
 {
-    jobAbortRequested=false;
+    requestAbort(false);
     if (!isConnected()) {
         emit actionStatus(NotConnected);
         return;
@@ -1398,7 +1460,7 @@ void MtpDevice::addSong(const Song &s, bool overwrite, bool copyCover)
 
         if (!opts.transcoderWhenDifferent || encoder.isDifferent(s.file)) {
             deleteTemp();
-            tempFile=new QTemporaryFile(QDir::tempPath()+"/cantata_XXXXXX"+encoder.extension);
+            tempFile=new QTemporaryFile(QDir::tempPath()+"/cantata_XXXXXX."+encoder.extension);
             tempFile->setAutoRemove(false);
 
             if (!tempFile->open()) {
@@ -1428,7 +1490,7 @@ void MtpDevice::addSong(const Song &s, bool overwrite, bool copyCover)
 
 void MtpDevice::copySongTo(const Song &s, const QString &musicPath, bool overwrite, bool copyCover)
 {
-    jobAbortRequested=false;
+    requestAbort(false);
     transcoding=false;
     if (!isConnected()) {
         emit actionStatus(NotConnected);
@@ -1468,7 +1530,7 @@ void MtpDevice::copySongTo(const Song &s, const QString &musicPath, bool overwri
 
 void MtpDevice::removeSong(const Song &s)
 {
-    jobAbortRequested=false;
+    requestAbort(false);
     if (!isConnected()) {
         emit actionStatus(NotConnected);
         return;
@@ -1485,7 +1547,7 @@ void MtpDevice::removeSong(const Song &s)
 
 void MtpDevice::cleanDirs(const QSet<QString> &dirs)
 {
-    jobAbortRequested=false;
+    requestAbort(false);
     if (!isConnected()) {
         emit actionStatus(NotConnected);
         return;
@@ -1495,7 +1557,7 @@ void MtpDevice::cleanDirs(const QSet<QString> &dirs)
 
 void MtpDevice::requestCover(const Song &song)
 {
-    jobAbortRequested=false;
+    requestAbort(false);
     if (isConnected()) {
         emit getCover(song);
     }
@@ -1648,8 +1710,17 @@ void MtpDevice::saveProperties(const QString &, const DeviceOptions &newOpts)
     if (configured && opts==newOpts) {
         return;
     }
+
+    QString newName=newOpts.name.isEmpty() ? defaultName : newOpts.name;
+    bool diffName=opts.name!=newName;
     opts=newOpts;
+    if (diffName) {
+        setData(newName);
+    }
     saveProperties();
+    if (diffName) {
+        emit renamed();
+    }
 }
 
 void MtpDevice::saveProperties()
