@@ -23,13 +23,14 @@
 
 #include "interfacesettings.h"
 #include "settings.h"
-#include "itemview.h"
-#include "localize.h"
-#include "musiclibraryitemalbum.h"
-#include "fancytabwidget.h"
-#include "basicitemdelegate.h"
-#include "playqueueview.h"
-#include "pathrequester.h"
+#include "widgets/itemview.h"
+#include "support/localize.h"
+#include "models/musiclibraryitemalbum.h"
+#include "models/albumsmodel.h"
+#include "support/fancytabwidget.h"
+#include "widgets/basicitemdelegate.h"
+#include "widgets/playqueueview.h"
+#include "support/pathrequester.h"
 #include <QComboBox>
 #ifndef ENABLE_KDE_SUPPORT
 #include <QDir>
@@ -37,12 +38,29 @@
 #include <QRegExp>
 #include <QSet>
 #endif
+#ifdef QT_QTDBUS_FOUND
+#include <QDBusConnection>
+#include <QDBusConnectionInterface>
+#endif
+#ifndef ENABLE_KDE_SUPPORT
+#include <QSystemTrayIcon>
+#endif
 
 #define REMOVE(w) \
     w->setVisible(false); \
     w->deleteLater(); \
     w=0;
 
+static void addAlbumSorts(QComboBox *box)
+{
+    box->addItem(i18n("Album, Artist, Year"), AlbumsModel::Sort_AlbumArtist);
+    box->addItem(i18n("Album, Year, Artist"), AlbumsModel::Sort_AlbumYear);
+    box->addItem(i18n("Artist, Album, Year"), AlbumsModel::Sort_ArtistAlbum);
+    box->addItem(i18n("Artist, Year, Album"), AlbumsModel::Sort_ArtistYear);
+    box->addItem(i18n("Year, Album, Artist"), AlbumsModel::Sort_YearAlbum);
+    box->addItem(i18n("Year, Artist, Album"), AlbumsModel::Sort_YearArtist);
+}
+
 static void addImageSizes(QComboBox *box)
 {
     box->addItem(i18n("None"), MusicLibraryItemAlbum::CoverNone);
@@ -52,20 +70,24 @@ static void addImageSizes(QComboBox *box)
     box->addItem(i18n("Extra Large"), MusicLibraryItemAlbum::CoverExtraLarge);
 }
 
-static void addViewTypes(QComboBox *box, bool iconMode=false, bool groupedTree=false, bool table=false)
+static QString viewTypeString(ItemView::Mode mode)
 {
-    box->addItem(i18n("Basic Tree (No Icons)"), ItemView::Mode_BasicTree);
-    box->addItem(i18n("Simple Tree"), ItemView::Mode_SimpleTree);
-    box->addItem(i18n("Detailed Tree"), ItemView::Mode_DetailedTree);
-    if (groupedTree) {
-        box->addItem(i18n("Grouped Albums"), ItemView::Mode_GroupedTree);
+    switch (mode) {
+    default:
+    case ItemView::Mode_BasicTree:    return i18n("Basic Tree (No Icons)");
+    case ItemView::Mode_SimpleTree:   return i18n("Simple Tree");
+    case ItemView::Mode_DetailedTree: return i18n("Detailed Tree");
+    case ItemView::Mode_GroupedTree:  return i18n("Grouped Albums");
+    case ItemView::Mode_List:         return i18n("List");
+    case ItemView::Mode_IconTop:      return i18n("Icon/List");
+    case ItemView::Mode_Table:        return i18n("Table");
     }
-    box->addItem(i18n("List"), ItemView::Mode_List);
-    if (iconMode) {
-        box->addItem(i18n("Icon/List"), ItemView::Mode_IconTop);
-    }
-    if (table) {
-        box->addItem(i18n("Table"), ItemView::Mode_Table);
+}
+
+static void addViewTypes(QComboBox *box, QList<ItemView::Mode> modes)
+{
+    foreach (ItemView::Mode m, modes) {
+        box->addItem(viewTypeString(m), m);
     }
 }
 
@@ -78,13 +100,13 @@ static void selectEntry(QComboBox *box, int v)
         }
     }
 }
+bool enableNotifications(true);
 
 static inline int getValue(QComboBox *box)
 {
     return box->itemData(box->currentIndex()).toInt();
 }
 
-static const int constPlayQueueGrouped=0;
 static const char * constValueProperty="value";
 
 InterfaceSettings::InterfaceSettings(QWidget *p)
@@ -93,15 +115,37 @@ InterfaceSettings::InterfaceSettings(QWidget *p)
     , loadedLangs(false)
     #endif
 {
+    #ifdef QT_QTDBUS_FOUND
+    // We have dbus, check that org.freedesktop.Notifications exists
+    bool enableNotifications=QDBusConnection::sessionBus().interface()->isServiceRegistered("org.freedesktop.Notifications");
+    #else
+    bool enableNotifications=true;
+    #endif
+    bool enableTrayItem=true;
+    #ifndef ENABLE_KDE_SUPPORT
+    if (!QSystemTrayIcon::isSystemTrayAvailable()) {
+        enableTrayItem=false;
+        #ifndef QT_QTDBUS_FOUND
+        enableNotifications=false;
+        #endif
+    }
+    #endif
+
     setupUi(this);
     addImageSizes(libraryCoverSize);
-    addViewTypes(libraryView, true);
-    addImageSizes(albumsCoverSize);
-    addViewTypes(albumsView, true);
-    addViewTypes(folderView);
-    addViewTypes(playlistsView, false, true, true);
+    QList<ItemView::Mode> standardViews=QList<ItemView::Mode>() << ItemView::Mode_BasicTree << ItemView::Mode_SimpleTree
+                                                                << ItemView::Mode_DetailedTree << ItemView::Mode_List;
+    addViewTypes(libraryView, QList<ItemView::Mode>() << standardViews << ItemView::Mode_IconTop);
+    addViewTypes(albumsView, QList<ItemView::Mode>() << standardViews << ItemView::Mode_IconTop);
+    addViewTypes(folderView, standardViews);
+    addViewTypes(playlistsView, QList<ItemView::Mode>() << standardViews << ItemView::Mode_GroupedTree << ItemView::Mode_Table);
+    addViewTypes(searchView, QList<ItemView::Mode>() << ItemView::Mode_List << ItemView::Mode_Table);
+    addViewTypes(playQueueView, QList<ItemView::Mode>() << ItemView::Mode_GroupedTree << ItemView::Mode_Table);
 
-    addView(i18n("Play queue"), QLatin1String("PlayQueuePage"));
+    addImageSizes(albumsCoverSize);
+    addAlbumSorts(albumSort);
+
+    addView(i18n("Play Queue"), QLatin1String("PlayQueuePage"));
     addView(i18n("Artists"), QLatin1String("LibraryPage"));
     addView(i18n("Albums"), QLatin1String("AlbumsPage"));
     addView(i18n("Folders"), QLatin1String("FolderPage"));
@@ -110,21 +154,21 @@ InterfaceSettings::InterfaceSettings(QWidget *p)
     addView(i18n("Dynamic Playlists"), QLatin1String("DynamicPage"));
     #endif
     #ifdef ENABLE_STREAMS
-    addViewTypes(streamsView);
+    addViewTypes(streamsView, standardViews);
     addView(i18n("Streams (e.g. Radio Stations)"), QLatin1String("StreamsPage"));
     #else
     REMOVE(streamsView)
     REMOVE(streamsViewLabel)
     #endif
     #ifdef ENABLE_ONLINE_SERVICES
-    addViewTypes(onlineView);
+    addViewTypes(onlineView, standardViews);
     addView(i18n("Online Services - Jamendo, Maganatune, SoundCloud, and Podcasts"), QLatin1String("OnlineServicesPage"));
     #else
     REMOVE(onlineView)
     REMOVE(onlineViewLabel)
     #endif
     #ifdef ENABLE_DEVICES_SUPPORT
-    addViewTypes(devicesView);
+    addViewTypes(devicesView, standardViews);
     addView(i18n("Devices - UMS, MTP (e.g. Android), and AudioCDs"), QLatin1String("DevicesPage"));
     #else
     REMOVE(devicesView)
@@ -134,21 +178,14 @@ InterfaceSettings::InterfaceSettings(QWidget *p)
     addView(i18n("Search (via MPD)"), QLatin1String("SearchPage"));
     addView(i18n("Info - Current song information (artist, album, and lyrics)"), QLatin1String("ContextPage"));
 
-    #if !defined ENABLE_STREAMS && !defined ENABLE_ONLINE_SERVICES && !defined ENABLE_DEVICES_SUPPORT
-    tabWidget->setTabText(4, i18n("Folders"));
-    folderViewLabel->setText(i18n("Style:"));
-    #endif
-
     connect(libraryView, SIGNAL(currentIndexChanged(int)), SLOT(libraryViewChanged()));
     connect(libraryCoverSize, SIGNAL(currentIndexChanged(int)), SLOT(libraryCoverSizeChanged()));
     connect(albumsView, SIGNAL(currentIndexChanged(int)), SLOT(albumsViewChanged()));
     connect(albumsCoverSize, SIGNAL(currentIndexChanged(int)), SLOT(albumsCoverSizeChanged()));
     connect(playlistsView, SIGNAL(currentIndexChanged(int)), SLOT(playlistsViewChanged()));
-    connect(playQueueGrouped, SIGNAL(currentIndexChanged(int)), SLOT(playQueueGroupedChanged()));
-    connect(systemTrayCheckBox, SIGNAL(toggled(bool)), minimiseOnClose, SLOT(setEnabled(bool)));
-    connect(systemTrayCheckBox, SIGNAL(toggled(bool)), SLOT(enableStartupState()));
-    connect(minimiseOnClose, SIGNAL(toggled(bool)), SLOT(enableStartupState()));
+    connect(playQueueView, SIGNAL(currentIndexChanged(int)), SLOT(playQueueViewChanged()));
     connect(forceSingleClick, SIGNAL(toggled(bool)), SLOT(forceSingleClickChanged()));
+    connect(touchFriendly, SIGNAL(toggled(bool)), SLOT(touchFriendlyChanged()));
     connect(views, SIGNAL(itemChanged(QListWidgetItem*)), SLOT(viewItemChanged(QListWidgetItem*)));
 
     #ifdef ENABLE_KDE_SUPPORT
@@ -182,6 +219,29 @@ InterfaceSettings::InterfaceSettings(QWidget *p)
     connect(playQueueBackground_none, SIGNAL(toggled(bool)), SLOT(enablePlayQueueBackgroundOptions()));
     connect(playQueueBackground_cover, SIGNAL(toggled(bool)), SLOT(enablePlayQueueBackgroundOptions()));
     connect(playQueueBackground_custom, SIGNAL(toggled(bool)), SLOT(enablePlayQueueBackgroundOptions()));
+
+    if (!enableNotifications) {
+        REMOVE(systemTrayPopup)
+    }
+    if (enableTrayItem) {
+        connect(systemTrayCheckBox, SIGNAL(toggled(bool)), minimiseOnClose, SLOT(setEnabled(bool)));
+        connect(systemTrayCheckBox, SIGNAL(toggled(bool)), SLOT(enableStartupState()));
+        connect(minimiseOnClose, SIGNAL(toggled(bool)), SLOT(enableStartupState()));
+    } else {
+        REMOVE(systemTrayCheckBox)
+        REMOVE(minimiseOnClose)
+        REMOVE(startupState)
+    }
+
+    if (!enableNotifications && !enableTrayItem) {
+        tabWidget->removeTab(6);
+    }
+    #if defined Q_OS_WIN || defined Q_OS_MAC || !defined QT_QTDBUS_FOUND
+    if (systemTrayPopup && systemTrayCheckBox) {
+        connect(systemTrayCheckBox, SIGNAL(toggled(bool)), SLOT(systemTrayCheckBoxToggled()));
+        connect(systemTrayPopup, SIGNAL(toggled(bool)), SLOT(systemTrayPopupToggled()));
+    }
+    #endif
 }
 
 void InterfaceSettings::load()
@@ -192,7 +252,7 @@ void InterfaceSettings::load()
     libraryYear->setChecked(Settings::self()->libraryYear());
     selectEntry(albumsView, Settings::self()->albumsView());
     albumsCoverSize->setCurrentIndex(Settings::self()->albumsCoverSize());
-    albumSort->setCurrentIndex(Settings::self()->albumSort());
+    selectEntry(albumSort, Settings::self()->albumSort());
     selectEntry(folderView, Settings::self()->folderView());
     selectEntry(playlistsView, Settings::self()->playlistsView());
     playListsStartClosed->setChecked(Settings::self()->playListsStartClosed());
@@ -204,12 +264,14 @@ void InterfaceSettings::load()
     #endif
     groupSingle->setChecked(Settings::self()->groupSingle());
     useComposer->setChecked(Settings::self()->useComposer());
-    groupMultiple->setChecked(Settings::self()->groupMultiple());
+    filteredOnly->setChecked(Settings::self()->filteredOnly());
     #ifdef ENABLE_DEVICES_SUPPORT
     showDeleteAction->setChecked(Settings::self()->showDeleteAction());
     selectEntry(devicesView, Settings::self()->devicesView());
     #endif
-    playQueueGrouped->setCurrentIndex(Settings::self()->playQueueGrouped() ? 0 : 1);
+    selectEntry(searchView, Settings::self()->searchView());
+
+    selectEntry(playQueueView, Settings::self()->playQueueView());
     playQueueAutoExpand->setChecked(Settings::self()->playQueueAutoExpand());
     playQueueStartClosed->setChecked(Settings::self()->playQueueStartClosed());
     playQueueScroll->setChecked(Settings::self()->playQueueScroll());
@@ -226,19 +288,31 @@ void InterfaceSettings::load()
     albumsViewChanged();
     albumsCoverSizeChanged();
     playlistsViewChanged();
-    playQueueGroupedChanged();
+    playQueueViewChanged();
     forceSingleClick->setChecked(Settings::self()->forceSingleClick());
-    systemTrayCheckBox->setChecked(Settings::self()->useSystemTray());
-    systemTrayPopup->setChecked(Settings::self()->showPopups());
-    minimiseOnClose->setChecked(Settings::self()->minimiseOnClose());
-    minimiseOnClose->setEnabled(systemTrayCheckBox->isChecked());
-    switch (Settings::self()->startupState()) {
-    case Settings::SS_ShowMainWindow: startupStateShow->setChecked(true); break;
-    case Settings::SS_HideMainWindow: startupStateHide->setChecked(true); break;
-    case Settings::SS_Previous: startupStateRestore->setChecked(true); break;
+    touchFriendly->setChecked(Settings::self()->touchFriendly());
+    showStopButton->setChecked(Settings::self()->showStopButton());
+    showCoverWidget->setChecked(Settings::self()->showCoverWidget());
+    if (systemTrayCheckBox) {
+        systemTrayCheckBox->setChecked(Settings::self()->useSystemTray());
+        if (minimiseOnClose) {
+            minimiseOnClose->setChecked(Settings::self()->minimiseOnClose());
+            minimiseOnClose->setEnabled(systemTrayCheckBox->isChecked());
+        }
+        if (startupState) {
+            switch (Settings::self()->startupState()) {
+            case Settings::SS_ShowMainWindow: startupStateShow->setChecked(true); break;
+            case Settings::SS_HideMainWindow: startupStateHide->setChecked(true); break;
+            case Settings::SS_Previous: startupStateRestore->setChecked(true); break;
+            }
+
+            enableStartupState();
+        }
+    }
+    if (systemTrayPopup) {
+        systemTrayPopup->setChecked(Settings::self()->showPopups());
     }
 
-    enableStartupState();
     cacheScaledCovers->setChecked(Settings::self()->cacheScaledCovers());
     fetchCovers->setChecked(Settings::self()->fetchCovers());
 
@@ -269,7 +343,7 @@ void InterfaceSettings::save()
     Settings::self()->saveLibraryYear(libraryYear->isChecked());
     Settings::self()->saveAlbumsView(getValue(albumsView));
     Settings::self()->saveAlbumsCoverSize(albumsCoverSize->currentIndex());
-    Settings::self()->saveAlbumSort(albumSort->currentIndex());
+    Settings::self()->saveAlbumSort(getValue(albumSort));
     Settings::self()->saveFolderView(getValue(folderView));
     Settings::self()->savePlaylistsView(getValue(playlistsView));
     Settings::self()->savePlayListsStartClosed(playListsStartClosed->isChecked());
@@ -281,12 +355,13 @@ void InterfaceSettings::save()
     #endif
     Settings::self()->saveGroupSingle(groupSingle->isChecked());
     Settings::self()->saveUseComposer(useComposer->isChecked());
-    Settings::self()->saveGroupMultiple(groupMultiple->isChecked());
+    Settings::self()->saveFilteredOnly(filteredOnly->isChecked());
     #ifdef ENABLE_DEVICES_SUPPORT
     Settings::self()->saveShowDeleteAction(showDeleteAction->isChecked());
     Settings::self()->saveDevicesView(getValue(devicesView));
     #endif
-    Settings::self()->savePlayQueueGrouped(constPlayQueueGrouped==playQueueGrouped->currentIndex());
+    Settings::self()->saveSearchView(getValue(searchView));
+    Settings::self()->savePlayQueueView(getValue(playQueueView));
     Settings::self()->savePlayQueueAutoExpand(playQueueAutoExpand->isChecked());
     Settings::self()->savePlayQueueStartClosed(playQueueStartClosed->isChecked());
     Settings::self()->savePlayQueueScroll(playQueueScroll->isChecked());
@@ -304,10 +379,13 @@ void InterfaceSettings::save()
 
     Settings::self()->savePlayQueueConfirmClear(playQueueConfirmClear->isChecked());
     Settings::self()->saveForceSingleClick(forceSingleClick->isChecked());
-    Settings::self()->saveUseSystemTray(systemTrayCheckBox->isChecked());
-    Settings::self()->saveShowPopups(systemTrayPopup->isChecked());
-    Settings::self()->saveMinimiseOnClose(minimiseOnClose->isChecked());
-    if (startupStateShow->isChecked()) {
+    Settings::self()->saveTouchFriendly(touchFriendly->isChecked());
+    Settings::self()->saveShowStopButton(showStopButton->isChecked());
+    Settings::self()->saveShowCoverWidget(showCoverWidget->isChecked());
+    Settings::self()->saveUseSystemTray(systemTrayCheckBox && systemTrayCheckBox->isChecked());
+    Settings::self()->saveShowPopups(systemTrayPopup && systemTrayPopup->isChecked());
+    Settings::self()->saveMinimiseOnClose(minimiseOnClose && minimiseOnClose->isChecked());
+    if (!startupState || startupStateShow->isChecked()) {
         Settings::self()->saveStartupState(Settings::SS_ShowMainWindow);
     } else if (startupStateHide->isChecked()) {
         Settings::self()->saveStartupState(Settings::SS_HideMainWindow);
@@ -370,7 +448,7 @@ void InterfaceSettings::showEvent(QShowEvent *e)
         transCodes+=translationCodes(qApp->applicationDirPath()+QLatin1String("/translations"));
         transCodes+=translationCodes(QDir::currentPath()+QLatin1String("/translations"));
         #ifndef Q_OS_WIN
-        transCodes+=translationCodes(INSTALL_PREFIX"/share/cantata/translations/");
+        transCodes+=translationCodes(CANTATA_SYS_TRANS_DIR);
         #endif
 
         foreach (const QString &code, transCodes) {
@@ -465,15 +543,16 @@ void InterfaceSettings::albumsCoverSizeChanged()
     }
 }
 
-void InterfaceSettings::playQueueGroupedChanged()
+void InterfaceSettings::playQueueViewChanged()
 {
-    playQueueAutoExpand->setEnabled(constPlayQueueGrouped==playQueueGrouped->currentIndex());
-    playQueueStartClosed->setEnabled(constPlayQueueGrouped==playQueueGrouped->currentIndex());
+    bool grouped=ItemView::Mode_GroupedTree==getValue(playQueueView);
+    playQueueAutoExpand->setEnabled(grouped);
+    playQueueStartClosed->setEnabled(grouped);
 }
 
 void InterfaceSettings::playlistsViewChanged()
 {
-    bool grouped=getValue(playlistsView)==ItemView::Mode_GroupedTree;
+    bool grouped=ItemView::Mode_GroupedTree==getValue(playlistsView);
     playListsStartClosed->setEnabled(grouped);
 }
 
@@ -482,9 +561,16 @@ void InterfaceSettings::forceSingleClickChanged()
     singleClickLabel->setOn(forceSingleClick->isChecked()!=Settings::self()->forceSingleClick());
 }
 
+void InterfaceSettings::touchFriendlyChanged()
+{
+    touchFriendlyNoteLabel->setOn(touchFriendly->isChecked()!=Settings::self()->touchFriendly());
+}
+
 void InterfaceSettings::enableStartupState()
 {
-    startupState->setEnabled(systemTrayCheckBox->isChecked() && minimiseOnClose->isChecked());
+    if (systemTrayCheckBox && minimiseOnClose && startupState) {
+        startupState->setEnabled(systemTrayCheckBox->isChecked() && minimiseOnClose->isChecked());
+    }
 }
 
 void InterfaceSettings::langChanged()
@@ -537,3 +623,17 @@ void InterfaceSettings::enablePlayQueueBackgroundOptions()
     playQueueBackgroundBlur->setEnabled(playQueueBackgroundOpacity->isEnabled());
     playQueueBackgroundBlurLabel->setEnabled(playQueueBackgroundOpacity->isEnabled());
 }
+
+void InterfaceSettings::systemTrayCheckBoxToggled()
+{
+    if (systemTrayCheckBox && systemTrayPopup && !systemTrayCheckBox->isChecked()) {
+        systemTrayPopup->setChecked(false);
+    }
+}
+
+void InterfaceSettings::systemTrayPopupToggled()
+{
+    if (systemTrayCheckBox && systemTrayPopup && systemTrayPopup->isChecked()) {
+        systemTrayCheckBox->setChecked(true);
+    }
+}
