@@ -22,18 +22,20 @@
  */
 
 #include "playlistspage.h"
-#include "playlistsmodel.h"
-#include "mpdconnection.h"
-#include "messagebox.h"
-#include "inputdialog.h"
-#include "localize.h"
-#include "icons.h"
+#include "models/playlistsmodel.h"
+#include "mpd/mpdconnection.h"
+#include "support/messagebox.h"
+#include "support/inputdialog.h"
+#include "support/localize.h"
+#include "widgets/icons.h"
 #include "stdactions.h"
-#include "actioncollection.h"
-#include "mpdconnection.h"
-#include "tableview.h"
+#include "support/actioncollection.h"
+#include "widgets/tableview.h"
+#include "settings.h"
 #include <QMenu>
-#ifndef ENABLE_KDE_SUPPORT
+#ifdef ENABLE_KDE_SUPPORT
+#include <KDE/KGlobalSettings>
+#else
 #include <QStyle>
 #endif
 
@@ -104,10 +106,12 @@ PlaylistsPage::PlaylistsPage(QWidget *p)
     menu->addAction(StdActions::self()->removeAction);
     menu->addAction(renamePlaylistAction);
     menuButton->setMenu(menu);
+    view->load(metaObject()->className());
 }
 
 PlaylistsPage::~PlaylistsPage()
 {
+    view->save(metaObject()->className());
 }
 
 void PlaylistsPage::saveConfig()
@@ -335,9 +339,10 @@ void PlaylistsPage::addItemsToPlayList(const QModelIndexList &indexes, const QSt
         return;
     }
 
+    bool filteredOnly=proxy.enabled() && Settings::self()->filteredOnly();
     // If we only have 1 item selected, see if it is a playlist. If so, we might be able to
     // just ask MPD to load it...
-    if (name.isEmpty() && 1==indexes.count() && 0==priorty) {
+    if (name.isEmpty() && 1==indexes.count() && 0==priorty && !filteredOnly) {
         QModelIndex idx=proxy.mapToSource(*(indexes.begin()));
         PlaylistsModel::Item *item=static_cast<PlaylistsModel::Item *>(idx.internalPointer());
 
@@ -347,11 +352,9 @@ void PlaylistsPage::addItemsToPlayList(const QModelIndexList &indexes, const QSt
         }
     }
 
-    QModelIndexList mapped;
-    bool checkNames=!name.isEmpty();
-    foreach (const QModelIndex &idx, indexes) {
-        QModelIndex m=proxy.mapToSource(idx);
-        if (checkNames) {
+    if (!name.isEmpty()) {
+        foreach (const QModelIndex &idx, indexes) {
+            QModelIndex m=proxy.mapToSource(idx);
             PlaylistsModel::Item *item=static_cast<PlaylistsModel::Item *>(m.internalPointer());
             if ( (item->isPlaylist() && static_cast<PlaylistsModel::PlaylistItem *>(item)->name==name) ||
                  (!item->isPlaylist() && static_cast<PlaylistsModel::SongItem *>(item)->parent->name==name) ) {
@@ -359,10 +362,9 @@ void PlaylistsPage::addItemsToPlayList(const QModelIndexList &indexes, const QSt
                 return;
             }
         }
-        mapped.append(m);
     }
 
-    QStringList files=PlaylistsModel::self()->filenames(mapped);
+    QStringList files=PlaylistsModel::self()->filenames(proxy.mapToSource(indexes, filteredOnly));
     if (!files.isEmpty()) {
         if (name.isEmpty()) {
             emit add(files, replace, priorty);
@@ -378,17 +380,10 @@ QList<Song> PlaylistsPage::selectedSongs(bool allowPlaylists) const
 {
     Q_UNUSED(allowPlaylists)
     QModelIndexList selected = view->selectedIndexes();
-
     if (selected.isEmpty()) {
         return QList<Song>();
     }
-
-    QModelIndexList mapped;
-    foreach (const QModelIndex &idx, selected) {
-        mapped.append(proxy.mapToSource(idx));
-    }
-
-    return PlaylistsModel::self()->songs(mapped);
+    return PlaylistsModel::self()->songs(proxy.mapToSource(selected, proxy.enabled() && Settings::self()->filteredOnly()));
 }
 
 void PlaylistsPage::addSelectionToDevice(const QString &udi)
