@@ -2,6 +2,7 @@
  * Cantata
  *
  * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2014 Niklas Wenzel <nikwen.developer@gmail.com>
  *
  * ----
  *
@@ -27,53 +28,33 @@
 #include <QStringList>
 #include <QPainter>
 #include <QFile>
-#include "localize.h"
-#include "qtplural.h"
-#ifdef ENABLE_KDE_SUPPORT
-#include <KDE/KGlobal>
-#endif
+#include "support/localize.h"
+#include "gui/plurals.h"
+#include "support/globalstatic.h"
 #include "albumsmodel.h"
-#include "settings.h"
+#include "gui/settings.h"
 #include "musiclibraryitemsong.h"
 #include "musiclibraryitemartist.h"
 #include "musiclibraryitemroot.h"
 #include "musiclibrarymodel.h"
 #include "playqueuemodel.h"
 #include "proxymodel.h"
-#include "song.h"
-#include "covers.h"
-#include "itemview.h"
-#include "mpdparseutils.h"
-#include "icons.h"
-#include "utils.h"
+#include "mpd/song.h"
+#include "gui/covers.h"
+#include "roles.h"
+#include "mpd/mpdparseutils.h"
+#include "widgets/icons.h"
+#include "support/utils.h"
 #include "config.h"
-
 #if defined ENABLE_MODEL_TEST
 #include "modeltest.h"
 #endif
 
 static int sortAlbums=AlbumsModel::Sort_AlbumArtist;
 
-#ifdef ENABLE_KDE_SUPPORT
-K_GLOBAL_STATIC(AlbumsModel, instance)
-#endif
-
-AlbumsModel * AlbumsModel::self()
-{
-    #ifdef ENABLE_KDE_SUPPORT
-    return instance;
-    #else
-    static AlbumsModel *instance=0;
-    if(!instance) {
-        instance=new AlbumsModel;
-        #if defined ENABLE_MODEL_TEST
-        new ModelTest(instance, instance);
-        #endif
-    }
-    return instance;
-    #endif
-}
+GLOBAL_STATIC(AlbumsModel, instance)
 
+#ifndef ENABLE_UBUNTU
 static MusicLibraryItemAlbum::CoverSize coverSize=MusicLibraryItemAlbum::CoverMedium;
 static QPixmap *theDefaultIcon=0;
 static QSize itemSize;
@@ -118,12 +99,40 @@ void AlbumsModel::setCoverSize(MusicLibraryItemAlbum::CoverSize size)
         coverSize=size;
     }
 }
+#endif
+
+AlbumsModel::Sort AlbumsModel::toSort(const QString &str)
+{
+    for (int i=0; i<=Sort_YearArtist; ++i) {
+        if (sortStr((Sort)i)==str) {
+            return (Sort)i;
+        }
+    }
+    return AlbumsModel::Sort_AlbumArtist;
+}
+
+QString AlbumsModel::sortStr(Sort m)
+{
+    switch (m) {
+    default:
+    case Sort_AlbumArtist: return QLatin1String("album-artist");
+    case Sort_AlbumYear:   return QLatin1String("album-year");
+    case Sort_ArtistAlbum: return QLatin1String("artist-album");
+    case Sort_ArtistYear:  return QLatin1String("artist-year");
+    case Sort_YearAlbum:   return QLatin1String("year-album");
+    case Sort_YearArtist:  return QLatin1String("year-artist");
+    }
+}
+
 
 AlbumsModel::AlbumsModel(QObject *parent)
     : ActionModel(parent)
     , enabled(false)
 //     , coversRequested(false)
 {
+    #if defined ENABLE_MODEL_TEST
+    new ModelTest(this, this);
+    #endif
 }
 
 AlbumsModel::~AlbumsModel()
@@ -193,6 +202,10 @@ QModelIndex AlbumsModel::index(int row, int col, const QModelIndex &parent) cons
     return row<items.count() ? createIndex(row, col, items.at(row)) : QModelIndex();
 }
 
+#ifdef ENABLE_UBUNTU
+static const QString constDefaultCover=QLatin1String("qrc:/album.svg");
+#endif
+
 QVariant AlbumsModel::data(const QModelIndex &index, int role) const
 {
     if (!index.isValid()) {
@@ -207,14 +220,21 @@ QVariant AlbumsModel::data(const QModelIndex &index, int role) const
         switch (role) {
         default:
             return ActionModel::data(index, role);
-        case ItemView::Role_Image:
+        #ifdef ENABLE_UBUNTU
+        case Cantata::Role_Image: {
+            QString cover=al->cover();
+            return cover.isEmpty() ? constDefaultCover : cover;
+        }
+        #else
+        case Cantata::Role_Image:
         case Qt::DecorationRole: {
-            if (al->cover) {
-                return *(al->cover);
-            }
             int iSize=iconSize();
-
-            if (Qt::DecorationRole==role && 0==iSize) {
+            if (iSize) {
+                QPixmap *pix=al->cover();
+                if (pix) {
+                    return *pix;
+                }
+            } else if (Qt::DecorationRole==role) {
                 return Icons::self()->albumIcon;
             }
 
@@ -227,41 +247,40 @@ QVariant AlbumsModel::data(const QModelIndex &index, int role) const
                 theDefaultIcon = new QPixmap(Icons::self()->albumIcon.pixmap(stdSize, stdSize)
                                             .scaled(QSize(cSize, cSize), Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
             }
-            if (!al->coverRequested && iSize && Song::SingleTracks!=al->type) {
-                al->coverRequested=true;
-                al->getCover();
-                if (al->cover) {
-                    return *(al->cover);
-                }
-            }
             return *theDefaultIcon;
         }
-        case Qt::ToolTipRole: {
-            quint32 year=al->songs.count() ? al->songs.at(0)->year : 0;
+        #endif
+        case Qt::ToolTipRole:
             return 0==al->songs.count()
-                ? al->name
-                : (year>0 ? QString("%1\n%2 (%3)\n").arg(al->artist).arg(al->album).arg(QString::number(year)) : QString("%1\n%2\n").arg(al->artist).arg(al->album))+
-                    #ifdef ENABLE_KDE_SUPPORT
-                    i18np("1 Track (%2)", "%1 Tracks (%2)", al->trackCount(), Song::formattedTime(al->totalTime(), true));
-                    #else
-                    QTP_TRACKS_DURATION_STR(al->trackCount(), Song::formattedTime(al->totalTime(), true));
-                    #endif
-        }
+                    ? QString()
+                    : (al->artist+QLatin1Char('\n')+al->albumDisplay()+QLatin1Char('\n')+
+                        Plurals::tracksWithDuration(al->trackCount(), Utils::formatTime(al->totalTime(), true)));
         case Qt::DisplayRole:
-            return al->name;
-        case ItemView::Role_MainText:
-            return Sort_AlbumArtist==sortAlbums ? al->album : al->artist;
-        case ItemView::Role_ImageSize: {
+            return al->album;
+        case Qt::FontRole:
+            if (al->isNew) {
+                QFont f=ActionModel::data(index, role).value<QFont>();
+                f.setBold(true);
+                return f;
+            }
+            break;
+        case Cantata::Role_MainText:
+            return al->albumDisplay();
+        case Cantata::Role_BriefMainText:
+            return al->album;
+        #ifndef ENABLE_UBUNTU
+        case Cantata::Role_ImageSize:
             return iconSize();
-        }
-        case ItemView::Role_SubText:
-            return Sort_AlbumArtist==sortAlbums ? al->artist : al->album;
-        case ItemView::Role_TitleText:
-            return i18nc("Album by Artist", "%1 by %2", al->album, al->artist);
         case Qt::SizeHintRole:
             if (!itemSize.isNull()) {
                 return itemSize;
             }
+            break;
+        #endif
+        case Cantata::Role_SubText:
+            return al->artist;
+        case Cantata::Role_TitleText:
+            return i18nc("Album by Artist", "%1 by %2", al->album, al->artist);
         }
     } else {
         SongItem *si=static_cast<SongItem *>(item);
@@ -269,16 +288,16 @@ QVariant AlbumsModel::data(const QModelIndex &index, int role) const
         switch (role) {
         default:
             return ActionModel::data(index, role);
+        #ifdef ENABLE_UBUNTU
+        case Cantata::Role_Image:
+            return QString();
+        #else
         case Qt::DecorationRole:
             return Song::Playlist==si->type ? Icons::self()->playlistIcon : Icons::self()->audioFileIcon;
-        case Qt::ToolTipRole: {
-            quint32 year=si->parent->songs.count() ? si->parent->songs.at(0)->year : 0;
-            return si->parent->artist+QLatin1String("<br/>")+
-                   si->parent->album+(year>0 ? (QLatin1String(" (")+QString::number(year)+QChar(')')) : QString())+QLatin1String("<br/>")+
-                   data(index, Qt::DisplayRole).toString()+QLatin1String("<br/>")+
-                   Song::formattedTime(si->time, true)+QLatin1String("<br/>")+
-                   QLatin1String("<small><i>")+si->filePath()+QLatin1String("</i></small>");
-        }
+        #endif
+        case Qt::ToolTipRole:
+            return si->toolTip();
+        case Cantata::Role_MainText:
         case Qt::DisplayRole:
             if (Song::Playlist==si->type) {
                 return si->isCueFile() ? i18n("Cue Sheet") : i18n("Playlist");
@@ -286,11 +305,9 @@ QVariant AlbumsModel::data(const QModelIndex &index, int role) const
             if (Song::SingleTracks==si->parent->type) {
                 return si->artistSong();
             }
-            else {
-                return si->trackAndTitleStr(Song::isVariousArtists(si->parent->artist) && !Song::isVariousArtists(si->artist));
-            }
-        case ItemView::Role_SubText:
-            return Song::formattedTime(si->time, true);
+            return si->trackAndTitleStr();
+        case Cantata::Role_SubText:
+            return Utils::formatTime(si->time, true);
         }
     }
 
@@ -318,6 +335,8 @@ QStringList AlbumsModel::filenames(const QModelIndexList &indexes, bool allowPla
 QList<Song> AlbumsModel::songs(const QModelIndexList &indexes, bool allowPlaylists) const
 {
     QList<Song> songs;
+    QSet<QString> files;
+
     foreach(QModelIndex index, indexes) {
         Item *item=static_cast<Item *>(index.internalPointer());
 
@@ -325,39 +344,44 @@ QList<Song> AlbumsModel::songs(const QModelIndexList &indexes, bool allowPlaylis
             QList<Song> albumSongs;
             const SongItem *cue=allowPlaylists ? static_cast<AlbumItem*>(item)->getCueFile() : 0;
             if (cue) {
-                if (!songs.contains(*cue)) {
+                if (!files.contains(cue->file)) {
                     albumSongs << *cue;
+                    files << cue->file;
                 }
             } else {
                 foreach (const SongItem *s, static_cast<AlbumItem*>(item)->songs) {
-                    if ((/*allowPlaylists || */Song::Playlist!=s->type) && !songs.contains(*s)) {
+                    if ((/*allowPlaylists || */Song::Playlist!=s->type) && !files.contains(s->file)) {
                         albumSongs << *s;
+                        files << s->file;
                     }
                 }
             }
             qSort(albumSongs);
             songs << albumSongs;
-        } else if ((allowPlaylists || Song::Playlist!=static_cast<SongItem*>(item)->type) && !songs.contains(*static_cast<SongItem*>(item))) {
+        } else if ((allowPlaylists || Song::Playlist!=static_cast<SongItem*>(item)->type) && !files.contains(static_cast<SongItem*>(item)->file)) {
             songs << *static_cast<SongItem*>(item);
+            files << static_cast<SongItem*>(item)->file;
         }
     }
     return songs;
 }
 
+#ifndef ENABLE_UBUNTU
 QMimeData * AlbumsModel::mimeData(const QModelIndexList &indexes) const
 {
     QMimeData *mimeData = new QMimeData();
     QStringList files=filenames(indexes, true);
     PlayQueueModel::encode(*mimeData, PlayQueueModel::constFileNameMimeType, files);
-    if (!MPDConnection::self()->getDetails().dir.isEmpty()) {
-        QStringList paths;
-        foreach (const QString &f, files) {
-            paths << MPDConnection::self()->getDetails().dir+f;
-        }
-        PlayQueueModel::encode(*mimeData, PlayQueueModel::constUriMimeType, paths);
-    }
+//    if (!MPDConnection::self()->getDetails().dir.isEmpty()) {
+//        QStringList paths;
+//        foreach (const QString &f, files) {
+//            paths << MPDConnection::self()->getDetails().dir+f;
+//        }
+//        PlayQueueModel::encode(*mimeData, PlayQueueModel::constUriMimeType, paths);
+//    }
     return mimeData;
 }
+#endif
 
 void AlbumsModel::update(const MusicLibraryItemRoot *root)
 {
@@ -375,7 +399,6 @@ void AlbumsModel::update(const MusicLibraryItemRoot *root)
 
     for (; it!=end; ++it) {
         (*it)->updated=false;
-        //(*it)->coverRequested=false;
     }
 
     for (int i = 0; i < root->childCount(); i++) {
@@ -383,12 +406,12 @@ void AlbumsModel::update(const MusicLibraryItemRoot *root)
         const QString &artist=artistItem->data();
         for (int j = 0; j < artistItem->childCount(); j++) {
             MusicLibraryItemAlbum *albumItem = static_cast<MusicLibraryItemAlbum*>(artistItem->childItem(j));
-            const QString &album=albumItem->data();
+            const QString &albumId=albumItem->albumId();
             bool found=false;
             it=items.begin();
             end=items.end();
             for (; it!=end; ++it) {
-                if ((*it)->year==albumItem->year() && (*it)->artist==artist && (*it)->album==album) {
+                if ((*it)->year==albumItem->year() && (*it)->artist==artist && (*it)->albumId()==albumId) {
                     if (!resettingModel) {
                         QModelIndex albumIndex=index(items.indexOf(*it), 0, QModelIndex());
                         bool hadSongs=!(*it)->songs.isEmpty();
@@ -410,17 +433,25 @@ void AlbumsModel::update(const MusicLibraryItemRoot *root)
                     (*it)->genres=albumItem->genres();
                     (*it)->updated=true;
                     found=true;
+                    if ((*it)->isNew!=albumItem->isNew()) {
+                        (*it)->isNew=albumItem->isNew();
+                        if (!resettingModel) {
+                            QModelIndex albumIndex=index(items.indexOf(*it), 0, QModelIndex());
+                            emit dataChanged(albumIndex, albumIndex);
+                        }
+                    }
                     break;
                 }
             }
 
             if (!found) {
                 changesMade=true;
-                AlbumItem *a=new AlbumItem(artist, album, albumItem->year());
+                AlbumItem *a=new AlbumItem(artist, albumItem->data(), albumId, albumItem->year());
                 a->setSongs(albumItem);
                 a->genres=albumItem->genres();
                 a->updated=true;
                 a->type=albumItem->songType();
+                a->isNew=albumItem->isNew();
                 if (!resettingModel) {
                     beginInsertRows(QModelIndex(), items.count(), items.count());
                 }
@@ -452,32 +483,67 @@ void AlbumsModel::update(const MusicLibraryItemRoot *root)
     }
 }
 
-void AlbumsModel::setCover(const Song &song, const QImage &img, const QString &file, bool update)
+void AlbumsModel::setCover(const Song &song, const QImage &img, const QString &file)
 {
-    Q_UNUSED(file)
-    if (img.isNull() || song.isArtistImageRequest() || song.isCdda()) {
+    #ifdef ENABLE_UBUNTU
+    if (img.isNull() || file.isEmpty()) {
         return;
     }
     QList<AlbumItem *>::Iterator it=items.begin();
     QList<AlbumItem *>::Iterator end=items.end();
     QString artist=MusicLibraryItemRoot::artistName(song);
-    QString album=song.albumName();
+    const QString &albumId=song.albumId();
 
     for (int row=0; it!=end; ++it, ++row) {
-        if ((*it)->artist==artist && (*it)->album==album) {
-            if (!(*it)->cover || update) {
-                (*it)->setCover(img);
+        if ((*it)->artist==artist && (*it)->albumId()==albumId) {
+            if ((*it)->coverRequested) {
+                (*it)->coverFile="file://"+file;
+                (*it)->coverRequested=false;
                 QModelIndex idx=index(row, 0, QModelIndex());
                 emit dataChanged(idx, idx);
             }
             return;
         }
     }
+    #else
+    Q_UNUSED(song)
+    Q_UNUSED(img)
+    Q_UNUSED(file)
+    #endif
 }
 
-void AlbumsModel::updateCover(const Song &song, const QImage &img, const QString &file)
+void AlbumsModel::coverLoaded(const Song &song, int s)
 {
-    setCover(song, img, file, true);
+    #ifdef ENABLE_UBUNTU
+    Q_UNUSED(song)
+    Q_UNUSED(s)
+    #else
+    if (s==iconSize() && !song.isArtistImageRequest()) {
+        QList<AlbumItem *>::Iterator it=items.begin();
+        QList<AlbumItem *>::Iterator end=items.end();
+        const QString &albumArtist=song.albumArtist();
+        const QString &albumId=song.albumId();
+
+        for (int row=0; it!=end; ++it, ++row) {
+            if ((*it)->artist==albumArtist && (*it)->albumId()==albumId) {
+                QModelIndex idx=index(row, 0, QModelIndex());
+                emit dataChanged(idx, idx);
+            }
+        }
+    }
+    #endif
+}
+
+void AlbumsModel::clearNewState()
+{
+    for (int i=0; i<items.count(); ++i) {
+        AlbumItem *al=items.at(i);
+        if (al->isNew) {
+            al->isNew=false;
+            QModelIndex idx=index(i, 0, QModelIndex());
+            emit dataChanged(idx, idx);
+        }
+    }
 }
 
 void AlbumsModel::clear()
@@ -486,7 +552,10 @@ void AlbumsModel::clear()
     qDeleteAll(items);
     items.clear();
     endResetModel();
-//     coversRequested=false;
+
+    #ifdef ENABLE_UBUNTU //For displaying a message when there is no album
+    emit updated();
+    #endif
 }
 
 void AlbumsModel::setEnabled(bool e)
@@ -497,19 +566,21 @@ void AlbumsModel::setEnabled(bool e)
     enabled=e;
 
     if (enabled) {
+        #ifdef ENABLE_UBUNTU
         connect(Covers::self(), SIGNAL(cover(const Song &, const QImage &, const QString &)),
                 this, SLOT(setCover(const Song &, const QImage &, const QString &)));
-        connect(Covers::self(), SIGNAL(coverUpdated(const Song &, const QImage &, const QString &)),
-                this, SLOT(updateCover(const Song &, const QImage &, const QString &)));
-//         connect(MusicLibraryModel::self(), SIGNAL(updated(const MusicLibraryItemRoot *)), AlbumsModel::self(), SLOT(update(const MusicLibraryItemRoot *)));
+        #else
+        connect(Covers::self(), SIGNAL(loaded(Song,int)), this, SLOT(coverLoaded(Song,int)));
+        #endif
         update(MusicLibraryModel::self()->root());
     } else {
         clear();
+        #ifdef ENABLE_UBUNTU
         disconnect(Covers::self(), SIGNAL(cover(const Song &, const QImage &, const QString &)),
-                    this, SLOT(setCover(const Song&, const QImage &, const QString &)));
-        disconnect(Covers::self(), SIGNAL(coverUpdated(const Song &, const QImage &, const QString &)),
-                   this, SLOT(updateCover(const Song &, const QImage &, const QString &)));
-//         disconnect(MusicLibraryModel::self(), SIGNAL(updated(const MusicLibraryItemRoot *)), AlbumsModel::self(), SLOT(update(const MusicLibraryItemRoot *)));
+                   this, SLOT(setCover(const Song &, const QImage &, const QString &)));
+        #else
+        disconnect(Covers::self(), SIGNAL(loaded(Song,int)), this, SLOT(coverLoaded(Song,int)));
+        #endif
     }
 }
 
@@ -523,43 +594,101 @@ void AlbumsModel::setAlbumSort(int s)
     if (s!=sortAlbums) {
         beginResetModel();
         sortAlbums=s;
-        foreach (AlbumItem *a, items) {
-            a->setName();
-        }
         endResetModel();
     }
 }
 
-AlbumsModel::AlbumItem::AlbumItem(const QString &ar, const QString &al, quint16 y)
+AlbumsModel::AlbumItem::AlbumItem(const QString &ar, const QString &al, const QString &i, quint16 y)
     : artist(ar)
     , album(al)
+    , id(i)
     , year(y)
-    , cover(0)
     , updated(false)
-    , coverRequested(false)
     , numTracks(0)
     , time(0)
+    , isNew(false)
+    #ifdef ENABLE_UBUNTU
+    , coverRequested(false)
+    #endif
 {
-    setName();
+    if (artist.startsWith(QLatin1String("The "))) {
+        nonTheArtist=artist.mid(4);
+    }
 }
 
 AlbumsModel::AlbumItem::~AlbumItem()
 {
     clearSongs();
-    delete cover;
 }
 
 bool AlbumsModel::AlbumItem::operator<(const AlbumItem &o) const
 {
-    if (AlbumsModel::Sort_ArtistAlbumYear==sortAlbums) {
-        int compare=artist.localeAwareCompare(o.artist);
-        if (0==compare) {
-            return year<o.year || (year==o.year && album.localeAwareCompare(o.album)<0);
-        } else {
+    switch (sortAlbums) {
+    default:
+    case Sort_AlbumArtist:  {
+        int compare=album.localeAwareCompare(o.album);
+        if (0!=compare) {
             return compare<0;
         }
-    } else {
-        return name.localeAwareCompare(o.name)<0;
+        compare=sortArtist().localeAwareCompare(o.sortArtist());
+        if (0!=compare) {
+            return compare<0;
+        }
+        return year==o.year ? id.compare(o.id)<0 : year<o.year;
+    }
+    case Sort_AlbumYear: {
+        int compare=album.localeAwareCompare(o.album);
+        if (0!=compare) {
+            return compare<0;
+        }
+        if (year!=o.year) {
+            return year<o.year;
+        }
+        compare=sortArtist().localeAwareCompare(o.sortArtist())<0;
+        return 0==compare ? id.compare(o.id)<0 : compare<0;
+    }
+    case Sort_ArtistAlbum: {
+        int compare=sortArtist().localeAwareCompare(o.sortArtist());
+        if (0!=compare) {
+            return compare<0;
+        }
+        compare=album.localeAwareCompare(o.album);
+        if (0!=compare) {
+            return compare<0;
+        }
+        return year==o.year ? id.compare(o.id)<0 : year<o.year;
+    }
+    case Sort_ArtistYear: {
+        int compare=sortArtist().localeAwareCompare(o.sortArtist());
+        if (0!=compare) {
+            return compare<0;
+        }
+        if (year!=o.year) {
+            return year<o.year;
+        }
+        compare=album.localeAwareCompare(o.album);
+        return 0==compare ? id.compare(o.id)<0 : compare<0;
+    }
+    case Sort_YearAlbum:
+        if (year==o.year) {
+            int compare=album.localeAwareCompare(o.album);
+            if (0!=compare) {
+                return compare<0;
+            }
+            compare=sortArtist().localeAwareCompare(o.sortArtist());
+            return 0==compare ? id.compare(o.id)<0 : compare<0;
+        }
+        return year<o.year;
+    case Sort_YearArtist:
+        if (year==o.year) {
+            int compare=sortArtist().localeAwareCompare(o.sortArtist());
+            if (0!=compare) {
+                return compare<0;
+            }
+            compare=album.localeAwareCompare(o.album);
+            return 0==compare ? id.compare(o.id)<0 : compare<0;
+        }
+        return year<o.year;
     }
 }
 
@@ -577,13 +706,6 @@ void AlbumsModel::AlbumItem::setSongs(MusicLibraryItemAlbum *ai)
     }
 }
 
-void AlbumsModel::AlbumItem::setName()
-{
-    name=AlbumsModel::Sort_AlbumArtist==sortAlbums
-            ? (album+QLatin1String(" - ")+artist)
-            : (artist+QLatin1String(" - ")+album);
-}
-
 quint32 AlbumsModel::AlbumItem::trackCount()
 {
     updateStats();
@@ -609,51 +731,45 @@ void AlbumsModel::AlbumItem::updateStats()
     }
 }
 
-void AlbumsModel::AlbumItem::getCover()
+#ifdef ENABLE_UBUNTU
+QString AlbumsModel::AlbumItem::cover()
+{
+    if (Song::SingleTracks!=type && songs.count() && coverFile.isEmpty() && !coverRequested) {
+        coverRequested=true;
+        coverFile=Covers::self()->requestImage(coverSong()).fileName;
+        if (!coverFile.isEmpty()) {
+            coverRequested=false;
+            coverFile="file://"+coverFile;
+        }
+    }
+
+    return coverFile;
+}
+#else
+QPixmap * AlbumsModel::AlbumItem::cover()
 {
     if (Song::SingleTracks!=type && songs.count()) {
-        cover=Covers::getScaledCover(artist, album, iconSize());
-        if (cover) {
-            return;
-        }
-
-        SongItem *firstSong=songs.first();
-        Song s;
-        if (Song::MultipleArtists==type) {  // Then Cantata has placed this album under 'Various Artists' but the actual album as a different AlbumArtist tag
-            s.artist=firstSong->albumArtist();
-        } else {
-            s.artist=firstSong->artist;
-            s.albumartist=Song::useComposer() && !firstSong->composer.isEmpty()
-                            ? firstSong->albumArtist() : artist;
-        }
-        s.album=album;
-        s.year=year;
-        s.file=firstSong->file;
-        s.type=type;
-        s.composer=firstSong->composer;
-        Covers::Image img=Covers::self()->requestImage(s);
-        if (!img.img.isNull()) {
-            setCover(img.img);
-        }
+        return Covers::self()->get(coverSong(), iconSize());
     }
+    return 0;
 }
+#endif
 
-void AlbumsModel::AlbumItem::setCover(const QImage &img)
+const Song & AlbumsModel::AlbumItem::coverSong()
 {
-    if (cover) {
-        delete cover;
+    if (cSong.isEmpty() && songs.count()) {
+        SongItem *firstSong=songs.first();
+        cSong.artist=firstSong->artist;
+        cSong.albumartist=Song::useComposer() && !firstSong->composer().isEmpty()
+                ? firstSong->albumArtist() : artist;
+        cSong.album=album;
+        cSong.year=year;
+        cSong.file=firstSong->file;
+        cSong.type=type;
+        cSong.setComposer(firstSong->composer());
+        cSong.setMbAlbumId(firstSong->mbAlbumId());
     }
-
-    if (Song::SingleTracks!=type && songs.count() && !img.isNull()) {
-        int size=iconSize();
-        QImage scaled=img.scaled(QSize(size, size), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
-        Covers::saveScaledCover(scaled, artist, album, size);
-        cover=new QPixmap(QPixmap::fromImage(scaled));
-        return;
-    }
-
-    int size=iconSize();
-    cover=new QPixmap(QPixmap::fromImage(img.scaled(QSize(size, size), Qt::IgnoreAspectRatio, Qt::SmoothTransformation)));
+    return cSong;
 }
 
 const AlbumsModel::SongItem *AlbumsModel::AlbumItem::getCueFile() const
