@@ -31,57 +31,39 @@
 #include <QFile>
 #include <QXmlStreamReader>
 #include <QXmlStreamWriter>
-#include "localize.h"
-#include "qtplural.h"
+#include "support/localize.h"
+#include "gui/plurals.h"
 #include "dirviewmodel.h"
 #include "dirviewitem.h"
 #include "dirviewitemfile.h"
 #include "playqueuemodel.h"
 #include "musiclibrarymodel.h"
-#include "itemview.h"
-#include "settings.h"
-#include "mpdconnection.h"
-#include "icon.h"
-#include "icons.h"
+#include "roles.h"
+#include "gui/settings.h"
+#include "mpd/mpdconnection.h"
+#include "support/icon.h"
+#include "widgets/icons.h"
 #include "config.h"
-#include "utils.h"
+#include "support/utils.h"
 #include "qtiocompressor/qtiocompressor.h"
-
+#include "support/globalstatic.h"
 #if defined ENABLE_MODEL_TEST
 #include "modeltest.h"
 #endif
 
-#ifdef ENABLE_KDE_SUPPORT
-K_GLOBAL_STATIC(DirViewModel, instance)
-#endif
-
-static const QLatin1String constCacheName("-folder-listing");
+const QLatin1String DirViewModel::constCacheName("-folder-listing");
 
 static QString cacheFileName()
 {
     MPDConnectionDetails details=MPDConnection::self()->getDetails();
     QString fileName=(!details.isLocal() ? details.hostname+'_'+QString::number(details.port) : details.hostname)
-                     +constCacheName+MusicLibraryModel::constLibraryCompressedExt;
+                     +DirViewModel::constCacheName+MusicLibraryModel::constLibraryCompressedExt;
     fileName.replace('/', '_');
     fileName.replace('~', '_');
     return Utils::cacheDir(MusicLibraryModel::constLibraryCache)+fileName;
 }
 
-DirViewModel * DirViewModel::self()
-{
-    #ifdef ENABLE_KDE_SUPPORT
-    return instance;
-    #else
-    static DirViewModel *instance=0;
-    if(!instance) {
-        instance=new DirViewModel;
-        #if defined ENABLE_MODEL_TEST
-        new ModelTest(instance, instance);
-        #endif
-    }
-    return instance;
-    #endif
-}
+GLOBAL_STATIC(DirViewModel, instance)
 
 DirViewModel::DirViewModel(QObject *parent)
     : ActionModel(parent)
@@ -89,6 +71,9 @@ DirViewModel::DirViewModel(QObject *parent)
     , databaseTimeUnreliable(false)
     , enabled(false)
 {
+    #if defined ENABLE_MODEL_TEST
+    new ModelTest(this, this);
+    #endif
 }
 
 DirViewModel::~DirViewModel()
@@ -179,6 +164,10 @@ int DirViewModel::columnCount(const QModelIndex &) const
     return 1;
 }
 
+#ifdef ENABLE_UBUNTU
+static const QString constFolderIcon=QLatin1String("qrc:/folder.svg");
+#endif
+
 QVariant DirViewModel::data(const QModelIndex &index, int role) const
 {
     if (!index.isValid()) {
@@ -188,34 +177,40 @@ QVariant DirViewModel::data(const QModelIndex &index, int role) const
     DirViewItem *item = static_cast<DirViewItem *>(index.internalPointer());
 
     switch (role) {
+    #ifdef ENABLE_UBUNTU
+    case Cantata::Role_Image:
+        return DirViewItem::Type_Dir==item->type() ? constFolderIcon : QString();
+    #else
     case Qt::DecorationRole: {
-        if (item->type() == DirViewItem::Type_Dir) {
+        if (DirViewItem::Type_Dir==item->type()) {
             return Icons::self()->folderIcon;
-        } else if (item->type() == DirViewItem::Type_File) {
+        } else if (DirViewItem::Type_File==item->type()) {
             return DirViewItemFile::Audio!=static_cast<DirViewItemFile *>(item)->fileType() ? Icons::self()->playlistIcon : Icons::self()->audioFileIcon;
         }
         break;
     }
+    #endif
+    #ifdef ENABLE_UBUNTU
+    case Cantata::Role_TitleText:
+        switch (item->type()) {
+        case DirViewItem::Type_Dir:
+            return static_cast<DirViewItemDir *>(item)->fullName();
+        default:
+        case DirViewItem::Type_File:
+            return item->data();
+        }
+    #endif
+    case Cantata::Role_MainText:
     case Qt::DisplayRole:
         return item->data();
     case Qt::ToolTipRole:
         return 0==item->childCount()
             ? item->data()
-            : item->data()+"\n"+
-                #ifdef ENABLE_KDE_SUPPORT
-                i18np("1 Entry", "%1 Entries", item->childCount());
-                #else
-                QTP_ENTRIES_STR(item->childCount());
-                #endif
-    case ItemView::Role_SubText:
+            : item->data()+"\n"+Plurals::entries(item->childCount());
+    case Cantata::Role_SubText:
         switch (item->type()) {
         case DirViewItem::Type_Dir:
-            #ifdef ENABLE_KDE_SUPPORT
-            return i18np("1 Entry", "%1 Entries", item->childCount());
-            #else
-            return QTP_ENTRIES_STR(item->childCount());
-            #endif
-            break;
+            return Plurals::entries(item->childCount());
         case DirViewItem::Type_File:
             switch (static_cast<DirViewItemFile *>(item)->fileType()) {
             case DirViewItemFile::Audio:    return i18n("Audio File");
@@ -259,7 +254,7 @@ static quint32 constVersion=2;
 void DirViewModel::toXML()
 {
     QString filename=cacheFileName();
-    if ((!rootItem || 0==rootItem->childCount()) && databaseTime.date().year()<2000) {
+    if ((!rootItem || 0==rootItem->childCount()) && !MusicLibraryModel::validCacheDate(databaseTime)) {
         if (QFile::exists(filename)) {
             QFile::remove(filename);
         }
@@ -398,8 +393,7 @@ void DirViewModel::updateDirView(DirViewItemRoot *newroot, const QDateTime &dbUp
 
     bool incremental=enabled && rootItem->childCount() && newroot->childCount();
     bool updatedListing=false;
-    bool needToUpdate=!databaseTime.isValid();
-    bool needToSave=dbUpdate>databaseTime;
+    bool needToSave=!databaseTime.isValid() || (MusicLibraryModel::validCacheDate(dbUpdate) && dbUpdate>databaseTime);
 
     if (incremental && !QFile::exists(cacheFileName())) {
         incremental=false;
@@ -439,23 +433,27 @@ void DirViewModel::updateDirView(DirViewItemRoot *newroot, const QDateTime &dbUp
     //
     // Mopidy users, and users of the proxy DB plugin, will have to force Cantata to refresh :-(
     if (!fromFile) {
-        databaseTimeUnreliable=!dbUpdate.isValid() || dbUpdate.date().year()<2000; // See note in updatingMpd()
+        databaseTimeUnreliable=!MusicLibraryModel::validCacheDate(dbUpdate); // See note in updatingMpd()
     }
-    if ((!databaseTime.isValid() && !dbUpdate.isValid()) || (databaseTime.date().year()<2000 && dbUpdate.date().year()<2000)) {
+    if (!MusicLibraryModel::validCacheDate(databaseTime) && !MusicLibraryModel::validCacheDate(dbUpdate)) {
         databaseTime=QDateTime::currentDateTime();
     }
 
-    if ((updatedListing || needToUpdate) && (!fromFile && (needToSave || needToUpdate))) {
+    if (!fromFile && (needToSave || updatedListing)) {
         toXML();
     }
+
+    #ifdef ENABLE_UBUNTU
+    emit updated();
+    #endif
 }
 
 void DirViewModel::updatingMpd()
 {
     // MPD/Mopidy is being updated. If MPD's database-time is not reliable (as is the case for older proxy DBs, and Mopidy)
     // then we set the databaseTime to NOW when updated. This means we will miss any updates. So, for these scenarios, when
-    // a users presses 'Refresh Database' in Cantata's main window, we need to reset our view of the databaseTime to null, s
-    // that we update. This does mean that we will ALWAYS fetch the whole listing - but we have n oway of knowing if it changed
+    // a user presses 'Refresh Database' in Cantata's main window, we need to reset our view of the databaseTime to null, so
+    // that we update. This does mean that we will ALWAYS fetch the whole listing - but we have no way of knowing if it changed
     // or not :-(
     if (databaseTimeUnreliable) {
         removeCache();
@@ -547,20 +545,22 @@ QStringList DirViewModel::filenames(const QModelIndexList &indexes, bool allowPl
     return fnames;
 }
 
+#ifndef ENABLE_UBUNTU
 QMimeData *DirViewModel::mimeData(const QModelIndexList &indexes) const
 {
     QMimeData *mimeData = new QMimeData();
     QStringList files=filenames(indexes, true);
     PlayQueueModel::encode(*mimeData, PlayQueueModel::constFileNameMimeType, files);
-    if (!MPDConnection::self()->getDetails().dir.isEmpty()) {
-        QStringList paths;
-        foreach (const QString &f, files) {
-            paths << MPDConnection::self()->getDetails().dir+f;
-        }
-        PlayQueueModel::encode(*mimeData, PlayQueueModel::constUriMimeType, paths);
-    }
+//    if (!MPDConnection::self()->getDetails().dir.isEmpty()) {
+//        QStringList paths;
+//        foreach (const QString &f, files) {
+//            paths << MPDConnection::self()->getDetails().dir+f;
+//        }
+//        PlayQueueModel::encode(*mimeData, PlayQueueModel::constUriMimeType, paths);
+//    }
     return mimeData;
 }
+#endif
 
 static inline void addFile(DirViewItem *item, QStringList &insertInto, QStringList &checkAgainst, bool allowPlaylists)
 {
