@@ -31,17 +31,20 @@
 #include "musiclibrarymodel.h"
 #include "albumsmodel.h"
 #include "playqueuemodel.h"
-#include "settings.h"
+#include "dirviewmodel.h"
 #include "config.h"
-#include "covers.h"
-#include "itemview.h"
-#include "mpdparseutils.h"
-#include "mpdconnection.h"
-#include "localize.h"
-#include "utils.h"
-#include "icons.h"
-#include "stdactions.h"
+#include "gui/covers.h"
+#include "mpd/mpdparseutils.h"
+#include "mpd/mpdconnection.h"
+#include "support/localize.h"
+#include "support/utils.h"
+#include "widgets/icons.h"
+#include "gui/stdactions.h"
 #include "qtiocompressor/qtiocompressor.h"
+#include "support/globalstatic.h"
+#ifndef ENABLE_UBUNTU
+#include "gui/settings.h"
+#endif
 #include <QCommonStyle>
 #include <QFile>
 #include <QTimer>
@@ -50,33 +53,11 @@
 #include <QDir>
 #include <QMimeData>
 #include <QStringList>
-#ifdef ENABLE_KDE_SUPPORT
-#include <KDE/KGlobal>
-#endif
-
 #if defined ENABLE_MODEL_TEST
 #include "modeltest.h"
 #endif
 
-#ifdef ENABLE_KDE_SUPPORT
-K_GLOBAL_STATIC(MusicLibraryModel, instance)
-#endif
-
-MusicLibraryModel * MusicLibraryModel::self()
-{
-    #ifdef ENABLE_KDE_SUPPORT
-    return instance;
-    #else
-    static MusicLibraryModel *instance=0;
-    if(!instance) {
-        instance=new MusicLibraryModel;
-        #if defined ENABLE_MODEL_TEST
-        new ModelTest(instance, instance);
-        #endif
-    }
-    return instance;
-    #endif
-}
+GLOBAL_STATIC(MusicLibraryModel, instance)
 
 const QLatin1String MusicLibraryModel::constLibraryCache("library/");
 const QLatin1String MusicLibraryModel::constLibraryExt(".xml");
@@ -119,26 +100,34 @@ void MusicLibraryModel::convertCache(const QString &compressedName)
     }
 }
 
-//void MusicLibraryModel::cleanCache()
-//{
-//    QSet<QString> existing;
-//    QList<MPDConnectionDetails> connections=Settings::self()->allConnections();
-//    foreach (const MPDConnectionDetails &conn, connections) {
-//        QString withPort=cacheFileName(conn);
-//        QString withoutPort=cacheFileName(conn, false);
-//        if (withPort!=withoutPort) {
-//            existing.insert(withoutPort);
-//        }
-//        existing.insert(withPort);
-//    }
-//    QDir dir(Utils::cacheDir(constLibraryCache));
-//    QFileInfoList files=dir.entryInfoList(QStringList() << "*"+constLibraryExt << "*"+constLibraryCompressedExt, QDir::Files);
-//    foreach (const QFileInfo &file, files) {
-//        if (!existing.contains(file.fileName())) {
-//            QFile::remove(file.absoluteFilePath());
-//        }
-//    }
-//}
+void MusicLibraryModel::cleanCache()
+{
+    #ifdef ENABLE_UBUNTU
+    // TODO???
+    #else
+    QSet<QString> existing;
+    QList<MPDConnectionDetails> connections=Settings::self()->allConnections();
+    QString dirPath=Utils::cacheDir(MusicLibraryModel::constLibraryCache, false);
+    if (dirPath.isEmpty()) {
+        return;
+    }
+
+    foreach (const MPDConnectionDetails &conn, connections) {
+        QString fileName=cacheFileName(conn).mid(dirPath.length());
+        existing.insert(fileName);
+        // Dir view cache file...
+        fileName=fileName.left(fileName.length()-QString(constLibraryCompressedExt).length());
+        fileName+=DirViewModel::constCacheName+(constLibraryCompressedExt);
+        existing.insert(fileName);
+    }
+    QFileInfoList files=QDir(dirPath).entryInfoList(QStringList() << "*"+constLibraryExt << "*"+constLibraryCompressedExt, QDir::Files);
+    foreach (const QFileInfo &file, files) {
+        if (!existing.contains(file.fileName())) {
+            QFile::remove(file.absoluteFilePath());
+        }
+    }
+    #endif
+}
 
 MusicLibraryModel::MusicLibraryModel(QObject *parent, bool isMpdModel, bool isCheckable)
     : MusicModel(parent)
@@ -149,17 +138,21 @@ MusicLibraryModel::MusicLibraryModel(QObject *parent, bool isMpdModel, bool isCh
 {
     if (mpdModel)
     {
-        connect(Covers::self(), SIGNAL(artistImage(const Song &, const QImage &, const QString &)),
-                this, SLOT(setArtistImage(const Song &, const QImage &)));
+        connect(Covers::self(), SIGNAL(loaded(Song,int)), this, SLOT(coverLoaded(Song,int)));
         connect(Covers::self(), SIGNAL(cover(const Song &, const QImage &, const QString &)),
                 this, SLOT(setCover(const Song &, const QImage &, const QString &)));
-        connect(Covers::self(), SIGNAL(coverUpdated(const Song &, const QImage &, const QString &)),
-                this, SLOT(updateCover(const Song &, const QImage &, const QString &)));
+        #ifdef ENABLE_UBUNTU
+        connect(Covers::self(), SIGNAL(artistImage(const Song &, const QImage &, const QString &)),
+                this, SLOT(setArtistImage(const Song &, const QImage &, const QString &)));
+        #endif
         connect(MPDConnection::self(), SIGNAL(updatingDatabase()), this, SLOT(updatingMpd()));
         connect(MPDConnection::self(), SIGNAL(musicLibraryUpdated(MusicLibraryItemRoot *, QDateTime)),
                 this, SLOT(updateMusicLibrary(MusicLibraryItemRoot *, QDateTime)));
     }
     rootItem->setModel(this);
+    #if defined ENABLE_MODEL_TEST
+    new ModelTest(this, this);
+    #endif
 }
 
 MusicLibraryModel::~MusicLibraryModel()
@@ -231,6 +224,16 @@ QVariant MusicLibraryModel::data(const QModelIndex &index, int role) const
     if (checkable && Qt::CheckStateRole==role) {
         return static_cast<MusicLibraryItem *>(index.internalPointer())->checkState();
     }
+
+    if (!checkable && Qt::FontRole==role) {
+        MusicLibraryItem *item = static_cast<MusicLibraryItem *>(index.internalPointer());
+        if ((MusicLibraryItem::Type_Album==item->itemType() || MusicLibraryItem::Type_Artist==item->itemType()) &&
+            static_cast<MusicLibraryItemContainer *>(item)->isNew()) {
+            QFont f=MusicModel::data(index, role).value<QFont>();
+            f.setBold(true);
+            return f;
+        }
+    }
     return MusicModel::data(index, role);
 }
 
@@ -423,6 +426,38 @@ QSet<QString> MusicLibraryModel::getAlbumArtists()
     return a;
 }
 
+void MusicLibraryModel::checkForNewSongs()
+{
+    foreach (MusicLibraryItem *artist, rootItem->childItems()) {
+        MusicLibraryItemArtist *artistItem=static_cast<MusicLibraryItemArtist *>(artist);
+        if (artistItem->isNew()) {
+            emit haveNewItems(true);
+            return;
+        }
+    }
+}
+
+void MusicLibraryModel::clearNewState()
+{
+    foreach (MusicLibraryItem *artist, rootItem->childItems()) {
+        MusicLibraryItemArtist *artistItem=static_cast<MusicLibraryItemArtist *>(artist);
+        if (artistItem->isNew()) {
+            QModelIndex artistIndex=index(artistItem->row(), 0, QModelIndex());
+            foreach (MusicLibraryItem *album, artistItem->childItems()) {
+                MusicLibraryItemAlbum *albumItem=static_cast<MusicLibraryItemAlbum *>(album);
+                if (albumItem->isNew()) {
+                    albumItem->setIsNew(false);
+                    QModelIndex albumIndex=index(albumItem->row(), 0, artistIndex);
+                    emit dataChanged(albumIndex, albumIndex);
+                }
+            }
+            artistItem->setIsNew(false);
+            emit dataChanged(artistIndex, artistIndex);
+        }
+    }
+    emit haveNewItems(false);
+}
+
 void MusicLibraryModel::updateMusicLibrary(MusicLibraryItemRoot *newroot, QDateTime dbUpdate, bool fromFile)
 {
     if (!mpdModel || (databaseTime.isValid() && databaseTime >= dbUpdate)) {
@@ -433,9 +468,8 @@ void MusicLibraryModel::updateMusicLibrary(MusicLibraryItemRoot *newroot, QDateT
     newroot->setUseAlbumImages(rootItem->useAlbumImages());
     newroot->setUseArtistImages(rootItem->useArtistImages());
     bool updatedSongs=false;
-    bool needToSave=dbUpdate>databaseTime;
+    bool needToSave=!databaseTime.isValid() || (validCacheDate(dbUpdate) && dbUpdate>databaseTime);
     bool incremental=rootItem->childCount() && newroot->childCount();
-    bool needToUpdate=!databaseTime.isValid();
 
     if (incremental && !QFile::exists(cacheFileName())) {
         incremental=false;
@@ -444,6 +478,9 @@ void MusicLibraryModel::updateMusicLibrary(MusicLibraryItemRoot *newroot, QDateT
     databaseTime = dbUpdate;
     if (incremental) {
         updatedSongs=update(newroot->allSongs());
+        if (updatedSongs) {
+            checkForNewSongs();
+        }
         delete newroot;
     } else {
         const MusicLibraryItemRoot *oldRoot = rootItem;
@@ -465,32 +502,58 @@ void MusicLibraryModel::updateMusicLibrary(MusicLibraryItemRoot *newroot, QDateT
     //
     // Mopidy users, and users of the proxy DB plugin, will have to force Cantata to refresh :-(
     if (!fromFile) {
-        databaseTimeUnreliable=!dbUpdate.isValid() || dbUpdate.date().year()<2000; // See note in updatingMpd()
+        databaseTimeUnreliable=!validCacheDate(dbUpdate); // See note in updatingMpd()
     }
-    if ((!databaseTime.isValid() && !dbUpdate.isValid()) || (databaseTime.date().year()<2000 && dbUpdate.date().year()<2000)) {
+    if (!validCacheDate(databaseTime) && !validCacheDate(dbUpdate)) {
         databaseTime=QDateTime::currentDateTime();
     }
 
-    if ((updatedSongs || needToUpdate) && (!fromFile && (needToSave || needToUpdate))) {
+    if (!fromFile && (needToSave || updatedSongs)) {
         rootItem->toXML(cacheFileName(), databaseTime, databaseTimeUnreliable);
     }
 
     AlbumsModel::self()->update(rootItem);
     emit updateGenres(rootItem->genres());
+    #ifdef ENABLE_UBUNTU
+    if (updatedSongs) {
+        emit updated();
+    }
+    #endif
 }
 
 void MusicLibraryModel::updatingMpd()
 {
     // MPD/Mopidy is being updated. If MPD's database-time is not reliable (as is the case for older proxy DBs, and Mopidy)
     // then we set the databaseTime to NOW when updated. This means we will miss any updates. So, for these scenarios, when
-    // a users presses 'Refresh Database' in Cantata's main window, we need to reset our view of the databaseTime to null, s
-    // that we update. This does mean that we will ALWAYS fetch the whole listing - but we have n oway of knowing if it changed
+    // a user presses 'Refresh Database' in Cantata's main window, we need to reset our view of the databaseTime to null, so
+    // that we update. This does mean that we will ALWAYS fetch the whole listing - but we have no way of knowing if it changed
     // or not :-(
     if (databaseTimeUnreliable) {
         removeCache();
     }
 }
 
+void MusicLibraryModel::setArtistImage(const Song &song, const QImage &img, const QString &file)
+{
+    #ifdef ENABLE_UBUNTU
+    if (!rootItem->useArtistImages() || img.isNull() || file.isEmpty() ||
+        song.file.startsWith("http://") || song.name().startsWith("http://")) {
+        return;
+    }
+
+    MusicLibraryItemArtist *artistItem = rootItem->artist(song, false);
+    if (artistItem && artistItem->coverName().isEmpty()) {
+        artistItem->setCover(file);
+        QModelIndex idx=index(artistItem->row(), 0, QModelIndex());
+        emit dataChanged(idx, idx);
+    }
+    #else
+    Q_UNUSED(song)
+    Q_UNUSED(img)
+    Q_UNUSED(file)
+    #endif
+}
+
 bool MusicLibraryModel::update(const QSet<Song> &songs)
 {
     bool updatedSongs=rootItem->update(songs);
@@ -648,50 +711,69 @@ QList<Song> MusicLibraryModel::getArtistAlbumsFirstTracks(const Song &song) cons
     return tracks;
 }
 
-void MusicLibraryModel::setArtistImage(const Song &song, const QImage &img, bool update)
+// Currently ONLY artist images are always loaded from non UI thread.
+void MusicLibraryModel::coverLoaded(const Song &song, int size)
 {
-    if (!rootItem->useArtistImages() || img.isNull() || MusicLibraryItemAlbum::CoverNone==MusicLibraryItemAlbum::currentCoverSize() ||
-        song.file.startsWith("http://") || song.name.startsWith("http://")) {
+    if (
+        #ifndef ENABLE_UBUNTU
+        MusicLibraryItemAlbum::CoverNone==MusicLibraryItemAlbum::currentCoverSize() || size!=MusicLibraryItemAlbum::iconSize(rootItem->useLargeImages()) ||
+        #endif
+        //song.isCdda() || (song.isArtistImageRequest() && !rootItem->useArtistImages()) ||
+        song.isCdda() || !song.isArtistImageRequest() || !rootItem->useArtistImages() ||
+        song.file.startsWith("http://") || song.name().startsWith("http://")) {
         return;
     }
-
-    MusicLibraryItemArtist *artistItem = rootItem->artist(song, false);
-    if (artistItem && artistItem->setCover(img, update)) {
-        QModelIndex idx=index(artistItem->row(), 0, QModelIndex());
-        emit dataChanged(idx, idx);
-    }
+    //if (song.isArtistImageRequest()) {
+        MusicLibraryItemArtist *artistItem = rootItem->artist(song, false);
+        if (artistItem) {
+            QModelIndex idx=index(artistItem->row(), 0, QModelIndex());
+            emit dataChanged(idx, idx);
+        }
+//    } else {
+//        MusicLibraryItemArtist *artistItem = rootItem->artist(song, false);
+//        if (artistItem) {
+//            MusicLibraryItemAlbum *albumItem = artistItem->album(song, false);
+//            if (albumItem) {
+//                QModelIndex idx=index(albumItem->row(), 0, index(artistItem->row(), 0, QModelIndex()));
+//                emit dataChanged(idx, idx);
+//            }
+//        }
+//    }
 }
 
 void MusicLibraryModel::setCover(const Song &song, const QImage &img, const QString &file)
 {
-    setCover(song, img, file, false);
-}
-
-void MusicLibraryModel::updateCover(const Song &song, const QImage &img, const QString &file)
-{
-    if (song.isArtistImageRequest()) {
-        setArtistImage(song, img, true);
-    } else {
-        setCover(song, img, file, true);
-    }
-}
-
-void MusicLibraryModel::setCover(const Song &song, const QImage &img, const QString &file, bool update)
-{
-    Q_UNUSED(file)
-    if (!rootItem->useAlbumImages() || img.isNull() || MusicLibraryItemAlbum::CoverNone==MusicLibraryItemAlbum::currentCoverSize() ||
-        song.isCdda() || song.file.startsWith("http:/") || song.name.startsWith("http:/")) {
+    #ifdef ENABLE_UBUNTU
+    if (!rootItem->useAlbumImages() || img.isNull() || file.isEmpty() ||
+        song.isCdda() || song.file.startsWith("http:/") || song.name().startsWith("http:/")) {
         return;
     }
 
     MusicLibraryItemArtist *artistItem = rootItem->artist(song, false);
     if (artistItem) {
         MusicLibraryItemAlbum *albumItem = artistItem->album(song, false);
-        if (albumItem && albumItem->setCover(img, update)) {
+        if (albumItem && albumItem->coverName().isEmpty()) {
+            albumItem->setCover(file);
             QModelIndex idx=index(albumItem->row(), 0, index(artistItem->row(), 0, QModelIndex()));
             emit dataChanged(idx, idx);
         }
     }
+    #else
+    Q_UNUSED(file)
+    if (!rootItem->useAlbumImages() || img.isNull() || MusicLibraryItemAlbum::CoverNone==MusicLibraryItemAlbum::currentCoverSize() ||
+        song.isCdda() || song.file.startsWith("http:/") || song.name().startsWith("http:/")) {
+        return;
+    }
+
+    MusicLibraryItemArtist *artistItem = rootItem->artist(song, false);
+    if (artistItem) {
+        MusicLibraryItemAlbum *albumItem = artistItem->album(song, false);
+        if (albumItem && albumItem->saveToCache(img)) {
+            QModelIndex idx=index(albumItem->row(), 0, index(artistItem->row(), 0, QModelIndex()));
+            emit dataChanged(idx, idx);
+        }
+    }
+    #endif
 }
 
 /**
@@ -743,18 +825,20 @@ QStringList MusicLibraryModel::filenames(const QModelIndexList &indexes, bool al
     return fnames;
 }
 
-static inline void addSong(const MusicLibraryItem *song, QList<Song> &insertInto, QList<Song> &checkAgainst, bool allowPlaylists)
+static inline void addSong(const MusicLibraryItem *song, QList<Song> &insertInto, QSet<QString> &checkAgainst, bool allowPlaylists)
 {
     if (MusicLibraryItem::Type_Song==song->itemType() &&
         (allowPlaylists || Song::Playlist!=static_cast<const MusicLibraryItemSong*>(song)->song().type) &&
-        !checkAgainst.contains(static_cast<const MusicLibraryItemSong*>(song)->song())) {
+        !checkAgainst.contains(static_cast<const MusicLibraryItemSong*>(song)->file())) {
         insertInto << static_cast<const MusicLibraryItemSong*>(song)->song();
+        checkAgainst << static_cast<const MusicLibraryItemSong*>(song)->file();
     }
 }
 
 QList<Song> MusicLibraryModel::songs(const QModelIndexList &indexes, bool allowPlaylists) const
 {
     QList<Song> songs;
+    QSet<QString> files;
 
     foreach(QModelIndex index, indexes) {
         const MusicLibraryItem *item = static_cast<MusicLibraryItem *>(index.internalPointer());
@@ -769,10 +853,10 @@ QList<Song> MusicLibraryModel::songs(const QModelIndexList &indexes, bool allowP
                 QList<Song> albumSongs;
                 const MusicLibraryItemSong *cue=allowPlaylists ? static_cast<const MusicLibraryItemAlbum *>(i)->getCueFile() : 0;
                 if (cue) {
-                    addSong(cue, albumSongs, songs, true);
+                    addSong(cue, albumSongs, files, true);
                 } else {
                     foreach (const MusicLibraryItem *song, static_cast<const MusicLibraryItemContainer *>(i)->childItems()) {
-                        addSong(song, albumSongs, songs, false);
+                        addSong(song, albumSongs, files, false);
                     }
                 }
                 qSort(albumSongs);
@@ -784,10 +868,10 @@ QList<Song> MusicLibraryModel::songs(const QModelIndexList &indexes, bool allowP
             QList<Song> albumSongs;
             const MusicLibraryItemSong *cue=allowPlaylists ? static_cast<const MusicLibraryItemAlbum *>(item)->getCueFile() : 0;
             if (cue) {
-                addSong(cue, albumSongs, songs, true);
+                addSong(cue, albumSongs, files, true);
             } else {
                 foreach (const MusicLibraryItem *song, static_cast<const MusicLibraryItemContainer *>(item)->childItems()) {
-                    addSong(song, albumSongs, songs, false);
+                    addSong(song, albumSongs, files, false);
                 }
             }
             qSort(albumSongs);
@@ -795,7 +879,7 @@ QList<Song> MusicLibraryModel::songs(const QModelIndexList &indexes, bool allowP
             break;
         }
         case MusicLibraryItem::Type_Song:
-            addSong(item, songs, songs, allowPlaylists);
+            addSong(item, songs, files, allowPlaylists);
             break;
         default:
             break;
@@ -838,6 +922,7 @@ QList<Song> MusicLibraryModel::songs(const QStringList &filenames, bool insertNo
     return songs;
 }
 
+#ifndef ENABLE_UBUNTU
 /**
 * Convert the data at indexes into mimedata ready for transport
 *
@@ -849,12 +934,14 @@ QMimeData *MusicLibraryModel::mimeData(const QModelIndexList &indexes) const
     QMimeData *mimeData = new QMimeData();
     QStringList files=filenames(indexes, true);
     PlayQueueModel::encode(*mimeData, PlayQueueModel::constFileNameMimeType, files);
-    if (!MPDConnection::self()->getDetails().dir.isEmpty()) {
-        QStringList paths;
-        foreach (const QString &f, files) {
-            paths << MPDConnection::self()->getDetails().dir+f;
-        }
-        PlayQueueModel::encode(*mimeData, PlayQueueModel::constUriMimeType, paths);
-    }
+//    if (!MPDConnection::self()->getDetails().dir.isEmpty()) {
+//        QStringList paths;
+//        foreach (const QString &f, files) {
+//            paths << MPDConnection::self()->getDetails().dir+f;
+//        }
+//        PlayQueueModel::encode(*mimeData, PlayQueueModel::constUriMimeType, paths);
+//    }
     return mimeData;
 }
+#endif
+
