@@ -30,57 +30,42 @@
 #include "musiclibrarymodel.h"
 #include "dirviewmodel.h"
 #include "onlineservicesmodel.h"
-#include "onlineservice.h"
+#include "online/onlineservice.h"
 #ifdef ENABLE_DEVICES_SUPPORT
-#include "onlinedevice.h"
+#include "online/onlinedevice.h"
 #endif
-#include "jamendoservice.h"
-#include "magnatuneservice.h"
-#include "soundcloudservice.h"
-#include "podcastservice.h"
+#include "online/jamendoservice.h"
+#include "online/magnatuneservice.h"
+#include "online/soundcloudservice.h"
+#include "online/podcastservice.h"
 #include "playqueuemodel.h"
-#include "itemview.h"
-#include "mpdparseutils.h"
-#include "localize.h"
-#include "qtplural.h"
-#include "icons.h"
-#include "filejob.h"
-#include "utils.h"
-#include "stdactions.h"
-#include "actioncollection.h"
-#include "networkaccessmanager.h"
-#include "settings.h"
+#include "roles.h"
+#include "mpd/mpdparseutils.h"
+#include "support/localize.h"
+#include "gui/plurals.h"
+#include "widgets/icons.h"
+#include "devices/filejob.h"
+#include "support/utils.h"
+#include "gui/stdactions.h"
+#include "support/actioncollection.h"
+#include "network/networkaccessmanager.h"
+#include "gui/settings.h"
+#include "support/globalstatic.h"
+#ifdef ENABLE_STREAMS
+#include "streamsmodel.h"
+#endif
 #include <QStringList>
 #include <QMimeData>
 #include <QFile>
 #include <QDir>
 #include <QDebug>
-#ifdef ENABLE_KDE_SUPPORT
-#include <KDE/KGlobal>
-K_GLOBAL_STATIC(OnlineServicesModel, instance)
-#endif
-
 #if defined ENABLE_MODEL_TEST
 #include "modeltest.h"
 #endif
 
 QString OnlineServicesModel::constUdiPrefix("online-service://");
 
-OnlineServicesModel * OnlineServicesModel::self()
-{
-    #ifdef ENABLE_KDE_SUPPORT
-    return instance;
-    #else
-    static OnlineServicesModel *instance=0;
-    if(!instance) {
-        instance=new OnlineServicesModel;
-        #if defined ENABLE_MODEL_TEST
-        new ModelTest(instance, instance);
-        #endif
-    }
-    return instance;
-    #endif
-}
+GLOBAL_STATIC(OnlineServicesModel, instance)
 
 OnlineServicesModel::OnlineServicesModel(QObject *parent)
     : MultiMusicModel(parent)
@@ -93,7 +78,25 @@ OnlineServicesModel::OnlineServicesModel(QObject *parent)
     subscribeAction = ActionCollection::get()->createAction("subscribeonlineservice", i18n("Add Subscription"), "list-add");
     unSubscribeAction = ActionCollection::get()->createAction("unsubscribeonlineservice", i18n("Remove Subscription"), "list-remove");
     refreshSubscriptionAction = ActionCollection::get()->createAction("refreshsubscription", i18n("Refresh Subscription"), "view-refresh");
+    #ifdef ENABLE_STREAMS
+    searchAction = StreamsModel::self()->searchAct();
+    #else
+    // For Mac/Unity we try to hide icons from menubar menus. However, search is used in the menubar AND in the streams view. We
+    // need the icon on the streams view. Therefore, if the StdAction has no icon -  we create a new one and forward all signals...
+    if (StdActions::self()->searchAction->icon().isNull()) {
+        searchAction = new Action(Icon("edit-find"), StdActions::self()->searchAction->text(), this);
+        searchAction->setToolTip(StdActions::self()->searchAction->toolTip());
+        connect(searchAction, SIGNAL(triggered(bool)), StdActions::self()->searchAction, SIGNAL(triggered(bool)));
+        connect(searchAction, SIGNAL(triggered()), StdActions::self()->searchAction, SIGNAL(triggered()));
+        connect(ActionCollection::get(), SIGNAL(tooltipUpdated(QAction *)), SLOT(tooltipUpdated(QAction *)));
+    } else {
+        searchAction = StdActions::self()->searchAction;
+    }
+    #endif
     load();
+    #if defined ENABLE_MODEL_TEST
+    new ModelTest(this, this);
+    #endif
 }
 
 OnlineServicesModel::~OnlineServicesModel()
@@ -145,7 +148,7 @@ QVariant OnlineServicesModel::data(const QModelIndex &index, int role) const
             return MusicModel::data(index, role).toString()+"<br/>"+i18n("Last Search:%1", static_cast<OnlineService *>(item)->currentSearchString());
         }
         break;
-    case ItemView::Role_SubText:
+    case Cantata::Role_SubText:
         switch(item->itemType()) {
         case MusicLibraryItem::Type_Root: {
             OnlineService *srv=static_cast<OnlineService *>(item);
@@ -163,18 +166,10 @@ QVariant OnlineServicesModel::data(const QModelIndex &index, int role) const
                 return i18n("Use search to locate tracks");
             }
             if (srv->isSearchBased()) {
-                #ifdef ENABLE_KDE_SUPPORT
-                return i18np("1 Track", "%1 Tracks", item->childCount());
-                #else
-                return QTP_TRACKS_STR(item->childCount());
-                #endif
+                return Plurals::tracks(item->childCount());
             }
             if (srv->isPodcasts()) {
-                #ifdef ENABLE_KDE_SUPPORT
-                return i18np("1 Podcast", "%1 Podcasts", item->childCount());
-                #else
-                return QTP_PODCASTS_STR(item->childCount());
-                #endif
+                return Plurals::podcasts(item->childCount());
             }
             break;
         }
@@ -191,7 +186,7 @@ QVariant OnlineServicesModel::data(const QModelIndex &index, int role) const
             break;
         }
         break;
-    case ItemView::Role_Actions: {
+    case Cantata::Role_Actions: {
         QList<Action *> actions;
         if (MusicLibraryItem::Type_Root==item->itemType()) {
             OnlineService *srv=static_cast<OnlineService *>(item);
@@ -204,7 +199,7 @@ QVariant OnlineServicesModel::data(const QModelIndex &index, int role) const
                 actions << refreshSubscriptionAction;
             }
             if (srv->isSearchBased() || srv->isLoaded()) {
-                actions << StdActions::self()->searchAction;
+                actions << searchAction;
             }
             if (srv->canSubscribe()) {
                 actions << subscribeAction;
@@ -238,6 +233,7 @@ QVariant OnlineServicesModel::data(const QModelIndex &index, int role) const
             return f;
         }
         break;
+    case Cantata::Role_MainText:
     case Qt::DisplayRole:
         if (MusicLibraryItem::Type_Podcast==item->itemType() && static_cast<MusicLibraryItemPodcast *>(item)->unplayedEpisodes()) {
             return i18nc("podcast name (num unplayed episodes)", "%1 (%2)", item->data(), static_cast<MusicLibraryItemPodcast *>(item)->unplayedEpisodes());
@@ -586,25 +582,15 @@ void OnlineServicesModel::imageDownloaded()
             emit dataChanged(idx, idx);
 
         }
-    } else if (song.album.isEmpty()) {
-        if (srv->useArtistImages() && srv->id()==id) {
-            MusicLibraryItemArtist *artistItem = srv->artist(song, false);
-            if (artistItem && artistItem->setCover(img)) {
-                QModelIndex idx=index(artistItem->row(), 0, index(row(srv), 0, QModelIndex()));
+    } else if (!song.album.isEmpty() && srv->useAlbumImages() && !srv->isPodcasts() && srv->id()==id) {
+        MusicLibraryItemArtist *artistItem = srv->artist(song, false);
+        if (artistItem) {
+            MusicLibraryItemAlbum *albumItem = artistItem->album(song, false);
+            if (albumItem && albumItem->saveToCache(img)) {
+                QModelIndex idx=index(albumItem->row(), 0, index(artistItem->row(), 0, index(row(srv), 0, QModelIndex())));
                 emit dataChanged(idx, idx);
             }
         }
-    } else {
-        if (srv->useAlbumImages() && !srv->isPodcasts() && srv->id()==id) {
-            MusicLibraryItemArtist *artistItem = srv->artist(song, false);
-            if (artistItem) {
-                MusicLibraryItemAlbum *albumItem = artistItem->album(song, false);
-                if (albumItem && albumItem->setCover(img)) {
-                    QModelIndex idx=index(albumItem->row(), 0, index(artistItem->row(), 0, index(row(srv), 0, QModelIndex())));
-                    emit dataChanged(idx, idx);
-                }
-            }
-        }
     }
 
     int maxSize=j->property(constMaxSizeProperty).toInt();
@@ -638,6 +624,18 @@ void OnlineServicesModel::imageDownloaded()
     }
 }
 
+// Required due to icon missing for StdActions::searchAction for Mac/Unity... See note in constructor above.
+void OnlineServicesModel::tooltipUpdated(QAction *act)
+{
+    #ifdef ENABLE_STREAMS
+    Q_UNUSED(act)
+    #else
+    if (act!=searchAction && act==StdActions::self()->searchAction) {
+        searchAction->setToolTip(StdActions::self()->searchAction->toolTip());
+    }
+    #endif
+}
+
 void OnlineServicesModel::save()
 {
     QStringList disabled;
