@@ -2,6 +2,7 @@
  * Cantata
  *
  * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2014 Niklas Wenzel <nikwen.developer@gmail.com>
  *
  */
 /*
@@ -24,25 +25,29 @@
  * along with QtMPC.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "localize.h"
+#include "support/localize.h"
 #include "playqueuemodel.h"
-#include "groupedview.h"
-#include "playqueueview.h"
-#include "mpdconnection.h"
-#include "mpdparseutils.h"
-#include "mpdstats.h"
-#include "mpdstatus.h"
-#include "streamfetcher.h"
+#include "mpd/mpdconnection.h"
+#include "mpd/mpdparseutils.h"
+#include "mpd/mpdstats.h"
+#include "mpd/mpdstatus.h"
+#include "streams/streamfetcher.h"
 #include "streamsmodel.h"
-#include "httpserver.h"
-#include "settings.h"
-#include "icon.h"
-#include "utils.h"
+#include "http/httpserver.h"
+#include "gui/settings.h"
+#include "support/icon.h"
+#include "support/utils.h"
 #include "config.h"
-#include "action.h"
-#include "actioncollection.h"
+#include "support/action.h"
+#include "support/actioncollection.h"
+#include "gui/covers.h"
+#include "widgets/groupedview.h"
+#include "roles.h"
 #ifdef ENABLE_DEVICES_SUPPORT
 #include "devicesmodel.h"
+#if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
+#include "devices/audiocddevice.h"
+#endif
 #endif
 #include <QPalette>
 #include <QFont>
@@ -53,15 +58,24 @@
 #include <QUrl>
 #include <QTimer>
 #include <QApplication>
+#include <QMenu>
 
 #if defined ENABLE_MODEL_TEST
 #include "modeltest.h"
 #endif
 
+#ifndef ENABLE_UBUNTU
 const QLatin1String PlayQueueModel::constMoveMimeType("cantata/move");
 const QLatin1String PlayQueueModel::constFileNameMimeType("cantata/filename");
 const QLatin1String PlayQueueModel::constUriMimeType("text/uri-list");
 
+static const char * constSortByKey="sort-by";
+static const QLatin1String constSortByArtistKey("artist");
+static const QLatin1String constSortByAlbumArtistKey("albumartist");
+static const QLatin1String constSortByAlbumKey("album");
+static const QLatin1String constSortByGenreKey("genre");
+static const QLatin1String constSortByYearKey("year");
+
 static bool checkExtension(const QString &file)
 {
     static QSet<QString> constExtensions=QSet<QString>()
@@ -75,6 +89,30 @@ static bool checkExtension(const QString &file)
     return pos>1 ? constExtensions.contains(file.mid(pos+1).toLower()) : false;
 }
 
+static QStringList parseUrls(const QStringList &urls, bool percentEncoded)
+{
+    QStringList useable;
+    foreach (const QString &path, urls) {
+        QUrl u=percentEncoded ? QUrl::fromPercentEncoding(path.toUtf8()) : QUrl(path);
+        #if defined ENABLE_DEVICES_SUPPORT && (defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND)
+        QString cdDevice=AudioCdDevice::getDevice(u);
+        if (!cdDevice.isEmpty()) {
+            DevicesModel::self()->playCd(cdDevice);
+        } else
+        #endif
+        if (QLatin1String("http")==u.scheme()) {
+            useable.append(u.toString());
+        } else if ((u.scheme().isEmpty() || QLatin1String("file")==u.scheme()) && checkExtension(u.path())) {
+            if (!HttpServer::self()->forceUsage() && MPDConnection::self()->getDetails().isLocal()  && !u.path().startsWith(QLatin1String("/media/"))) {
+                useable.append(QLatin1String("file://")+u.path());
+            } else if (HttpServer::self()->isAlive()) {
+                useable.append(HttpServer::self()->encodeUrl(u.path()));
+            }
+        }
+    }
+    return useable;
+}
+
 void PlayQueueModel::encode(QMimeData &mimeData, const QString &mime, const QStringList &values)
 {
     QByteArray encodedData;
@@ -87,6 +125,16 @@ void PlayQueueModel::encode(QMimeData &mimeData, const QString &mime, const QStr
     mimeData.setData(mime, encodedData);
 }
 
+void PlayQueueModel::encode(QMimeData &mimeData, const QString &mime, const QList<quint32> &values)
+{
+    QByteArray encodedData;
+    QDataStream stream(&encodedData, QIODevice::WriteOnly);
+    foreach (quint32 v, values) {
+        stream << v;
+    }
+    mimeData.setData(mime, encodedData);
+}
+
 QStringList PlayQueueModel::decode(const QMimeData &mimeData, const QString &mime)
 {
     QByteArray encodedData=mimeData.data(mime);
@@ -99,33 +147,57 @@ QStringList PlayQueueModel::decode(const QMimeData &mimeData, const QString &mim
     return rv;
 }
 
+QList<quint32> PlayQueueModel::decodeInts(const QMimeData &mimeData, const QString &mime)
+{
+    QByteArray encodedData=mimeData.data(mime);
+    QDataStream stream(&encodedData, QIODevice::ReadOnly);
+    QList<quint32> rv;
+
+    while (!stream.atEnd()) {
+        quint32 v;
+        stream >> v;
+        rv.append(v);
+    }
+    return rv;
+}
+
 QString PlayQueueModel::headerText(int col)
 {
     switch (col) {
-    case COL_TITLE:  return i18n("Title");
-    case COL_ARTIST: return i18n("Artist");
-    case COL_ALBUM:  return i18n("Album");
-    case COL_TRACK:  return i18n("Track");
-    case COL_LENGTH: return i18n("Length");
-    case COL_DISC:   return i18n("Disc");
-    case COL_YEAR:   return i18n("Year");
-    case COL_GENRE:  return i18n("Genre");
-    case COL_PRIO:   return i18n("Priority");
-    default:         return QString();
+    case COL_TITLE:     return i18n("Title");
+    case COL_ARTIST:    return i18n("Artist");
+    case COL_ALBUM:     return i18n("Album");
+    case COL_TRACK:     return i18n("Track");
+    case COL_LENGTH:    return i18n("Length");
+    case COL_DISC:      return i18n("Disc");
+    case COL_YEAR:      return i18n("Year");
+    case COL_GENRE:     return i18n("Genre");
+    case COL_PRIO:      return i18n("Priority");
+    case COL_COMPOSER:  return i18n("Composer");
+    case COL_PERFORMER: return i18n("Performer");
+    default:            return QString();
     }
 }
+#endif
 
 PlayQueueModel::PlayQueueModel(QObject *parent)
     : QAbstractItemModel(parent)
     , currentSongId(-1)
     , currentSongRowNum(-1)
+    , time(0)
     , mpdState(MPDState_Inactive)
-    , dropAdjust(0)
     , stopAfterCurrent(false)
     , stopAfterTrackId(-1)
+    #ifdef ENABLE_UBUNTU
+    , undoLimit(0)
+    #else
     , undoLimit(Settings::self()->undoSteps())
+    #endif
     , undoEnabled(undoLimit>0)
     , lastCommand(Cmd_Other)
+    #ifndef ENABLE_UBUNTU
+    , dropAdjust(0)
+    #endif
 {
     fetcher=new StreamFetcher(this);
     connect(this, SIGNAL(modelReset()), this, SLOT(stats()));
@@ -145,7 +217,7 @@ PlayQueueModel::PlayQueueModel(QObject *parent)
     connect(this, SIGNAL(clearEntries()), MPDConnection::self(), SLOT(clear()));
     connect(this, SIGNAL(addAndPlay(QString)), MPDConnection::self(), SLOT(addAndPlay(QString)));
     connect(this, SIGNAL(startPlayingSongId(qint32)), MPDConnection::self(), SLOT(startPlayingSongId(qint32)));
-    #ifdef ENABLE_DEVICES_SUPPORT
+    #ifdef ENABLE_DEVICES_SUPPORT //TODO: Problems here with devices support!!!
     connect(DevicesModel::self(), SIGNAL(invalid(QList<Song>)), SLOT(remove(QList<Song>)));
     connect(DevicesModel::self(), SIGNAL(updatedDetails(QList<Song>)), SLOT(updateDetails(QList<Song>)));
     #endif
@@ -153,17 +225,47 @@ PlayQueueModel::PlayQueueModel(QObject *parent)
     new ModelTest(this, this);
     #endif
 
+    #ifdef ENABLE_UBUNTU
+    connect(Covers::self(), SIGNAL(cover(const Song &, const QImage &, const QString &)), this, 
+            SLOT(setCover(const Song &, const QImage &, const QString &)));
+    #else
+
+    #ifdef ENABLE_KDE_SUPPORT
+    bool menuIcons=true;
+    #else
+    bool menuIcons=!QCoreApplication::testAttribute(Qt::AA_DontShowIconsInMenus);
+    #endif
+
     removeDuplicatesAction=new Action(i18n("Remove Duplicates"), this);
     removeDuplicatesAction->setEnabled(false);
-    undoAction=ActionCollection::get()->createAction("playqueue-undo", i18n("Undo"), "edit-undo");
+    undoAction=ActionCollection::get()->createAction("playqueue-undo", i18n("Undo"), menuIcons ? "edit-undo" : "");
     undoAction->setShortcut(Qt::ControlModifier+Qt::Key_Z);
-    redoAction=ActionCollection::get()->createAction("playqueue-redo", i18n("Redo"), "edit-redo");
+    redoAction=ActionCollection::get()->createAction("playqueue-redo", i18n("Redo"), menuIcons ? "edit-redo" : "");
     redoAction->setShortcut(Qt::ControlModifier+Qt::ShiftModifier+Qt::Key_Z);
     connect(undoAction, SIGNAL(triggered(bool)), this, SLOT(undo()));
     connect(redoAction, SIGNAL(triggered(bool)), this, SLOT(redo()));
     connect(removeDuplicatesAction, SIGNAL(triggered(bool)), this, SLOT(removeDuplicates()));
 
+    shuffleAction=new Action(i18n("Shuffle"), this);
+    shuffleAction->setMenu(new QMenu(0));
+    Action *shuffleTracksAction = new Action(i18n("Tracks"), shuffleAction);
+    Action *shuffleAlbumsAction = new Action(i18n("Albums"), shuffleAction);
+    connect(shuffleTracksAction, SIGNAL(triggered(bool)), MPDConnection::self(), SLOT(shuffle()));
+    connect(shuffleAlbumsAction, SIGNAL(triggered(bool)), this, SLOT(shuffleAlbums()));
+    shuffleAction->menu()->addAction(shuffleTracksAction);
+    shuffleAction->menu()->addAction(shuffleAlbumsAction);
+
+    sortAction=new Action(i18n("Sort By"), this);
+    sortAction->setMenu(new QMenu(0));
+    addSortAction(i18n("Artist"), constSortByArtistKey);
+    addSortAction(i18n("Album Artist"), constSortByAlbumArtistKey);
+    addSortAction(i18n("Album"), constSortByAlbumKey);
+    addSortAction(i18n("Genre"), constSortByGenreKey);
+    addSortAction(i18n("Year"), constSortByYearKey);
     controlActions();
+    shuffleAction->setEnabled(false);
+    sortAction->setEnabled(false);
+    #endif
 }
 
 PlayQueueModel::~PlayQueueModel()
@@ -181,6 +283,7 @@ QModelIndex PlayQueueModel::parent(const QModelIndex &idx) const
     return QModelIndex();
 }
 
+#ifndef ENABLE_UBUNTU
 QVariant PlayQueueModel::headerData(int section, Qt::Orientation orientation, int role) const
 {
     if (Qt::Horizontal==orientation) {
@@ -193,6 +296,8 @@ QVariant PlayQueueModel::headerData(int section, Qt::Orientation orientation, in
             case COL_ARTIST:
             case COL_ALBUM:
             case COL_GENRE:
+            case COL_COMPOSER:
+            case COL_PERFORMER:
             default:
                 return int(Qt::AlignVCenter|Qt::AlignLeft);
             case COL_TRACK:
@@ -202,19 +307,21 @@ QVariant PlayQueueModel::headerData(int section, Qt::Orientation orientation, in
             case COL_PRIO:
                 return int(Qt::AlignVCenter|Qt::AlignRight);
             }
-        case TableView::Role_Hideable:
-            return COL_YEAR==section || COL_DISC==section || COL_GENRE==section || COL_PRIO==section? true : false;
-        case TableView::Role_Width:
+        case Cantata::Role_Hideable:
+            return COL_YEAR==section || COL_DISC==section || COL_GENRE==section || COL_PRIO==section || COL_COMPOSER==section || COL_PERFORMER==section ? true : false;
+        case Cantata::Role_Width:
             switch (section) {
-            case COL_TRACK:  return 0.075;
-            case COL_DISC:   return 0.03;
-            case COL_TITLE:  return 0.3;
-            case COL_ARTIST: return 0.27;
-            case COL_ALBUM:  return 0.27;
-            case COL_LENGTH: return 0.05;
-            case COL_YEAR:   return 0.05;
-            case COL_GENRE:  return 0.1;
-            case COL_PRIO:   return 0.015;
+            case COL_TRACK:     return 0.075;
+            case COL_DISC:      return 0.03;
+            case COL_TITLE:     return 0.3;
+            case COL_ARTIST:    return 0.27;
+            case COL_ALBUM:     return 0.27;
+            case COL_LENGTH:    return 0.05;
+            case COL_YEAR:      return 0.05;
+            case COL_GENRE:     return 0.1;
+            case COL_PRIO:      return 0.015;
+            case COL_COMPOSER:  return 0.2;
+            case COL_PERFORMER: return 0.2;
             }
         default:
             break;
@@ -223,6 +330,7 @@ QVariant PlayQueueModel::headerData(int section, Qt::Orientation orientation, in
 
     return QVariant();
 }
+#endif
 
 int PlayQueueModel::rowCount(const QModelIndex &idx) const
 {
@@ -244,6 +352,21 @@ static QString basicPath(const Song &song)
     return -1==marker ? path : path.left(marker);
 }
 
+#ifdef ENABLE_UBUNTU
+static const QString constDefaultCover=QLatin1String("qrc:/album.svg");
+
+//Expose role names, so that they can be accessed via QML
+QHash<int, QByteArray> PlayQueueModel::roleNames() const
+{
+    QHash<int, QByteArray> roles;
+    roles[Cantata::Role_MainText] = "mainText";
+    roles[Cantata::Role_SubText] = "subText";
+    roles[Cantata::Role_Image] = "image";
+    roles[Cantata::Role_Time] = "time";
+    return roles;
+}
+#endif
+
 QVariant PlayQueueModel::data(const QModelIndex &index, int role) const
 {
     if (Qt::SizeHintRole!=role && (!index.isValid() || index.row() >= songs.size())) {
@@ -260,20 +383,48 @@ QVariant PlayQueueModel::data(const QModelIndex &index, int role) const
 //     }
 
     switch (role) {
-    case GroupedView::Role_IsCollection:
+    case Cantata::Role_MainText: {
+        const Song &s=songs.at(index.row());
+        return s.title.isEmpty() ? s.file : s.trackAndTitleStr(false);
+    }
+    case Cantata::Role_SubText: {
+        const Song &s=songs.at(index.row());
+        return s.artist+QLatin1String(" - ")+s.displayAlbum();
+    }
+    case Cantata::Role_Time: {
+        const Song &s=songs.at(index.row());
+        return s.time>0 ? Utils::formatTime(s.time) : QLatin1String("");
+    }
+    #ifdef ENABLE_UBUNTU
+    case Cantata::Role_Image: {
+        Song s=songs.at(index.row());
+        QMap<quint16, QString>::ConstIterator it=covers.find(s.key);
+        if (it!=covers.constEnd()) {
+            return it.value().isEmpty() ? constDefaultCover : it.value();
+        }
+        QString fileName=Covers::self()->requestImage(s).fileName;
+        covers.insert(s.key, fileName);
+        if (fileName.isEmpty()) {
+            coverRequests.insert(s.key, s);
+            return constDefaultCover;
+        }
+        return fileName;
+    }
+    #endif
+    case Cantata::Role_IsCollection:
         return false;
-    case GroupedView::Role_CollectionId:
+    case Cantata::Role_CollectionId:
         return 0;
-    case GroupedView::Role_Key:
+    case Cantata::Role_Key:
         return songs.at(index.row()).key;
-    case GroupedView::Role_Id:
+    case Cantata::Role_Id:
         return songs.at(index.row()).id;
-    case GroupedView::Role_Song: {
+    case Cantata::Role_Song: {
         QVariant var;
         var.setValue<Song>(songs.at(index.row()));
         return var;
     }
-    case GroupedView::Role_AlbumDuration: {
+    case Cantata::Role_AlbumDuration: {
         const Song &first = songs.at(index.row());
         quint32 d=first.time;
         for (int i=index.row()+1; i<songs.count(); ++i) {
@@ -294,7 +445,7 @@ QVariant PlayQueueModel::data(const QModelIndex &index, int role) const
         }
         return d;
     }
-    case GroupedView::Role_SongCount: {
+    case Cantata::Role_SongCount: {
         const Song &first = songs.at(index.row());
         quint32 count=1;
         for (int i=index.row()+1; i<songs.count(); ++i) {
@@ -315,7 +466,7 @@ QVariant PlayQueueModel::data(const QModelIndex &index, int role) const
         }
         return count;
     }
-    case GroupedView::Role_CurrentStatus: {
+    case Cantata::Role_CurrentStatus: {
         quint16 key=songs.at(index.row()).key;
         for (int i=index.row()+1; i<songs.count(); ++i) {
             const Song &s=songs.at(i);
@@ -335,7 +486,7 @@ QVariant PlayQueueModel::data(const QModelIndex &index, int role) const
         }
         return QVariant();
     }
-    case GroupedView::Role_Status: {
+    case Cantata::Role_Status: {
         qint32 id=songs.at(index.row()).id;
         if (id==currentSongId) {
             switch (mpdState) {
@@ -383,17 +534,24 @@ QVariant PlayQueueModel::data(const QModelIndex &index, int role) const
         case COL_ARTIST:
             return song.artist.isEmpty() ? Song::unknown() : song.artist;
         case COL_ALBUM:
-            return song.album.isEmpty() && !song.name.isEmpty() && song.isStream() ? song.name : song.album;
+            if (song.isStream() && song.album.isEmpty()) {
+                QString n=song.name();
+                if (!n.isEmpty()) {
+                    return n;
+                }
+            }
+            return song.album;
         case COL_TRACK:
             if (song.track <= 0) {
                 return QVariant();
             }
             return song.track;
         case COL_LENGTH:
-            return Song::formattedTime(song.time);
+            return Utils::formatTime(song.time);
         case COL_DISC:
-            if (song.disc <= 0)
+            if (song.disc <= 0) {
                 return QVariant();
+            }
             return song.disc;
         case COL_YEAR:
             if (song.year <= 0) {
@@ -401,9 +559,13 @@ QVariant PlayQueueModel::data(const QModelIndex &index, int role) const
             }
             return song.year;
         case COL_GENRE:
-            return song.genre;
+            return song.displayGenre();
         case COL_PRIO:
             return song.priority;
+        case COL_COMPOSER:
+            return song.composer();
+        case COL_PERFORMER:
+            return song.performer();
         default:
             break;
         }
@@ -414,12 +576,7 @@ QVariant PlayQueueModel::data(const QModelIndex &index, int role) const
         if (s.album.isEmpty() && s.isStream()) {
             return basicPath(s);
         } else {
-            return s.albumArtist()+QLatin1String("<br/>")+
-                   s.album+(s.year>0 ? (QLatin1String(" (")+QString::number(s.year)+QChar(')')) : QString())+QLatin1String("<br/>")+
-                   s.trackAndTitleStr(Song::isVariousArtists(s.albumArtist()))+QLatin1String("<br/>")+
-                   Song::formattedTime(s.time)+QLatin1String("<br/>")+
-                   (s.priority>0 ? i18n("<b>(Priority: %1)</b>", s.priority)+QLatin1String("<br/>") : QString())+
-                   QLatin1String("<small><i>")+basicPath(s)+QLatin1String("</i></small>");
+            return s.toolTip();
         }
     }
     case Qt::TextAlignmentRole:
@@ -428,6 +585,8 @@ QVariant PlayQueueModel::data(const QModelIndex &index, int role) const
         case COL_ARTIST:
         case COL_ALBUM:
         case COL_GENRE:
+        case COL_COMPOSER:
+        case COL_PERFORMER:
         default:
             return int(Qt::AlignVCenter|Qt::AlignLeft);
         case COL_TRACK:
@@ -437,7 +596,8 @@ QVariant PlayQueueModel::data(const QModelIndex &index, int role) const
         case COL_PRIO:
             return int(Qt::AlignVCenter|Qt::AlignRight);
         }
-    case PlayQueueView::Role_Decoration: {
+    #ifndef ENABLE_UBUNTU
+    case Cantata::Role_Decoration: {
         qint32 id=songs.at(index.row()).id;
         if (id==currentSongId) {
             switch (mpdState) {
@@ -451,6 +611,7 @@ QVariant PlayQueueModel::data(const QModelIndex &index, int role) const
         }
         break;
     }
+    #endif
     default:
         break;
     }
@@ -458,15 +619,17 @@ QVariant PlayQueueModel::data(const QModelIndex &index, int role) const
     return QVariant();
 }
 
+#ifndef ENABLE_UBUNTU
 bool PlayQueueModel::setData(const QModelIndex &index, const QVariant &value, int role)
 {
-    if (GroupedView::Role_DropAdjust==role) {
+    if (Cantata::Role_DropAdjust==role) {
         dropAdjust=value.toUInt();
         return true;
     } else {
         return QAbstractItemModel::setData(index, value, role);
     }
 }
+#endif
 
 Qt::DropActions PlayQueueModel::supportedDropActions() const
 {
@@ -481,6 +644,7 @@ Qt::ItemFlags PlayQueueModel::flags(const QModelIndex &index) const
     return Qt::ItemIsDropEnabled;
 }
 
+#ifndef ENABLE_UBUNTU
 /**
  * @return A QStringList with the mimetypes we support
  */
@@ -498,28 +662,18 @@ QStringList PlayQueueModel::mimeTypes() const
 QMimeData *PlayQueueModel::mimeData(const QModelIndexList &indexes) const
 {
     QMimeData *mimeData = new QMimeData();
-    QStringList positions;
+    QList<quint32> positions;
     QStringList filenames;
-    /*
-     * Loop over all our indexes. However we have rows*columns indexes
-     * We pack per row so ingore the columns
-     */
-    QList<int> rows;
-    foreach(QModelIndex index, indexes) {
-        if (index.isValid()) {
-            if (rows.contains(index.row())) {
-                continue;
-            }
 
-            positions.append(QString::number(index.row())); // getPosByRow(index.row())));
-            rows.append(index.row());
-            filenames.append(songs.at(index.row()).file);
+    foreach(QModelIndex index, indexes) {
+        if (index.isValid() && 0==index.column()) {
+            positions.append(index.row());
+            filenames.append(static_cast<Song *>(index.internalPointer())->file);
         }
     }
 
     encode(*mimeData, constMoveMimeType, positions);
     encode(*mimeData, constFileNameMimeType, filenames);
-
     return mimeData;
 }
 
@@ -542,19 +696,11 @@ bool PlayQueueModel::dropMimeData(const QMimeData *data,
     }
 
     row+=dropAdjust;
-    if (data->hasFormat(constMoveMimeType)) {
-        //Act on internal moves
-        QStringList positions=decode(*data, constMoveMimeType);
-        QList<quint32> items;
-
-        foreach (const QString &s, positions) {
-            items.append(s.toUInt());
-        }
-
+    if (data->hasFormat(constMoveMimeType)) { //Act on internal moves
         if (row < 0) {
-            emit move(items, songs.size(), songs.size());
+            emit move(decodeInts(*data, constMoveMimeType), songs.size(), songs.size());
         } else {
-            emit move(items, row, songs.size());
+            emit move(decodeInts(*data, constMoveMimeType), row, songs.size());
         }
         return true;
     } else if (data->hasFormat(constFileNameMimeType)) {
@@ -562,25 +708,7 @@ bool PlayQueueModel::dropMimeData(const QMimeData *data,
         addItems(decode(*data, constFileNameMimeType), row, false, 0);
         return true;
     } else if(data->hasFormat(constUriMimeType)/* && MPDConnection::self()->getDetails().isLocal()*/) {
-        QStringList orig=decode(*data, constUriMimeType);
-        QStringList useable;
-
-        foreach (QString u, orig) {
-            if (u.startsWith(QLatin1String("http://"))) {
-                useable.append(u);
-            } else if (u.startsWith('/') || u.startsWith(QLatin1String("file://"))) {
-                if (u.startsWith(QLatin1String("file://"))) {
-                    u=u.mid(7);
-                }
-                if (checkExtension(u)) {
-                    if (!HttpServer::self()->forceUsage() && MPDConnection::self()->getDetails().isLocal() && !u.startsWith(QLatin1String("/media/"))) {
-                        useable.append(QLatin1String("file://")+QUrl::fromPercentEncoding(u.toUtf8()));
-                    } else if (HttpServer::self()->isAlive()) {
-                        useable.append(HttpServer::self()->encodeUrl(QUrl::fromPercentEncoding(u.toUtf8())));
-                    }
-                }
-            }
-        }
+        QStringList useable=parseUrls(decode(*data, constUriMimeType), true);
         if (useable.count()) {
             addItems(useable, row, false, 0);
             return true;
@@ -588,6 +716,19 @@ bool PlayQueueModel::dropMimeData(const QMimeData *data,
     }
     return false;
 }
+#endif
+
+void PlayQueueModel::load(const QStringList &urls)
+{
+    #ifdef ENABLE_UBUNTU
+    Q_UNUSED(urls)
+    #else
+    QStringList useable=parseUrls(urls, false);
+    if (useable.count()) {
+        addItems(useable, songs.count(), songs.isEmpty(), 0);
+    }
+    #endif
+}
 
 void PlayQueueModel::addItems(const QStringList &items, int row, bool replace, quint8 priority)
 {
@@ -724,6 +865,7 @@ void PlayQueueModel::clear()
     currentSongId=-1;
     currentSongRowNum=0;
     stopAfterTrackId=-1;
+    time=0;
     endResetModel();
 }
 
@@ -751,17 +893,27 @@ void PlayQueueModel::setState(MPDState st)
 // Update playqueue with contents returned from MPD.
 void PlayQueueModel::update(const QList<Song> &songList)
 {
+    currentSongRowNum=-1;
     if (songList.isEmpty()) {
-        Song::clearKeyStore(MPDParseUtils::PlayQueue);
+        Song::clearKeyStore(MPDParseUtils::Loc_PlayQueue);
+        #ifdef ENABLE_UBUNTU
+        covers.clear();
+        coverRequests.clear();
+        #endif
     }
 
+    #ifndef ENABLE_UBUNTU
     removeDuplicatesAction->setEnabled(songList.count()>1);
+    #endif
     QList<Song> prev;
     if (undoEnabled) {
         prev=songs;
     }
 
     QSet<qint32> newIds;
+    #ifdef ENABLE_UBUNTU
+    QSet<quint16> currentKeys;
+    #endif
     foreach (const Song &s, songList) {
         newIds.insert(s.id);
     }
@@ -774,8 +926,14 @@ void PlayQueueModel::update(const QList<Song> &songList)
         if (songList.isEmpty()) {
             stopAfterTrackId=-1;
         }
+        
+        #ifdef ENABLE_UBUNTU
+        foreach (const Song &s, songs) {
+            currentKeys.insert(s.key);
+        }
+        #endif
     } else {
-        quint32 time = 0;
+        time = 0;
 
         QSet<qint32> removed=ids-newIds;
         foreach (qint32 id, removed) {
@@ -797,24 +955,39 @@ void PlayQueueModel::update(const QList<Song> &songList)
                 if (-1==existingPos) {
                     beginInsertRows(QModelIndex(), i, i);
                     songs.insert(i, s);
+                    #ifdef ENABLE_UBUNTU
+                    currentKeys.insert(s.key);
+                    #endif
                     endInsertRows();
                 } else {
                     beginMoveRows(QModelIndex(), existingPos, existingPos, QModelIndex(), i>existingPos ? i+1 : i);
                     Song old=songs.takeAt(existingPos);
 //                     old.pos=s.pos;
                     songs.insert(i, isEmpty ? old : s);
+                    #ifdef ENABLE_UBUNTU
+                    currentKeys.insert(isEmpty ? old.key : s.key);
+                    #endif
                     endMoveRows();
                 }
             } else if (isEmpty) {
                 s=curentSongAtPos;
+                #ifdef ENABLE_UBUNTU
+                currentKeys.insert(s.key);
+                #endif
             } else {
                 s.key=curentSongAtPos.key;
+                #ifdef ENABLE_UBUNTU
+                currentKeys.insert(s.key);
+                #endif
                 songs.replace(i, s);
-                if (s.name!=curentSongAtPos.name || s.title!=curentSongAtPos.title || s.artist!=curentSongAtPos.artist) {
+                if (s.title!=curentSongAtPos.title || s.artist!=curentSongAtPos.artist || s.name()!=curentSongAtPos.name()) {
                     emit dataChanged(index(i, 0), index(i, columnCount(QModelIndex())-1));
                 }
             }
 
+            if (s.id==currentSongId) {
+                currentSongRowNum=i;
+            }
             time += s.time;
         }
 
@@ -835,6 +1008,18 @@ void PlayQueueModel::update(const QList<Song> &songList)
     }
 
     saveHistory(prev);
+    #ifdef ENABLE_UBUNTU
+    QSet<quint16> removedKeys=covers.keys().toSet()-currentKeys;
+    if (!removedKeys.isEmpty()) {
+        foreach (quint16 k, removedKeys) {
+            covers.remove(k);
+            coverRequests.remove(k);
+        }
+    }
+    #else
+    shuffleAction->setEnabled(songs.count()>1);
+    sortAction->setEnabled(songs.count()>1);
+    #endif
 }
 
 void PlayQueueModel::setStopAfterTrack(qint32 track)
@@ -918,7 +1103,9 @@ void PlayQueueModel::enableUndo(bool e)
         undoStack.clear();
         redoStack.clear();
     }
+    #ifndef ENABLE_UBUNTU
     controlActions();
+    #endif
 }
 
 static PlayQueueModel::UndoItem getState(const QList<Song> &songs)
@@ -997,10 +1184,13 @@ void PlayQueueModel::saveHistory(const QList<Song> &prevList)
         }
     }
 
+    #ifndef ENABLE_UBUNTU
     controlActions();
+    #endif
     lastCommand=Cmd_Other;
 }
 
+#ifndef ENABLE_UBUNTU
 void PlayQueueModel::controlActions()
 {
     undoAction->setEnabled(!undoStack.isEmpty());
@@ -1009,6 +1199,74 @@ void PlayQueueModel::controlActions()
     redoAction->setVisible(undoLimit>0);
 }
 
+void PlayQueueModel::addSortAction(const QString &name, const QString &key)
+{
+    Action *action=new Action(name, sortAction);
+    action->setProperty(constSortByKey, key);
+    sortAction->menu()->addAction(action);
+    connect(action, SIGNAL(triggered(bool)), SLOT(sortBy()));
+}
+
+static bool artistSort(const Song &s1, const Song &s2)
+{
+    int c=s1.artist.localeAwareCompare(s2.artist);
+    return c<0 || (c==0 && s1<s2);
+}
+
+static bool albumArtistSort(const Song &s1, const Song &s2)
+{
+    int c=s1.albumArtist().localeAwareCompare(s2.albumArtist());
+    return c<0 || (c==0 && s1<s2);
+}
+
+static bool albumSort(const Song &s1, const Song &s2)
+{
+    int c=s1.albumId().localeAwareCompare(s2.albumId());
+    return c<0 || (c==0 && s1<s2);
+}
+
+static bool genreSort(const Song &s1, const Song &s2)
+{
+    int c=s1.genre.localeAwareCompare(s2.genre);
+    return c<0 || (c==0 && s1<s2);
+}
+
+static bool yearSort(const Song &s1, const Song &s2)
+{
+    return s1.year<s2.year || (s1.year==s2.year && s1<s2);
+}
+#endif
+
+void PlayQueueModel::sortBy()
+{
+    #ifndef ENABLE_UBUNTU
+    Action *act=qobject_cast<Action *>(sender());
+    if (act) {
+        QString key=act->property(constSortByKey).toString();
+        QList<Song> copy=songs;
+        if (constSortByArtistKey==key) {
+            qSort(copy.begin(), copy.end(), artistSort);
+        } else if (constSortByAlbumArtistKey==key) {
+            qSort(copy.begin(), copy.end(), albumArtistSort);
+        } else if (constSortByAlbumKey==key) {
+            qSort(copy.begin(), copy.end(), albumSort);
+        } else if (constSortByGenreKey==key) {
+            qSort(copy.begin(), copy.end(), genreSort);
+        } else if (constSortByYearKey==key) {
+            qSort(copy.begin(), copy.end(), yearSort);
+        }
+        if (copy!=songs) {
+            QStringList files;
+            foreach (const Song &s, copy) {
+                files.append(s.file);
+            }
+
+            emit filesAdded(files, 0, 0, MPDState_Playing==MPDStatus::self()->state() ? MPDConnection::AddReplaceAndPlay : MPDConnection::AddAndReplace , 0);
+        }
+    }
+    #endif
+}
+
 void PlayQueueModel::removeDuplicates()
 {
     QMap<QString, QList<Song> > map;
@@ -1031,6 +1289,60 @@ void PlayQueueModel::removeDuplicates()
     }
 }
 
+void PlayQueueModel::setCover(const Song &song, const QImage &img, const QString &file)
+{
+    Q_UNUSED(img)
+    #ifdef ENABLE_UBUNTU
+    if (file.isEmpty() || coverRequests.isEmpty()) {
+        return;
+    }
+    
+    // In case of multi-disc albums we will have multiple 'keys' mapped to a single artist+album combo.
+    // Therefore, we need to find all keys that have this mapping.
+    //
+    // Also, library and albums models might have requested the cover at the same time as playqueue, in which
+    // case the incomming song would have no key set
+    QList<quint16> keys;
+    QMap<quint16, Song>::ConstIterator it=coverRequests.constBegin();
+    QMap<quint16, Song>::ConstIterator end=coverRequests.constEnd();
+    for (; it!=end; ++it) {
+        if (song.albumArtist()==it.value().albumArtist() && song.album==it.value().album) {
+            keys.append(it.key());
+        }
+    }
+
+    foreach (quint16 key, keys) {
+        coverRequests.remove(key);
+        QMap<quint16, QString>::ConstIterator it=covers.find(key);
+        if (it!=covers.end() && it.value().isEmpty()) {
+            coverRequests.remove(it.key());
+            covers[key]="file://"+file;
+            int start=-1;
+            int end=-1;
+            for (int i=0; i<songs.size(); ++i) {
+                if (songs.at(i).key==key) {
+                    if (-1==start) {
+                        start=end=i;
+                    } else {
+                        end++;
+                    }
+                } else if (-1!=end) {
+                    emit dataChanged(index(start, 0), index(end, columnCount(QModelIndex())-1));
+                    start=end=-1;
+                }
+            }
+            if (-1!=end) {
+                emit dataChanged(index(start, 0), index(end, columnCount(QModelIndex())-1));
+                start=end=-1;
+            }
+        }
+    }
+    #else
+    Q_UNUSED(song)
+    Q_UNUSED(file)
+    #endif
+}
+
 void PlayQueueModel::undo()
 {
     if (!undoEnabled || undoStack.isEmpty()) {
@@ -1063,8 +1375,7 @@ void PlayQueueModel::playSong(const QString &file)
 
 void PlayQueueModel::stats()
 {
-    quint32 time = 0;
-
+    time = 0;
     //Loop over all songs
     foreach(const Song &song, songs) {
         time += song.time;
@@ -1080,9 +1391,9 @@ void PlayQueueModel::cancelStreamFetch()
 
 void PlayQueueModel::shuffleAlbums()
 {
-    QMap<quint32, QStringList> albums;
+    QMap<quint32, QList<Song> > albums;
     foreach (const Song &s, songs) {
-        albums[s.key].append(s.file);
+        albums[s.key].append(s);
     }
 
     QList<quint32> keys=albums.keys();
@@ -1093,8 +1404,10 @@ void PlayQueueModel::shuffleAlbums()
     QStringList files;
     while (!keys.isEmpty()) {
         quint32 key=keys.takeAt(Utils::random(keys.count()));
-        foreach (const QString &file, albums[key]) {
-            files.append(file);
+        QList<Song> albumSongs=albums[key];
+        qSort(albumSongs);
+        foreach (const Song &song, albumSongs) {
+            files.append(song.file);
         }
     }
 
@@ -1149,9 +1462,9 @@ void PlayQueueModel::updateDetails(const QList<Song> &updated)
         if (songMap.contains(current.file)) {
             Song updatedSong=songMap[current.file];
             updatedSong.id=current.id;
-            updatedSong.setKey(MPDParseUtils::PlayQueue);
+            updatedSong.setKey(MPDParseUtils::Loc_PlayQueue);
 
-            if (updatedSong.name!=current.name || updatedSong.title!=current.title || updatedSong.artist!=current.artist) {
+            if (updatedSong.title!=current.title || updatedSong.artist!=current.artist || updatedSong.name()!=current.name()) {
                 songs.replace(i, updatedSong);
                 updatedRows.append(i);
                 if (currentSongId==current.id) {
