@@ -27,12 +27,11 @@
 #include <cmath>
 #include "config.h"
 #include "song.h"
-#include "musiclibraryitemalbum.h"
-#include "localize.h"
-#ifndef CANTATA_NO_SONG_TIME_FUNCTION
-#include "utils.h"
+#include "models/musiclibraryitemalbum.h"
+#include "support/localize.h"
+#if !defined CANTATA_NO_UI_FUNCTIONS && defined ENABLE_ONLINE_SERVICES
+#include "online/onlineservice.h"
 #endif
-#include <QMap>
 #include <QStringList>
 #include <QSet>
 #include <QChar>
@@ -40,7 +39,7 @@
 #include <QtAlgorithms>
 #include <QUrl>
 
-static const quint8 constOnlineDiscId=0xEE;
+//static const quint8 constOnlineDiscId=0xEE;
 
 const QString Song::constCddaProtocol=QLatin1String("cantata-cdda:/");
 const QString Song::constMopidyLocal=QLatin1String("local:track:");
@@ -66,7 +65,7 @@ void Song::initTranslations()
 // When displaying albums, we use the 1st track's year as the year of the album.
 // The map below stores the mapping from artist+album to year.
 // This way the grouped view can find this quickly...
-static QMap<QString, quint16> albumYears;
+static QHash<QString, quint16> albumYears;
 
 void Song::storeAlbumYear(const Song &s)
 {
@@ -75,7 +74,7 @@ void Song::storeAlbumYear(const Song &s)
 
 int Song::albumYear(const Song &s)
 {
-    QMap<QString, quint16>::ConstIterator it=albumYears.find(s.albumKey());
+    QHash<QString, quint16>::ConstIterator it=albumYears.find(s.albumKey());
     return it==albumYears.end() ? s.year : it.value();
 }
 
@@ -147,17 +146,16 @@ void Song::setUseComposer(bool u)
 const quint16 Song::constNullKey(0xFFFF);
 
 Song::Song()
-    : id(-1)
-//       , pos(0)
-      , disc(0)
-      , priority(0)
-      , time(0)
-      , track(0)
-      , year(0)
-      , type(Standard)
-      , guessed(false)
-      , size(0)
-      , key(constNullKey)
+    : disc(0)
+    , priority(0)
+    , time(0)
+    , track(0)
+    , year(0)
+    , type(Standard)
+    , guessed(false)
+    , id(-1)
+    , size(0)
+    , key(constNullKey)
 {
 }
 
@@ -169,7 +167,6 @@ Song & Song::operator=(const Song &s)
     album = s.album;
     artist = s.artist;
     albumartist = s.albumartist;
-    composer = s.composer;
     title = s.title;
     track = s.track;
 //     pos = s.pos;
@@ -177,11 +174,11 @@ Song & Song::operator=(const Song &s)
     priority = s.priority;
     year = s.year;
     genre = s.genre;
-    name = s.name;
     size = s.size;
     key = s.key;
     type = s.type;
     guessed = s.guessed;
+    extra = s.extra;
     return *this;
 }
 
@@ -211,7 +208,6 @@ int Song::compareTo(const Song &o) const
         }
 
         int compare=artistOrComposer().localeAwareCompare(o.artistOrComposer());
-
         if (0!=compare) {
             return compare;
         }
@@ -219,6 +215,10 @@ int Song::compareTo(const Song &o) const
         if (0!=compare) {
             return compare;
         }
+        compare=mbAlbumId().compare(o.mbAlbumId());
+        if (0!=compare) {
+            return compare;
+        }
         if (disc!=o.disc) {
             return disc<o.disc ? -1 : 1;
         }
@@ -232,7 +232,7 @@ int Song::compareTo(const Song &o) const
         if (0!=compare) {
             return compare;
         }
-        compare=name.compare(o.name);
+        compare=name().compare(o.name());
         if (0!=compare) {
             return compare;
         }
@@ -249,7 +249,7 @@ int Song::compareTo(const Song &o) const
 
 bool Song::isEmpty() const
 {
-    return (artist.isEmpty() && album.isEmpty() && title.isEmpty() && name.isEmpty()) || file.isEmpty();
+    return (artist.isEmpty() && album.isEmpty() && title.isEmpty() && name().isEmpty()) || file.isEmpty();
 }
 
 void Song::guessTags()
@@ -275,7 +275,7 @@ void Song::guessTags()
 
         if (!title.isEmpty()) {
             int dot=title.lastIndexOf('.');
-            if (dot==title.length()-4) {
+            if (dot>0 && dot<title.length()-2) {
                 title=title.left(dot);
             }
             static const QSet<QChar> constSeparators=QSet<QChar>() << QLatin1Char(' ') << QLatin1Char('-') << QLatin1Char('_') << QLatin1Char('.');
@@ -328,26 +328,49 @@ struct KeyStore
 {
     KeyStore() : currentKey(0) { }
     quint16 currentKey;
-    QMap<QString, quint16> keys;
+    QHash<QString, quint16> keys;
 };
 
-static QMap<int, KeyStore> storeMap;
+static QHash<int, KeyStore> storeMap;
 
 void Song::clearKeyStore(int location)
 {
     storeMap.remove(location);
 }
 
+QString Song::displayAlbum(const QString &albumName, quint16 albumYear)
+{
+    return albumYear>0 ? albumName+QLatin1String(" (")+QString::number(albumYear)+QLatin1Char(')') : albumName;
+}
+
+QString Song::combineGenres(const QSet<QString> &genres)
+{
+    if (1==genres.size()) {
+        return *(genres.begin());
+    }
+
+    QStringList list=genres.toList();
+    qSort(list);
+    QString g;
+    foreach (const QString &e, list) {
+        if (!g.isEmpty()) {
+            g+=constGenreSep;
+        }
+        g+=e;
+    }
+    return g;
+}
+
 void Song::setKey(int location)
 {
-    if (isStream() && !isCantataStream()) {
+    if (isStandardStream()) {
         key=0;
         return;
     }
 
     KeyStore &store=storeMap[location];
     QString songKey(albumKey());
-    QMap<QString, quint16>::ConstIterator it=store.keys.find(songKey);
+    QHash<QString, quint16>::ConstIterator it=store.keys.find(songKey);
     if (it!=store.keys.end()) {
         key=it.value();
     } else {
@@ -375,51 +398,36 @@ void Song::clear()
     disc = 0;
     year = 0;
     genre.clear();
-    name.clear();
     size = 0;
+    extra.clear();
     type = Standard;
 }
 
-#ifndef CANTATA_NO_SONG_TIME_FUNCTION
-QString Song::formattedTime(quint32 seconds, bool zeroIsUnknown)
+const QLatin1Char Song::constGenreSep(';');
+
+void Song::addGenre(const QString &g)
 {
-    if (0==seconds && zeroIsUnknown) {
-        return unknownStr;
+    if (g.isEmpty()) {
+        return;
     }
-
-    static const quint32 constHour=60*60;
-    if (seconds>constHour) {
-        return Utils::formatDuration(seconds);
+    if (!genre.isEmpty()) {
+        genre+=constGenreSep;
     }
-
-    QString result(QString::number(floor(seconds / 60.0))+QChar(':'));
-    if (seconds % 60 < 10) {
-        result += "0";
-    }
-    return result+QString::number(seconds % 60);
+    genre+=g.trimmed();
 }
-#endif
 
-/*
- * Genarate a string with song info.
- * Currently in this format:
- * artist - [album -][#.] song
- */
-QString Song::format()
+QStringList Song::genres() const
 {
-    QString s = artist + " - ";
+    return genre.split(constGenreSep, QString::SkipEmptyParts);
+}
 
-    if (!album.isEmpty()) {
-        s += album + " - ";
+void Song::orderGenres()
+{
+    QStringList g=genres();
+    if (g.count()>1) {
+        qSort(g);
+        genre=g.join(QString()+constGenreSep);
     }
-
-    if (track != 0) {
-        s += QString::number(track) + ". ";
-    }
-
-    s += title;
-
-    return s;
 }
 
 QString Song::entryName() const
@@ -433,23 +441,46 @@ QString Song::entryName() const
 
 QString Song::artistOrComposer() const
 {
-    return useComposerIfSet && !composer.isEmpty() ? composer : albumArtist();
+    if (useComposerIfSet){
+        QString c=composer();
+        if (!c.isEmpty()) {
+            return c;
+        }
+    }
+
+    return albumArtist();
 }
 
 QString Song::albumName() const
 {
-    return useComposerIfSet && !composer.isEmpty() && composer!=albumArtist()
-            ? album+QLatin1String(" (")+albumArtist()+QLatin1Char(')')
-            : album;
+    if (useComposerIfSet) {
+        QString c=composer();
+        if (!c.isEmpty() && c!=albumArtist()) {
+            return album+QLatin1String(" (")+albumArtist()+QLatin1Char(')');
+        }
+    }
+    return album;
+}
+
+QString Song::albumId() const
+{
+    QString mb=mbAlbumId();
+    return mb.isEmpty() ? album : mb;
 }
 
 QString Song::artistSong() const
 {
-    return artist+QLatin1String(" - ")+title;
+    //return artist+QLatin1String(" - ")+title;
+    return title+QLatin1String(" - ")+artist;
 }
 
-QString Song::trackAndTitleStr(bool addArtist) const
+QString Song::trackAndTitleStr(bool showArtistIfDifferent) const
 {
+    #if !defined CANTATA_NO_UI_FUNCTIONS && defined ENABLE_ONLINE_SERVICES
+    if ((OnlineSvrTrack==type || Song::CantataStream) && OnlineService::showLogoAsCover(*this)) {
+        return artistSong();
+    }
+    #endif
 //    if (isFromOnlineService()) {
 //        return (disc>0 && disc!=constOnlineDiscId ? (QString::number(disc)+QLatin1Char('.')) : QString())+
 //               (track>0 ? (track>9 ? QString::number(track) : (QLatin1Char('0')+QString::number(track))) : QString())+
@@ -457,7 +488,61 @@ QString Song::trackAndTitleStr(bool addArtist) const
 //    }
     return //(disc>0 ? (QString::number(disc)+QLatin1Char('.')) : QString())+
            (track>0 ? (track>9 ? QString::number(track) : (QLatin1Char('0')+QString::number(track))) : QString())+
-           QLatin1Char(' ')+(addArtist ? artistSong() : title);
+           QLatin1Char(' ')+(showArtistIfDifferent && diffArtist() ? artistSong() : title);
+}
+
+#ifndef CANTATA_NO_UI_FUNCTIONS
+static void addField(const QString &name, const QString &val, QString &tt)
+{
+    if (!val.isEmpty()) {
+        tt+=QString("<tr><td align=\"right\"><b>%1:&nbsp;&nbsp;</b></td><td>%2</td></tr>").arg(name).arg(val);
+    }
+}
+#endif
+
+QString Song::toolTip() const
+{
+    #ifdef CANTATA_NO_UI_FUNCTIONS
+    return QString();
+    #else
+    QString toolTip=QLatin1String("<table>");
+    addField(i18n("Title"), title, toolTip);
+    addField(i18n("Artist"), artist, toolTip);
+    if (albumartist!=artist) {
+        addField(i18n("Album artist"), albumartist, toolTip);
+    }
+    addField(i18n("Composer"), composer(), toolTip);
+    addField(i18n("Performer"), performer(), toolTip);
+    addField(i18n("Album"), album, toolTip);
+    if (track>0) {
+        addField(i18n("Track number"), QString::number(track), toolTip);
+    }
+    if (disc>0) {
+        addField(i18n("Disc number"), QString::number(disc), toolTip);
+    }
+    addField(i18n("Genre"), displayGenre(), toolTip);
+    if (year>0) {
+        addField(i18n("Year"), QString::number(year), toolTip);
+    }
+    if (time>0) {
+        addField(i18n("Length"), Utils::formatTime(time, true), toolTip);
+    }
+    toolTip+=QLatin1String("</table>");
+
+    if (isNonMPD()) {
+        return toolTip;
+    }
+    return toolTip+QLatin1String("<br/><br/><small><i>")+filePath()+QLatin1String("</i></small>");
+    #endif
+}
+
+void Song::setExtraField(int f, const QString &v)
+{
+    if (v.isEmpty()) {
+        extra.remove(f);
+    } else {
+        extra[f]=v;
+    }
 }
 
 bool Song::isVariousArtists(const QString &str)
@@ -465,6 +550,11 @@ bool Song::isVariousArtists(const QString &str)
     return QLatin1String("Various Artists")==str || variousArtistsStr==str;
 }
 
+bool Song::diffArtist() const
+{
+    return isVariousArtists() || (!albumartist.isEmpty() && !artist.isEmpty() && albumartist!=artist);
+}
+
 bool Song::fixVariousArtists()
 {
     if (isVariousArtists()) {
@@ -529,13 +619,33 @@ bool Song::capitalise()
     albumartist=capitalize(albumartist);
     album=capitalize(album);
     title=capitalize(title);
+    QString c=composer();
+    if (!c.isEmpty()) {
+        setComposer(capitalize(c));
+    }
+    /* Performer is not currently in tag editor...
+    QString p=performer();
+    if (!p.isEmpty()) {
+        setPerformer(capitalize(p));
+    }
+    */
 
     return artist!=origArtist || albumartist!=origAlbumArtist || album!=origAlbum || title!=origTitle;
 }
 
+QString Song::albumKey() const
+{
+    #if !defined CANTATA_NO_UI_FUNCTIONS && defined ENABLE_ONLINE_SERVICES
+    if ((OnlineSvrTrack==type || Song::CantataStream) && OnlineService::showLogoAsCover(*this)) {
+        return onlineService();
+    }
+    #endif
+    return albumArtist()+QLatin1Char(':')+albumId()+QLatin1Char(':')+QString::number(disc);
+}
+
 QString Song::basicArtist() const
 {
-    if (!albumartist.isEmpty() && (artist.isEmpty() || (albumartist.length()<artist.length() && artist.startsWith(albumartist)))) {
+    if (!albumartist.isEmpty() && (artist.isEmpty() || albumartist==artist || (albumartist.length()<artist.length() && artist.startsWith(albumartist)))) {
         return albumartist;
     }
 
@@ -553,24 +663,41 @@ QString Song::basicArtist() const
     return artist;
 }
 
-bool Song::isFromOnlineService() const
+QString Song::describe(bool withMarkup) const
 {
-    return constOnlineDiscId==disc && (isCantataStream() || file.startsWith("http://")) && albumartist==album;
+    QString albumText=album.isEmpty() ? name() : displayAlbum(album, Song::albumYear(*this));
+
+    if (title.isEmpty()) {
+        return withMarkup ? albumText : (QLatin1String("<b>")+albumText+QLatin1String("</b>"));
+    }
+    QString descr=artist.isEmpty()
+                    ? i18nc("Song on Album", "<b>%1</b> on <b>%2</b>", title, albumText)
+                    : i18nc("Song by Artist on Album", "<b>%1</b> by <b>%2</b> on <b>%3</b>", title, artist, albumText);
+
+    if (!withMarkup) {
+        descr=descr.replace("<b>", "");
+        descr=descr.replace("</b>", "");
+    }
+    return descr;
 }
 
-void Song::setIsFromOnlineService(const QString &service)
-{
-    disc=constOnlineDiscId;
-    album=service;
-    albumartist=service;
-}
+//QString Song::basicDescription() const
+//{
+//    return isStandardStream()
+//            ? name
+//            : title.isEmpty()
+//                ? QString()
+//                : artist.isEmpty()
+//                    ? title
+//                    : i18nc("track - artist", "%1 - %2", title, artist);
+//}
 
 #ifdef ENABLE_EXTERNAL_TAGS
 QDataStream & operator<<(QDataStream &stream, const Song &song)
 {
-    stream << song.id << song.file << song.album << song.artist << song.albumartist << song.composer << song.title
-           << song.genre << song.name << song.disc << song.priority << song.time << song.track << (quint16)song.year
-           << (quint16)song.type << (bool)song.guessed << song.size;
+    stream << song.id << song.file << song.album << song.artist << song.albumartist << song.title
+           << song.genre << song.disc << song.priority << song.time << song.track << (quint16)song.year
+           << (quint16)song.type << (bool)song.guessed << song.size << song.extra;
     return stream;
 }
 
@@ -579,9 +706,9 @@ QDataStream & operator>>(QDataStream &stream, Song &song)
     quint16 type;
     quint16 year;
     bool guessed;
-    stream >> song.id >> song.file >> song.album >> song.artist >> song.albumartist >> song.composer >> song.title
-           >> song.genre >> song.name >> song.disc >> song.priority >> song.time >> song.track >> year
-           >> type >> guessed >> song.size;
+    stream >> song.id >> song.file >> song.album >> song.artist >> song.albumartist >> song.title
+           >> song.genre >> song.disc >> song.priority >> song.time >> song.track >> year
+           >> type >> guessed >> song.size >> song.extra;
     song.type=(Song::Type)type;
     song.year=year;
     song.guessed=guessed;
