@@ -22,28 +22,25 @@
  */
 
 #include "onlineservicespage.h"
-#include "musiclibraryitemroot.h"
-#include "musiclibraryitemartist.h"
-#include "musiclibraryitemalbum.h"
-#include "musiclibraryitemsong.h"
-#include "musiclibraryitempodcast.h"
-#include "onlineservicesmodel.h"
+#include "models/musiclibraryitemroot.h"
+#include "models/musiclibraryitemartist.h"
+#include "models/musiclibraryitemalbum.h"
+#include "models/musiclibraryitemsong.h"
+#include "models/musiclibraryitempodcast.h"
+#include "models/onlineservicesmodel.h"
 #include "jamendoservice.h"
 #include "magnatuneservice.h"
 #include "soundcloudservice.h"
 #include "podcastservice.h"
-#include "settings.h"
-#include "messagebox.h"
-#include "localize.h"
-#include "icons.h"
-#include "mainwindow.h"
-#include "stdactions.h"
-#include "actioncollection.h"
-#include "inputdialog.h"
+#include "gui/settings.h"
+#include "support/messagebox.h"
+#include "support/localize.h"
+#include "widgets/icons.h"
+#include "gui/stdactions.h"
+#include "support/actioncollection.h"
+#include "support/inputdialog.h"
 #include "podcastsearchdialog.h"
-#ifdef ENABLE_KDE_SUPPORT
-#include <KDE/KGlobalSettings>
-#endif
+#include <QMenu>
 
 OnlineServicesPage::OnlineServicesPage(QWidget *p)
     : QWidget(p)
@@ -181,13 +178,7 @@ QStringList OnlineServicesPage::selectedFiles() const
     if (selected.isEmpty()) {
         return QStringList();
     }
-
-    QModelIndexList mapped;
-    foreach (const QModelIndex &idx, selected) {
-        mapped.append(proxy.mapToSource(idx));
-    }
-
-    return OnlineServicesModel::self()->filenames(mapped);
+    return OnlineServicesModel::self()->filenames(proxy.mapToSource(selected, proxy.enabled() && Settings::self()->filteredOnly()));
 }
 
 QList<Song> OnlineServicesPage::selectedSongs(bool allowPlaylists) const
@@ -197,21 +188,7 @@ QList<Song> OnlineServicesPage::selectedSongs(bool allowPlaylists) const
     if (selected.isEmpty()) {
         return QList<Song>();
     }
-
-    QModelIndexList mapped;
-    foreach (const QModelIndex &idx, selected) {
-        QModelIndex index = proxy.mapToSource(idx);
-        mapped.append(index);
-        MusicLibraryItem *item=static_cast<MusicLibraryItem *>(index.internalPointer());
-
-        if (item && MusicLibraryItem::Type_Root!=item->itemType()) {
-            while(item->parentItem()) {
-                item=item->parentItem();
-            }
-        }
-    }
-
-    return OnlineServicesModel::self()->songs(mapped);
+    return OnlineServicesModel::self()->songs(proxy.mapToSource(selected, proxy.enabled() && Settings::self()->filteredOnly()));
 }
 
 void OnlineServicesPage::addSelectionToPlaylist(const QString &name, bool replace, quint8 priorty, bool randomAlbums)
@@ -229,6 +206,17 @@ void OnlineServicesPage::addSelectionToPlaylist(const QString &name, bool replac
     }
 }
 
+void OnlineServicesPage::focusSearch()
+{
+    if (view->searchVisible()) {
+        view->clearSearchText();
+        view->focusSearch();
+        controlSearch(true);
+    } else {
+        view->focusSearch();
+    }
+}
+
 void OnlineServicesPage::refresh()
 {
     OnlineServicesModel::self()->resetModel();
