@@ -22,21 +22,21 @@
  */
 
 #include "podcastsearchdialog.h"
-#include "pagewidget.h"
-#include "lineedit.h"
-#include "networkaccessmanager.h"
+#include "support/pagewidget.h"
+#include "support/lineedit.h"
+#include "network/networkaccessmanager.h"
 #include "opmlparser.h"
-#include "icons.h"
-#include "spinner.h"
-#include "qjson/parser.h"
-#include "basicitemdelegate.h"
-#include "onlineservicesmodel.h"
+#include "widgets/icons.h"
+#include "support/spinner.h"
+#include "widgets/basicitemdelegate.h"
+#include "models/onlineservicesmodel.h"
 #include "podcastservice.h"
-#include "utils.h"
-#include "action.h"
-#include "textbrowser.h"
-#include "messagewidget.h"
+#include "support/utils.h"
+#include "support/action.h"
+#include "widgets/textbrowser.h"
+#include "support/messagewidget.h"
 #include "rssparser.h"
+#include "config.h"
 #include <QLabel>
 #include <QPushButton>
 #include <QTreeWidget>
@@ -47,12 +47,17 @@
 #include <QFile>
 #include <QXmlStreamReader>
 #include <QCryptographicHash>
-#include <QProcess>
 #include <QCache>
 #include <QImage>
 #include <QBuffer>
+#include <QCoreApplication>
+#include <QDesktopServices>
+#include <QNetworkRequest>
 #if QT_VERSION >= 0x050000
 #include <QUrlQuery>
+#include <QJsonDocument>
+#else
+#include "qjson/parser.h"
 #endif
 
 static int iCount=0;
@@ -312,7 +317,7 @@ void PodcastPage::jobFinished()
 
 void PodcastPage::openLink(const QUrl &url)
 {
-    QProcess::startDetached(QLatin1String("xdg-open"), QStringList() << url.toString());
+    QDesktopServices::openUrl(url);
 }
 
 PodcastSearchPage::PodcastSearchPage(QWidget *p, const QString &n, const QString &i, const QUrl &qu, const QString &qk, const QStringList &other)
@@ -381,8 +386,11 @@ void PodcastSearchPage::parseResonse(QIODevice *dev)
         emit error(i18n("Failed to fetch podcasts from %1", name()));
         return;
     }
-    QJson::Parser parser;
-    QVariant data = parser.parse(dev);
+    #if QT_VERSION >= 0x050000
+    QVariant data=QJsonDocument::fromJson(dev->readAll()).toVariant();
+    #else
+    QVariant data = QJson::Parser().parse(dev->readAll());
+    #endif
     if (data.isNull()) {
         emit error(i18n("There was a problem parsing the response from %1", name()));
         return;
@@ -403,11 +411,7 @@ OpmlBrowsePage::OpmlBrowsePage(QWidget *p, const QString &n, const QString &i, c
     tree->addAction(act);
     connect(act, SIGNAL(triggered(bool)), this, SLOT(reload()));
     tree->setContextMenuPolicy(Qt::ActionsContextMenu);
-    if (i.isEmpty()) {
-        icn=Icon("folder");
-    } else {
-        icn.addFile(":"+i);
-    }
+    icn.addFile(i.isEmpty() || !QFile::exists(i) ? ":podcasts" : i);
 }
 
 void OpmlBrowsePage::showEvent(QShowEvent *e)
@@ -613,21 +617,9 @@ PodcastSearchDialog::PodcastSearchDialog(QWidget *parent)
     pageWidget->addPage(gpodder, i18n("Search %1", gpodder->name()), gpodder->icon(), i18n("Search for podcasts on %1", gpodder->name()));
     pages << gpodder;
 
-    QFile file(":podcast_directories.xml");
-    if (file.open(QIODevice::ReadOnly)) {
-        QXmlStreamReader reader(&file);
-        while (!reader.atEnd()) {
-            reader.readNext();
-            if (reader.isStartElement() && QLatin1String("directory")==reader.name()) {
-                OpmlBrowsePage *page=new OpmlBrowsePage(pageWidget,
-                                                        reader.attributes().value(QLatin1String("name")).toString(),
-                                                        reader.attributes().value(QLatin1String("icon")).toString(),
-                                                        QUrl(reader.attributes().value(QLatin1String("url")).toString()));
-                pageWidget->addPage(page, i18n("Browse %1", page->name()), page->icon(), i18n("Browse %1 podcasts", page->name()));
-                pages << page;
-            }
-        }
-    }
+    QSet<QString> loaded;
+    pages << loadDirectories(Utils::dataDir(), false, loaded);
+    pages << loadDirectories(CANTATA_SYS_CONFIG_DIR, true, loaded);
 
     foreach (PodcastPage *p, pages) {
         connect(p, SIGNAL(rssSelected(QUrl)), SLOT(rssSelected(QUrl)));
@@ -684,6 +676,41 @@ void PodcastSearchDialog::pageChanged()
     rssSelected(page ? page->currentRss() : QUrl());
 }
 
+QList<PodcastPage *> PodcastSearchDialog::loadDirectories(const QString &dir, bool isSystem, QSet<QString> &loaded)
+{
+    QList<PodcastPage *> pages;
+
+    if (dir.isEmpty()) {
+        return pages;
+    }
+
+    QFile file(dir+QLatin1String("/podcast_directories.xml"));
+
+    if (file.open(QIODevice::ReadOnly)) {
+        QXmlStreamReader reader(&file);
+        while (!reader.atEnd()) {
+            reader.readNext();
+            if (reader.isStartElement() && QLatin1String("directory")==reader.name()) {
+                QString url=reader.attributes().value(QLatin1String("url")).toString();
+                if (!loaded.contains(url)) {
+                    QString icon=reader.attributes().value(QLatin1String("icon")).toString();
+                    if (!icon.isEmpty()) {
+                        icon=dir+(isSystem ? "../icons/" : "")+icon;
+                    }
+                    OpmlBrowsePage *page=new OpmlBrowsePage(pageWidget,
+                                                            reader.attributes().value(QLatin1String("name")).toString(),
+                                                            icon,
+                                                            QUrl(url));
+                    pageWidget->addPage(page, i18n("Browse %1", page->name()), page->icon(), i18n("Browse %1 podcasts", page->name()));
+                    pages << page;
+                    loaded.insert(url);
+                }
+            }
+        }
+    }
+    return pages;
+}
+
 void PodcastSearchDialog::slotButtonClicked(int button)
 {
     switch (button) {
