@@ -23,15 +23,14 @@
 
 #include "podcastservice.h"
 #include "podcastsettingsdialog.h"
-#include "networkaccessmanager.h"
-#include "onlineservicesmodel.h"
-#include "musiclibraryitempodcast.h"
-#include "musiclibraryitemsong.h"
-#include "utils.h"
-#include "settings.h"
-#include "mpdconnection.h"
+#include "models/onlineservicesmodel.h"
+#include "models/musiclibraryitempodcast.h"
+#include "models/musiclibraryitemsong.h"
+#include "support/utils.h"
+#include "gui/settings.h"
+#include "mpd/mpdconnection.h"
 #include "config.h"
-#include "httpserver.h"
+#include "http/httpserver.h"
 #include <QCoreApplication>
 #include <QDir>
 #include <QUrl>
@@ -50,16 +49,6 @@ static const char * constRssUrlProperty="rss-url";
 static const char * constDestProperty="dest";
 static const QLatin1String constPartialExt(".partial");
 
-// Move files from previous ~/.config/cantata to ~/.local/share/cantata
-static void moveToNewLocation()
-{
-    #if !defined Q_OS_WIN && !defined Q_OS_MAC // Not required for windows - as already stored in data location!
-    if (Settings::self()->version()<CANTATA_MAKE_VERSION(1, 51, 0)) {
-        Utils::moveDir(Utils::configDir(MusicLibraryItemPodcast::constDir), Utils::dataDir(MusicLibraryItemPodcast::constDir, true));
-    }
-    #endif
-}
-
 QUrl PodcastService::fixUrl(const QString &url)
 {
     QString trimmed(url.trimmed());
@@ -103,19 +92,11 @@ PodcastService::PodcastService(MusicModel *m)
     : OnlineService(m, i18n("Podcasts"))
     , rssUpdateTimer(0)
 {
-    moveToNewLocation();
-
     loaded=true;
-    setUseArtistImages(false);
-    setUseAlbumImages(false);
     QMetaObject::invokeMethod(this, "loadAll", Qt::QueuedConnection);
     connect(MPDConnection::self(), SIGNAL(currentSongUpdated(const Song &)), this, SLOT(currentMpdSong(const Song &)));
     if (iconFile.isEmpty()) {
-        #ifdef Q_OS_WIN
-        iconFile=QCoreApplication::applicationDirPath()+"/icons/podcasts.png";
-        #else
-        iconFile=QString(INSTALL_PREFIX"/share/")+QCoreApplication::applicationName()+"/icons/podcasts.png";
-        #endif
+        iconFile=QString(CANTATA_SYS_ICONS_DIR+"podcasts.png");
     }
 }
 
@@ -124,6 +105,7 @@ Song PodcastService::fixPath(const Song &orig, bool) const
     Song song=orig;
     song.setPodcastLocalPath(QString());
     song.setIsFromOnlineService(constName);
+    song.album=constName;
     if (!orig.podcastLocalPath().isEmpty() && QFile::exists(orig.podcastLocalPath())) {
         if (!HttpServer::self()->forceUsage() && MPDConnection::self()->getDetails().isLocal()) {
             song.file=QLatin1String("file://")+orig.podcastLocalPath();
@@ -195,8 +177,8 @@ void PodcastService::rssJobFinished()
     bool isNew=j->property(constNewFeedProperty).toBool();
 
     if (j->ok()) {
-        if (updateUrls.contains(j->url())){
-            updateUrls.remove(j->url());
+        if (updateUrls.contains(j->origUrl())){
+            updateUrls.remove(j->origUrl());
             if (updateUrls.isEmpty()) {
                 lastRssUpdate=QDateTime::currentDateTime();
                 Settings::self()->saveLastRssUpdate(lastRssUpdate);
@@ -223,7 +205,7 @@ void PodcastService::rssJobFinished()
                 endInsertRows();
 //                emitNeedToSort();
             } else {
-                MusicLibraryItemPodcast *orig = getPodcast(j->url());
+                MusicLibraryItemPodcast *orig = getPodcast(j->origUrl());
                 if (!orig) {
                     delete podcast;
                     return;
@@ -284,13 +266,13 @@ void PodcastService::rssJobFinished()
         } else if (isNew) {
             delete podcast;
             if (MusicLibraryItemPodcast::VideoPodcast==loadStatus) {
-                emitError(i18n("Cantata only supports audio podcasts! %1 contains only video podcasts.", j->url().toString()), isNew);
+                emitError(i18n("Cantata only supports audio podcasts! %1 contains only video podcasts.", j->origUrl().toString()), isNew);
             } else {
-                emitError(i18n("Failed to parse %1", j->url().toString()), isNew);
+                emitError(i18n("Failed to parse %1", j->origUrl().toString()), isNew);
             }
         }
     } else {
-        emitError(i18n("Failed to download %1", j->url().toString()), isNew);
+        emitError(i18n("Failed to download %1", j->origUrl().toString()), isNew);
     }
     setBusy(!rssJobs.isEmpty() || !downloadJobs.isEmpty());
 }
@@ -334,8 +316,17 @@ void PodcastService::unSubscribe(MusicLibraryItem *item)
 {
     int row=m_childItems.indexOf(item);
     if (row>=0) {
+        MusicLibraryItemPodcast *pod=static_cast<MusicLibraryItemPodcast *>(item);
+        if (!downloadJobs.isEmpty()) {
+            foreach (MusicLibraryItem *e, pod->childItems()) {
+                cancelDownload(QUrl(static_cast<MusicLibraryItemPodcastEpisode *>(e)->file()));
+                if (downloadJobs.isEmpty()) {
+                    break;
+                }
+            }
+        }
         beginRemoveRows(index(), row, row);
-        static_cast<MusicLibraryItemPodcast *>(item)->removeFiles();
+        pod->removeFiles();
         delete m_childItems.takeAt(row);
         resetRows();
         endRemoveRows();
@@ -361,7 +352,7 @@ void PodcastService::refreshSubscription(MusicLibraryItem *item)
 bool PodcastService::processingUrl(const QUrl &url) const
 {
     foreach (NetworkJob *j, rssJobs) {
-        if (j->url()==url) {
+        if (j->origUrl()==url) {
             return true;
         }
     }
@@ -380,7 +371,7 @@ void PodcastService::addUrl(const QUrl &url, bool isNew)
 bool PodcastService::downloadingEpisode(const QUrl &url) const
 {
     foreach (NetworkJob *j, downloadJobs) {
-        if (j->url()==url) {
+        if (j->origUrl()==url) {
             return true;
         }
     }
@@ -471,7 +462,7 @@ void PodcastService::downloadEpisode(const MusicLibraryItemPodcast *podcast, con
 void PodcastService::cancelDownload(const QUrl &url)
 {
     foreach (NetworkJob *j, downloadJobs) {
-        if (j->url()==url) {
+        if (j->origUrl()==url) {
             cancelDownload(j);
             downloadJobs.removeAll(j);
             break;
@@ -516,7 +507,7 @@ void PodcastService::downloadJobFinished()
             if (QFile::rename(partial, dest)) {
                 MusicLibraryItemPodcast *pod=getPodcast(job->property(constRssUrlProperty).toUrl());
                 if (pod) {
-                    MusicLibraryItemPodcastEpisode *song=getEpisode(pod, job->url());
+                    MusicLibraryItemPodcastEpisode *song=getEpisode(pod, job->origUrl());
                     if (song) {
                         song->setLocalPath(dest);
                         song->setDownloadProgress(-1);
@@ -574,7 +565,7 @@ void PodcastService::downloadPercent(int pc)
     }
     MusicLibraryItemPodcast *pod=getPodcast(job->property(constRssUrlProperty).toUrl());
     if (pod) {
-        MusicLibraryItemPodcastEpisode *song=getEpisode(pod, job->url());
+        MusicLibraryItemPodcastEpisode *song=getEpisode(pod, job->origUrl());
         if (song) {
             song->setDownloadProgress(pc);
             emitDataChanged(createIndex(song));
@@ -631,11 +622,12 @@ void PodcastService::updateRss()
 void PodcastService::currentMpdSong(const Song &s)
 {
     if (s.isFromOnlineService() && s.album==constName) {
+        QString path=s.decodedPath();
         foreach (MusicLibraryItem *p, m_childItems) {
             MusicLibraryItemPodcast *podcast=static_cast<MusicLibraryItemPodcast *>(p);
             foreach (MusicLibraryItem *i, podcast->childItems()) {
                 MusicLibraryItemSong *song=static_cast<MusicLibraryItemSong *>(i);
-                if (song->file()==s.file || song->song().podcastLocalPath()==s.file) {
+                if (song->file()==path || song->song().podcastLocalPath()==path) {
                     if (!song->song().hasBeenPlayed()) {
                         podcast->setPlayed(song);
                         emitDataChanged(createIndex(song));
