@@ -23,19 +23,19 @@
 
 #include "rgdialog.h"
 #ifdef ENABLE_DEVICES_SUPPORT
-#include "device.h"
-#include "devicesmodel.h"
+#include "devices/device.h"
+#include "models/devicesmodel.h"
 #endif
-#include "settings.h"
-#include "tags.h"
+#include "gui/settings.h"
+#include "tags/tags.h"
 #include "tagreader.h"
-#include "utils.h"
-#include "localize.h"
-#include "messagebox.h"
+#include "support/utils.h"
+#include "support/localize.h"
+#include "support/messagebox.h"
 #include "jobcontroller.h"
-#include "basicitemdelegate.h"
-#include "action.h"
-#include "cuefile.h"
+#include "widgets/basicitemdelegate.h"
+#include "support/action.h"
+#include "mpd/cuefile.h"
 #include <QComboBox>
 #include <QTreeWidget>
 #include <QLabel>
@@ -45,18 +45,6 @@
 #include <QCloseEvent>
 #include <QCoreApplication>
 #include <QEventLoop>
-#ifdef ENABLE_KDE_SUPPORT
-#include <KDE/KLocale>
-#endif
-
-static inline QString formatNumber(double number, int precision)
-{
-    #ifdef ENABLE_KDE_SUPPORT
-    return KGlobal::locale()->formatNumber(number, precision);
-    #else
-    return QString::number(number, 'f', precision);
-    #endif
-}
 
 enum Columns
 {
@@ -85,15 +73,6 @@ static inline void setResizeMode(QHeaderView *hdr, int idx, QHeaderView::ResizeM
     #endif
 }
 
-static inline void setResizeMode(QHeaderView *hdr, QHeaderView::ResizeMode mode)
-{
-    #if QT_VERSION < 0x050000
-    hdr->setResizeMode(mode);
-    #else
-    hdr->setSectionResizeMode(mode);
-    #endif
-}
-
 RgDialog::RgDialog(QWidget *parent)
     : SongDialog(parent, "RgDialog", QSize(800, 400))
     , state(State_Idle)
@@ -409,15 +388,9 @@ bool RgDialog::saveTags()
         case Tags::Update_Failed:
             failed.append(filePath);
             break;
-        #ifdef ENABLE_EXTERNAL_TAGS
-        case Tags::Update_Timedout:
-            failed.append(i18nc("filename (Timeout)", "%1 (Timeout)", filePath));
-            someTimedout=true;
-            break;
         case Tags::Update_BadFile:
             failed.append(i18nc("filename (Corrupt tags?)", "%1 (Corrupt tags?)", filePath));
             break;
-        #endif
         default:
             break;
         }
@@ -444,7 +417,7 @@ void RgDialog::updateView()
 
     for (; it!=end; ++it) {
         if (100==it.value()) {
-            finished++;;
+            finished++;
         }
         totalProgress+=(*it);
     }
@@ -510,15 +483,15 @@ void RgDialog::scannerDone()
             Tags::ReplayGain updatedTags(it.value().gain, s->albumValues().gain, it.value().peak, s->albumValues().peak);
             QTreeWidgetItem *item=view->topLevelItem(it.key());
             if (it.value().ok) {
-                item->setText(COL_TRACKGAIN, i18n("%1 dB", formatNumber(updatedTags.trackGain, 2)));
-                item->setText(COL_TRACKPEAK, formatNumber(updatedTags.trackPeak, 6));
+                item->setText(COL_TRACKGAIN, i18n("%1 dB", Utils::formatNumber(updatedTags.trackGain, 2)));
+                item->setText(COL_TRACKPEAK, Utils::formatNumber(updatedTags.trackPeak, 6));
             } else {
                 item->setText(COL_TRACKGAIN, i18n("Failed"));
                 item->setText(COL_TRACKPEAK, i18n("Failed"));
             }
             if (s->albumValues().ok) {
-                item->setText(COL_ALBUMGAIN, i18n("%1 dB", formatNumber(updatedTags.albumGain, 2)));
-                item->setText(COL_ALBUMPEAK, formatNumber(updatedTags.albumPeak, 6));
+                item->setText(COL_ALBUMGAIN, i18n("%1 dB", Utils::formatNumber(updatedTags.albumGain, 2)));
+                item->setText(COL_ALBUMPEAK, Utils::formatNumber(updatedTags.albumPeak, 6));
             } else {
                 item->setText(COL_ALBUMGAIN, i18n("Failed"));
                 item->setText(COL_ALBUMPEAK, i18n("Failed"));
@@ -530,18 +503,22 @@ void RgDialog::scannerDone()
                 bool diffAlbum=false;
                 if (!Utils::equal(t.trackGain, updatedTags.trackGain, 0.01)) {
                     item->setFont(COL_TRACKGAIN, italic);
+                    item->setToolTip(COL_TRACKGAIN, i18n("Original: %1 dB", Utils::formatNumber(t.trackGain, 2)));
                     diff=true;
                 }
                 if (!Utils::equal(t.trackPeak, updatedTags.trackPeak, 0.000001)) {
                     item->setFont(COL_TRACKPEAK, italic);
+                    item->setToolTip(COL_TRACKPEAK, i18n("Original: %1", Utils::formatNumber(t.trackPeak, 6)));
                     diff=true;
                 }
                 if (!Utils::equal(t.albumGain, updatedTags.albumGain, 0.01)) {
                     item->setFont(COL_ALBUMGAIN, italic);
+                    item->setToolTip(COL_ALBUMGAIN, i18n("Original: %1 dB", Utils::formatNumber(t.albumGain, 2)));
                     diffAlbum=true;
                 }
                 if (!Utils::equal(t.albumPeak, updatedTags.albumPeak, 0.000001)) {
                     item->setFont(COL_ALBUMPEAK, italic);
+                    item->setToolTip(COL_ALBUMPEAK, i18n("Original: %1", Utils::formatNumber(t.albumPeak, 6)));
                     diffAlbum=true;
                 }
                 if (diff || diffAlbum) {
@@ -596,10 +573,10 @@ void RgDialog::songTags(int index, Tags::ReplayGain tags)
             if (!item) {
                 return;
             }
-            item->setText(COL_TRACKGAIN, i18n("%1 dB", formatNumber(tags.trackGain, 2)));
-            item->setText(COL_TRACKPEAK, formatNumber(tags.trackPeak, 6));
-            item->setText(COL_ALBUMGAIN, i18n("%1 dB", formatNumber(tags.albumGain, 2)));
-            item->setText(COL_ALBUMPEAK, formatNumber(tags.albumPeak, 6));
+            item->setText(COL_TRACKGAIN, i18n("%1 dB", Utils::formatNumber(tags.trackGain, 2)));
+            item->setText(COL_TRACKPEAK, Utils::formatNumber(tags.trackPeak, 6));
+            item->setText(COL_ALBUMGAIN, i18n("%1 dB", Utils::formatNumber(tags.albumGain, 2)));
+            item->setText(COL_ALBUMPEAK, Utils::formatNumber(tags.albumPeak, 6));
         }
     }
 }
