@@ -27,6 +27,7 @@
 #include "ui_streamssettings.h"
 
 class QListWidgetItem;
+class StreamProviderListDialog;
 
 class StreamsSettings : public QWidget, private Ui::StreamsSettings
 {
@@ -41,12 +42,18 @@ public:
 
 private Q_SLOTS:
     void currentCategoryChanged(int row);
-    void install();
+    void installFromFile();
+    void installFromWeb();
     void remove();
     void configure();
 
 private:
+    bool install(const QString &fileName, const QString &name, bool showErrors=true);
     QListWidgetItem * get(const QString &name);
+
+private:
+    friend class StreamProviderListDialog;
+    StreamProviderListDialog *providerDialog;
 };
 
 #endif
