@@ -32,6 +32,8 @@ class QWidget;
 
 namespace GtkStyle
 {
+    extern const char * constHideFrameProp;
+
     extern bool isActive();
     extern void drawSelection(const QStyleOptionViewItemV4 &opt, QPainter *painter, double opacity);
     extern QString themeName();
@@ -39,6 +41,7 @@ namespace GtkStyle
     extern void setThemeName(const QString &n);
     extern void setIconTheme(const QString &n);
     extern void applyTheme(QWidget *widget);
+    extern void registerWidget(QWidget *widget);
     extern bool useSymbolicIcons();
     extern QColor symbolicColor();
 }
