@@ -27,6 +27,7 @@
 #include <QFile>
 #include <QFileInfo>
 #include <QDir>
+#include <QProcess>
 #include <QSet>
 #include <QThread>
 #include <QApplication>
@@ -34,15 +35,17 @@
 #include <QTime>
 #include <QWidget>
 #include <QStyle>
+#if QT_VERSION >= 0x050000
+#include <QStandardPaths>
+#else
+#include <QDesktopServices>
+#endif
 #ifdef ENABLE_KDE_SUPPORT
 #include <KDE/KStandardDirs>
 #endif
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#ifdef Q_OS_WIN
-#include <QDesktopServices>
-#endif
 #ifndef Q_OS_WIN
 #include <grp.h>
 #include <pwd.h>
@@ -50,8 +53,8 @@
 #include <sys/types.h>
 #include <utime.h>
 
-QLatin1Char Utils::constDirSep('/');
-QLatin1String Utils::constDirSepStr("/");
+const QLatin1Char Utils::constDirSep('/');
+const QLatin1String Utils::constDirSepStr("/");
 const char * Utils::constDirSepCharStr="/";
 
 static const QLatin1String constHttp("http://");
@@ -144,6 +147,7 @@ QString Utils::convertDirFromDisplay(const QString &dir)
 
 QString Utils::getDir(const QString &file)
 {
+    bool isCueFile=file.contains("/cue:///") && file.contains("?pos=");
     QString d(file);
     int slashPos(d.lastIndexOf(constDirSep));
 
@@ -151,6 +155,9 @@ QString Utils::getDir(const QString &file)
         d.remove(slashPos+1, d.length());
     }
 
+    if (isCueFile) {
+        d.remove("cue:///");
+    }
     return fixPath(d);
 }
 
@@ -302,7 +309,7 @@ void Utils::msleep(int msecs)
 #include <sys/stat.h>
 
 // kde_file.h
-#ifndef Q_WS_WIN
+#ifndef Q_OS_WIN
 #if (defined _LFS64_LARGEFILE) && (defined _LARGEFILE64_SOURCE) && (!defined _GNU_SOURCE) && (!defined __sun)
 #define KDE_stat                ::stat64
 #define KDE_lstat               ::lstat64
@@ -314,17 +321,17 @@ void Utils::msleep(int msecs)
 #define KDE_struct_stat         struct stat
 #define KDE_mkdir               ::mkdir
 #endif
-#endif // Q_WS_WIN
+#endif // Q_OS_WIN
 
 // kstandarddirs.h
-extern bool Utils::makeDir(const QString &dir, int mode)
+bool Utils::makeDir(const QString &dir, int mode)
 {
     // we want an absolute path
     if (QDir::isRelativePath(dir)) {
         return false;
     }
 
-    #ifdef Q_WS_WIN
+    #ifdef Q_OS_WIN
     Q_UNUSED(mode)
     return QDir().mkpath(dir);
     #else
@@ -482,7 +489,7 @@ static QStringList systemPaths(const QString &pstr)
     return exePaths;
 }
 
-#ifdef Q_WS_MAC
+#ifdef Q_OS_MAC
 static QString getBundle(const QString &path)
 {
     //kDebug(180) << "getBundle(" << path << ", " << ignore << ") called";
@@ -491,7 +498,7 @@ static QString getBundle(const QString &path)
     bundle += QLatin1String(".app/Contents/MacOS/") + bundle.section(QLatin1Char('/'), -1);
     info.setFile( bundle );
     FILE *file;
-    if (file = fopen(info.absoluteFilePath().toUtf8().constData(), "r")) {
+    if ((file = fopen(info.absoluteFilePath().toUtf8().constData(), "r"))) {
         fclose(file);
         struct stat _stat;
         if ((stat(info.absoluteFilePath().toUtf8().constData(), &_stat)) < 0) {
@@ -510,7 +517,7 @@ static QString getBundle(const QString &path)
 
 static QString checkExecutable( const QString& path )
 {
-    #ifdef Q_WS_MAC
+    #ifdef Q_OS_MAC
     QString bundle = getBundle( path );
     if ( !bundle.isEmpty() ) {
         //kDebug(180) << "findExe(): returning " << bundle;
@@ -521,7 +528,7 @@ static QString checkExecutable( const QString& path )
     QFileInfo orig = info;
     #if defined(Q_OS_DARWIN) || defined(Q_OS_MAC)
     FILE *file;
-    if (file = fopen(orig.absoluteFilePath().toUtf8().constData(), "r")) {
+    if ((file = fopen(orig.absoluteFilePath().toUtf8().constData(), "r"))) {
         fclose(file);
         struct stat _stat;
         if ((stat(orig.absoluteFilePath().toUtf8().constData(), &_stat)) < 0) {
@@ -607,6 +614,24 @@ QString Utils::formatDuration(const quint32 totalseconds)
     #endif
 }
 
+QString Utils::formatTime(const quint32 seconds, bool zeroIsUnknown)
+{
+    if (0==seconds && zeroIsUnknown) {
+        return i18n("Unknown");
+    }
+
+    static const quint32 constHour=60*60;
+    if (seconds>constHour) {
+        return Utils::formatDuration(seconds);
+    }
+
+    QString result(QString::number(floor(seconds / 60.0))+QChar(':'));
+    if (seconds % 60 < 10) {
+        result += "0";
+    }
+    return result+QString::number(seconds % 60);
+}
+
 QString Utils::cleanPath(const QString &p)
 {
     QString path(p);
@@ -627,33 +652,81 @@ static QString userDir(const QString &mainDir, const QString &sub, bool create)
     return d.exists() || (create && d.mkpath(dir)) ? dir : QString();
 }
 
-QString Utils::configDir(const QString &sub, bool create)
-{
-    #if defined Q_OS_WIN
-    return userDir(QDesktopServices::storageLocation(QDesktopServices::DataLocation)+constDirSep, sub, create);
-    #else
-    QString env = qgetenv("XDG_CONFIG_HOME");
-    return userDir((env.isEmpty() ? QDir::homePath() + "/.config" : env) + constDirSep+QCoreApplication::applicationName()+constDirSep, sub, create);
-    #endif
-}
-
 QString Utils::dataDir(const QString &sub, bool create)
 {
-    #if defined Q_OS_WIN
-    return userDir(QDesktopServices::storageLocation(QDesktopServices::DataLocation)+constDirSep, sub, create);
+    #if defined Q_OS_WIN || defined Q_OS_MAC || defined ENABLE_UBUNTU
+
+    #if QT_VERSION >= 0x050000
+    return userDir(QStandardPaths::writableLocation(QStandardPaths::DataLocation)+constDirSep, sub, create);
     #else
-    QString env = qgetenv("XDG_DATA_HOME");
-    return userDir((env.isEmpty() ? QDir::homePath() + "/.local/share" : env) + constDirSep+QCoreApplication::applicationName()+constDirSep, sub, create);
+    return userDir(QDesktopServices::storageLocation(QDesktopServices::DataLocation)+constDirSep, sub, create);
+    #endif
+
+    #else
+
+    static QString location;
+    if (location.isEmpty()) {
+        #if QT_VERSION >= 0x050000
+        location=QStandardPaths::writableLocation(QStandardPaths::DataLocation);
+        location=location.replace(QCoreApplication::applicationName()+Utils::constDirSep+QCoreApplication::applicationName(),
+                                  QCoreApplication::applicationName());
+        #else
+        location=QDesktopServices::storageLocation(QDesktopServices::DataLocation);
+        location=location.replace(QLatin1String("data")+Utils::constDirSep+QCoreApplication::applicationName()+Utils::constDirSep+QCoreApplication::applicationName(),
+                                  QCoreApplication::applicationName());
+        #endif
+    }
+    return userDir(location+constDirSep, sub, create);
+
     #endif
 }
 
 QString Utils::cacheDir(const QString &sub, bool create)
 {
-    #if defined Q_OS_WIN
-    return userDir(QDesktopServices::storageLocation(QDesktopServices::CacheLocation)+constDirSep, sub, create);
+    #if defined Q_OS_WIN || defined Q_OS_MAC || defined ENABLE_UBUNTU
+
+    #if QT_VERSION >= 0x050000
+    return userDir(QStandardPaths::writableLocation(QStandardPaths::CacheLocation)+constDirSep, sub, create);
     #else
-    QString env = qgetenv("XDG_CACHE_HOME");
-    return userDir((env.isEmpty() ? QDir::homePath() + "/.cache" : env) + constDirSep+QCoreApplication::applicationName()+constDirSep, sub, create);
+    return userDir(QDesktopServices::storageLocation(QDesktopServices::CacheLocation)+constDirSep, sub, create);
+    #endif
+
+    #else
+
+    static QString location;
+    if (location.isEmpty()) {
+        #if QT_VERSION >= 0x050000
+        location=QStandardPaths::writableLocation(QStandardPaths::CacheLocation);
+        #else
+        location=QDesktopServices::storageLocation(QDesktopServices::CacheLocation);
+        #endif
+        location=location.replace(QCoreApplication::applicationName()+Utils::constDirSep+QCoreApplication::applicationName(),
+                                  QCoreApplication::applicationName());
+    }
+    return userDir(location+constDirSep, sub, create);
+
+    #endif
+}
+
+QString Utils::systemDir(const QString &sub)
+{
+    #if defined Q_OS_WIN
+    return fixPath(QCoreApplication::applicationDirPath())+(sub.isEmpty() ? QString() : (sub+constDirSep));
+    #elif defined Q_OS_MAC
+    return fixPath(QCoreApplication::applicationDirPath())+QLatin1String("Contents/Resources/")+(sub.isEmpty() ? QString() : (sub+constDirSep));
+    #else
+    return fixPath(QString(INSTALL_PREFIX "/share/")+QCoreApplication::applicationName()+constDirSep+(sub.isEmpty() ? QString() : sub));
+    #endif
+}
+
+QString Utils::helper(const QString &app)
+{
+    #if defined Q_OS_WIN
+    return systemDir(QLatin1String("helpers"))+app+QLatin1String(".exe");
+    #elif defined Q_OS_MAC
+    return systemDir(QLatin1String("helpers"))+app;
+    #else
+    return QString(INSTALL_PREFIX "/lib/")+QCoreApplication::applicationName()+constDirSep+app;
     #endif
 }
 
@@ -727,7 +800,7 @@ void Utils::touchFile(const QString &fileName)
 
 bool Utils::isDirReadable(const QString &dir)
 {
-    #ifdef Q_OS_WIN32
+    #ifdef Q_OS_WIN
     if (dir.isEmpty()) {
         return false;
     } else {
@@ -744,16 +817,34 @@ bool Utils::isDirReadable(const QString &dir)
     #endif
 }
 
-extern int Utils::layoutSpacing(QWidget *w)
+double Utils::smallFontFactor(const QFont &f)
+{
+    double sz=f.pointSizeF();
+    if (sz<=8.5) {
+        return 1.0;
+    }
+    if (sz<=9.0) {
+        return 0.9;
+    }
+    return 0.85;
+}
+
+QFont Utils::smallFont(QFont f)
+{
+    f.setPointSizeF(f.pointSizeF()*smallFontFactor(f));
+    return f;
+}
+
+int Utils::layoutSpacing(QWidget *w)
 {
     int spacing=(w ? w->style() : qApp->style())->layoutSpacing(QSizePolicy::DefaultType, QSizePolicy::DefaultType, Qt::Vertical);
     if (spacing<0) {
-        spacing=isHighDpi() ? 8 : 4;
+        spacing=scaleForDpi(4);
     }
     return spacing;
 }
 
-extern bool Utils::isHighDpi()
+bool Utils::isHighDpi()
 {
     static int fontHeight=-1;
     if (-1==fontHeight) {
@@ -762,23 +853,56 @@ extern bool Utils::isHighDpi()
     return fontHeight>22;
 }
 
-#if !defined Q_OS_WIN32 && !defined Q_OS_MAC
+int Utils::scaleForDpi(int v)
+{
+    return isHighDpi() ? v*2 : v;
+}
+
 Utils::Desktop Utils::currentDe()
 {
-    QByteArray desktop=qgetenv("XDG_CURRENT_DESKTOP").toLower();
-    if ("unity"==desktop) {
-        return Unity;
-    }
-    if ("kde"==desktop) {
-        return KDE;
-    }
-    if ("gnome"==desktop) {
-        return Gnome;
-    }
-    QByteArray kde=qgetenv("KDE_FULL_SESSION");
-    if ("true"==kde) {
-        return KDE;
+    #if !defined Q_OS_WIN && !defined Q_OS_MAC
+    static int de=-1;
+    if (-1==de) {
+        de=Other;
+        QByteArray desktop=qgetenv("XDG_CURRENT_DESKTOP").toLower();
+        if ("unity"==desktop) {
+            de=Unity;
+        } else if ("kde"==desktop) {
+            de=KDE;
+        } else if ("gnome"==desktop || "pantheon"==desktop) {
+            de=Gnome;
+        } else {
+            QByteArray kde=qgetenv("KDE_FULL_SESSION");
+            if ("true"==kde) {
+                de=KDE;
+            }
+        }
     }
+    return (Utils::Desktop)de;
+    #endif
     return Other;
 }
-#endif
+
+static bool isTouchFriendly=false;
+void Utils::setTouchFriendly(bool t)
+{
+    isTouchFriendly=t;
+}
+
+bool Utils::touchFriendly()
+{
+    return isTouchFriendly;
+}
+
+QPainterPath Utils::buildPath(const QRectF &r, double radius)
+{
+    QPainterPath path;
+    double diameter(radius*2);
+
+    path.moveTo(r.x()+r.width(), r.y()+r.height()-radius);
+    path.arcTo(r.x()+r.width()-diameter, r.y(), diameter, diameter, 0, 90);
+    path.arcTo(r.x(), r.y(), diameter, diameter, 90, 90);
+    path.arcTo(r.x(), r.y()+r.height()-diameter, diameter, diameter, 180, 90);
+    path.arcTo(r.x()+r.width()-diameter, r.y()+r.height()-diameter, diameter, diameter, 270, 90);
+    return path;
+}
