@@ -23,42 +23,58 @@
 
 #include "tageditor.h"
 #include "tags.h"
-#include "musiclibrarymodel.h"
-#include "mpdconnection.h"
-#include "settings.h"
-#include "messagebox.h"
-#include "inputdialog.h"
-#include "localize.h"
+#include "widgets/tagspinbox.h"
+#include "models/musiclibrarymodel.h"
+#include "mpd/mpdconnection.h"
+#include "gui/settings.h"
+#include "support/messagebox.h"
+#include "support/inputdialog.h"
+#include "support/localize.h"
 #include "trackorganiser.h"
-#include "cuefile.h"
-#include "utils.h"
+#include "mpd/cuefile.h"
+#include "support/utils.h"
 #ifdef ENABLE_DEVICES_SUPPORT
-#include "devicesmodel.h"
-#include "device.h"
+#include "models/devicesmodel.h"
+#include "devices/device.h"
 #endif
 #include <QMenu>
 #include <QCloseEvent>
 #include <QCoreApplication>
 #include <QEventLoop>
 #include <QDir>
-#include <QDebug>
+#include <QTimer>
 
 #define REMOVE(w) \
     w->setVisible(false); \
     w->deleteLater(); \
     w=0;
 
-static bool equalTags(const Song &a, const Song &b, bool compareCommon, bool composerSupport)
+//
+// NOTE: Cantata does NOT read, and store, comments from MPD. For comment support, Cantata will read these from the files when the tag editor is opened
+//       These are placed into the Song's 'name' field (as this is not currently used in this editor)
+//
+static bool equalTags(const Song &a, const Song &b, bool compareCommon, bool composerSupport, bool commentSupport)
 {
     return (compareCommon || a.track==b.track) && a.year==b.year && a.disc==b.disc &&
            a.artist==b.artist && a.genre==b.genre && a.album==b.album &&
-           a.albumartist==b.albumartist && (!composerSupport || a.composer==b.composer) && (compareCommon || a.title==b.title);
+           a.albumartist==b.albumartist && (!composerSupport || a.composer()==b.composer()) &&
+           (!commentSupport || a.comment()==b.comment()) && (compareCommon || a.title==b.title);
 }
 
-static void setString(QString &str, const QString &v, bool skipEmpty) {
+static bool setString(QString &str, const QString &v, bool skipEmpty) {
     if (!skipEmpty || !v.isEmpty()) {
         str=v;
+        return true;
     }
+    return false;
+}
+
+static QString trim(QString str) {
+    str=str.trimmed();
+    str=str.simplified();
+    str=str.replace(QLatin1String(" ;"), QLatin1String(";"));
+    str=str.replace(QLatin1String("; "), QLatin1String(";"));
+    return str;
 }
 
 static int iCount=0;
@@ -80,35 +96,22 @@ TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
     , haveArtists(false)
     , haveAlbumArtists(false)
     , haveComposers(false)
+    , haveComments(false)
     , haveAlbums(false)
     , haveGenres(false)
+    , haveDiscs(false)
+    , haveYears(false)
     , saving(false)
-    , composerSupport(true)
+    , composerSupport(false)
+    , commentSupport(false)
 {
     iCount++;
-    foreach (const Song &s, songs) {
-        if (CueFile::isCue(s.file)) {
-            continue;
-        }
-        if (s.guessed) {
-            Song song(s);
-            song.revertGuessedTags();
-            original.append(song);
-        } else {
-            original.append(s);
-        }
-    }
-
-    if (original.isEmpty()) {
-        deleteLater();
-        return;
-    }
-
     bool isMopidy=false;
     #ifdef ENABLE_DEVICES_SUPPORT
     if (deviceUdi.isEmpty()) {
         baseDir=MPDConnection::self()->getDetails().dir;
         composerSupport=MPDConnection::self()->composerTagSupported();
+        commentSupport=MPDConnection::self()->commentTagSupported();
         isMopidy=MPDConnection::self()->isMopdidy();
     } else {
         Device *dev=getDevice(udi, parentWidget());
@@ -123,8 +126,25 @@ TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
     #else
     baseDir=MPDConnection::self()->getDetails().dir;
     composerSupport=MPDConnection::self()->composerTagSupported();
+    commentSupport=MPDConnection::self()->commentTagSupported();
     isMopidy=MPDConnection::self()->isMopdidy();
     #endif
+
+    foreach (const Song &s, songs) {
+        if (CueFile::isCue(s.file)) {
+            continue;
+        }
+        Song song(s);
+        if (s.guessed) {
+            song.revertGuessedTags();
+        }
+        original.append(song);
+    }
+
+    if (original.isEmpty()) {
+        deleteLater();
+        return;
+    }
     qSort(original);
 
     if (!songsOk(original, baseDir, udi.isEmpty())) {
@@ -186,6 +206,14 @@ TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
         REMOVE(composerLabel)
     }
 
+    if (commentSupport) {
+        comment->clear();
+        composer->insertItems(0, strings);
+    } else {
+        REMOVE(comment)
+        REMOVE(commentLabel)
+    }
+
     strings=existingAlbums.toList();
     strings.sort();
     album->clear();
@@ -206,6 +234,7 @@ TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
         QSet<QString> songAlbums;
         QSet<QString> songGenres;
         QSet<QString> songComposers;
+        QSet<QString> songComments;
         QSet<int> songYears;
         QSet<int> songDiscs;
 
@@ -214,11 +243,17 @@ TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
             songAlbumArtists.insert(s.albumartist);
             songAlbums.insert(s.album);
             songGenres.insert(s.genre);
-            songComposers.insert(s.composer);
+            if (composerSupport) {
+                songComposers.insert(s.composer());
+            }
+            if (commentSupport) {
+                songComments.insert(s.comment());
+            }
             songYears.insert(s.year);
             songDiscs.insert(s.disc);
             if (songArtists.count()>1 && songAlbumArtists.count()>1 && songAlbums.count()>1 &&
-                songGenres.count()>1 && songYears.count()>1 && songDiscs.count()>1 && songComposers.count()>1) {
+                songGenres.count()>1 && songYears.count()>1 && songDiscs.count()>1 &&
+                (!composerSupport || songComposers.count()>1) && (!commentSupport || songComments.count()>1)) {
                 break;
             }
         }
@@ -227,7 +262,10 @@ TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
         all.title.clear();
         all.track=0;
         all.artist=1==songArtists.count() ? *(songArtists.begin()) : QString();
-        all.composer=1==songComposers.count() ? *(songComposers.begin()) : QString();
+        if (1==songComposers.count()) {
+            all.setComposer(*(songComposers.begin()));
+        }
+        all.setComment(1==songComments.count() ? *(songComments.begin()) : QString());
         all.albumartist=1==songAlbumArtists.count() ? *(songAlbumArtists.begin()) : QString();
         all.album=1==songAlbums.count() ? *(songAlbums.begin()) : QString();
         all.genre=1==songGenres.count() ? *(songGenres.begin()) : QString();
@@ -240,6 +278,8 @@ TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
         haveAlbums=songAlbums.count()>1;
         haveGenres=songGenres.count()>1;
         haveComposers=songComposers.count()>1;
+        haveDiscs=songDiscs.count()>1;
+        haveYears=songYears.count()>1;
     } else {
         title->setFocus();
     }
@@ -263,6 +303,9 @@ TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
         connect(composer, SIGNAL(activated(int)), SLOT(checkChanged()));
         connect(composer, SIGNAL(editTextChanged(const QString &)), SLOT(checkChanged()));
     }
+    if (commentSupport) {
+        connect(comment, SIGNAL(textChanged(const QString &)), SLOT(checkChanged()));
+    }
     connect(album, SIGNAL(activated(int)), SLOT(checkChanged()));
     connect(album, SIGNAL(editTextChanged(const QString &)), SLOT(checkChanged()));
     connect(genre, SIGNAL(activated(int)), SLOT(checkChanged()));
@@ -273,11 +316,13 @@ TagEditor::TagEditor(QWidget *parent, const QList<Song> &songs,
     connect(trackName, SIGNAL(activated(int)), SLOT(setIndex(int)));
     connect(this, SIGNAL(update()), MPDConnection::self(), SLOT(update()));
     adjustSize();
-    int w=600*(Utils::isHighDpi() ? 2 : 1);
+    int w=Utils::scaleForDpi(600);
     if (width()<w) {
         resize(w, height());
     }
-    setMaximumHeight(height());
+    if (commentSupport) {
+        QTimer::singleShot(0, this, SLOT(readComments()));
+    }
 }
 
 TagEditor::~TagEditor()
@@ -297,34 +342,58 @@ void TagEditor::fillSong(Song &s, bool isAll, bool skipEmpty) const
     setString(s.album, album->text().trimmed(), skipEmpty && (!haveAll || all.album.isEmpty()));
     setString(s.albumartist, albumArtist->text().trimmed(), skipEmpty && (!haveAll || all.albumartist.isEmpty()));
     if (composerSupport) {
-        setString(s.composer, composer->text().trimmed(), skipEmpty && (!haveAll || all.composer.isEmpty()));
+        QString str;
+        if (setString(str, composer->text().trimmed(), skipEmpty && (!haveAll || all.composer().isEmpty()))) {
+            s.setComposer(str);
+        }
+    }
+    if (commentSupport) {
+        QString str;
+        if (setString(str, comment->text().trimmed(), skipEmpty && (!haveAll || all.comment().isEmpty()))) {
+            s.setComment(str);
+        }
     }
-
     if (!isAll) {
         s.track=track->value();
     }
     if (!isAll || 0!=disc->value()) {
         s.disc=disc->value();
     }
-    setString(s.genre, genre->text().trimmed(), skipEmpty && (!haveAll || all.genre.isEmpty()));
+    setString(s.genre, trim(genre->text()), skipEmpty && (!haveAll || all.genre.isEmpty()));
     if (!isAll || 0!=year->value()) {
         s.year=year->value();
     }
 }
 
-void TagEditor::setPlaceholderTexts()
+void TagEditor::setVariousHint()
 {
-    QString various=i18n("(Various)");
-
     if(0==currentSongIndex && original.count()>1) {
         Song all=original.at(0);
-        artist->setPlaceholderText(all.artist.isEmpty() && haveArtists ? various : QString());
-        album->setPlaceholderText(all.album.isEmpty() && haveAlbums ? various : QString());
-        albumArtist->setPlaceholderText(all.albumartist.isEmpty() && haveAlbumArtists ? various : QString());
+        artist->setPlaceholderText(all.artist.isEmpty() && haveArtists ? TagSpinBox::variousStr() : QString());
+        album->setPlaceholderText(all.album.isEmpty() && haveAlbums ? TagSpinBox::variousStr() : QString());
+        albumArtist->setPlaceholderText(all.albumartist.isEmpty() && haveAlbumArtists ? TagSpinBox::variousStr() : QString());
         if (composerSupport) {
-            composer->setPlaceholderText(all.composer.isEmpty() && haveComposers ? various : QString());
+            composer->setPlaceholderText(all.composer().isEmpty() && haveComposers ? TagSpinBox::variousStr() : QString());
         }
-        genre->setPlaceholderText(all.genre.isEmpty() && haveGenres ? various : QString());
+        if (commentSupport) {
+            comment->setPlaceholderText(all.comment().isEmpty() && haveComments ? TagSpinBox::variousStr() : QString());
+        }
+        genre->setPlaceholderText(all.genre.isEmpty() && haveGenres ? TagSpinBox::variousStr() : QString());
+        disc->setVarious(0==all.disc && haveDiscs);
+        year->setVarious(0==all.year && haveYears);
+    } else {
+        artist->setPlaceholderText(QString());
+        album->setPlaceholderText(QString());
+        albumArtist->setPlaceholderText(QString());
+        if (composerSupport) {
+            composer->setPlaceholderText(QString());
+        }
+        if (commentSupport) {
+            comment->setPlaceholderText(QString());
+        }
+        genre->setPlaceholderText(QString());
+        disc->setVarious(false);
+        year->setVarious(false);
     }
 }
 
@@ -345,7 +414,10 @@ void TagEditor::setLabelStates()
     titleLabel->setOn(!isAll && o.title!=e.title);
     artistLabel->setOn(o.artist!=e.artist);
     if (composerSupport) {
-        composerLabel->setOn(o.composer!=e.composer);
+        composerLabel->setOn(o.composer()!=e.composer());
+    }
+    if (commentSupport) {
+        commentLabel->setOn(o.comment()!=e.comment());
     }
     albumArtistLabel->setOn(o.albumartist!=e.albumartist);
     albumLabel->setOn(o.album!=e.album);
@@ -355,6 +427,42 @@ void TagEditor::setLabelStates()
     yearLabel->setOn(o.year!=e.year);
 }
 
+void TagEditor::readComments()
+{
+    progress->setVisible(true);
+    progress->setRange(0, original.count());
+    bool haveMultiple=original.count()>1;
+    bool updated=false;
+
+    for (int i=0; i<original.count(); ++i) {
+        progress->setValue(i+1);
+        if (i && 0==i%10) {
+            QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
+        }
+
+        if (0==i && haveMultiple) {
+            continue;
+        }
+        Song song=original.at(i);
+        QString comment=Tags::readComment(baseDir+song.file);
+        if (!comment.isEmpty()) {
+            song.setComment(comment);
+            original.replace(i, song);
+            haveComments=true;
+            updated=true;
+        }
+    }
+    if (updated) {
+        edited=original;
+    }
+    progress->setVisible(false);
+    if (haveMultiple) {
+        setVariousHint();
+    } else {
+        setSong(original.at(0));
+    }
+}
+
 void TagEditor::applyVa()
 {
     bool isAll=0==currentSongIndex && original.count()>1;
@@ -528,7 +636,7 @@ void TagEditor::adjustTrackNumbers()
     }
 
     if (isAll) {
-        for (int i=0; i<edited.count(); ++i) {
+        for (int i=1; i<edited.count(); ++i) {
             Song s=edited.at(i);
             s.track+=adj;
             edited.replace(i, s);
@@ -594,7 +702,7 @@ void TagEditor::updateEditedStatus(int index)
 {
     bool isAll=0==index && original.count()>1;
     Song s=edited.at(index);
-    if (equalTags(s, original.at(index), isAll, composerSupport)) {
+    if (equalTags(s, original.at(index), isAll, composerSupport, commentSupport)) {
         if (editedIndexes.contains(index)) {
             editedIndexes.remove(index);
             updateTrackName(index, false);
@@ -624,8 +732,11 @@ void TagEditor::updateEdited(bool isFromAll)
         if (all.albumartist.isEmpty() && s.albumartist.isEmpty() && !o.albumartist.isEmpty()) {
             s.albumartist=o.albumartist;
         }
-        if (composerSupport && all.composer.isEmpty() && s.composer.isEmpty() && !o.composer.isEmpty()) {
-            s.composer=o.composer;
+        if (composerSupport && all.composer().isEmpty() && s.composer().isEmpty() && !o.composer().isEmpty()) {
+            s.setComposer(o.composer());
+        }
+        if (commentSupport && all.comment().isEmpty() && s.comment().isEmpty() && !o.comment().isEmpty()) {
+            s.setComment(o.comment());
         }
         if (all.album.isEmpty() && s.album.isEmpty() && !o.album.isEmpty()) {
             s.album=o.album;
@@ -635,7 +746,7 @@ void TagEditor::updateEdited(bool isFromAll)
         }
     }
 
-    if (equalTags(s, original.at(currentSongIndex), isFromAll || isAll, composerSupport)) {
+    if (equalTags(s, original.at(currentSongIndex), isFromAll || isAll, composerSupport, commentSupport)) {
         if (editedIndexes.contains(currentSongIndex)) {
             editedIndexes.remove(currentSongIndex);
             updateTrackName(currentSongIndex, false);
@@ -656,7 +767,10 @@ void TagEditor::setSong(const Song &s)
     artist->setText(s.artist);
     albumArtist->setText(s.albumartist);
     if (composerSupport) {
-        composer->setText(s.composer);
+        composer->setText(s.composer());
+    }
+    if (commentSupport) {
+        comment->setText(s.comment());
     }
     album->setText(s.album);
     track->setValue(s.track);
@@ -695,7 +809,7 @@ void TagEditor::setIndex(int idx)
         enableButton(User1, !isMultiple && idx<(original.count()-1)); // Next
         enableButton(User2, !isMultiple && idx>1); // Prev
     }
-    setPlaceholderTexts();
+    setVariousHint();
     enableOkButton();
     trackName->setCurrentIndex(idx);
     setLabelStates();
@@ -758,13 +872,14 @@ bool TagEditor::applyUpdates()
         }
         Song orig=original.at(idx);
         Song edit=edited.at(idx);
-        if (equalTags(orig, edit, false, composerSupport)) {
+        if (equalTags(orig, edit, false, composerSupport, commentSupport)) {
             continue;
         }
 
         QString file=orig.filePath();
-        switch(Tags::update(baseDir+file, orig, edit)) {
+        switch(Tags::update(baseDir+file, orig, edit, -1, commentSupport)) {
         case Tags::Update_Modified:
+            edit.setComment(QString());
             #ifdef ENABLE_DEVICES_SUPPORT
             if (!deviceUdi.isEmpty()) {
                 if (!dev->updateSong(orig, edit)) {
@@ -787,15 +902,9 @@ bool TagEditor::applyUpdates()
         case Tags::Update_Failed:
             failed.append(file);
             break;
-        #ifdef ENABLE_EXTERNAL_TAGS
-        case Tags::Update_Timedout:
-            failed.append(i18nc("filename (Timeout)", "%1 (Timeout)", file));
-            someTimedout=true;
-            break;
         case Tags::Update_BadFile:
             failed.append(i18nc("filename (Corrupt tags?)", "%1 (Corrupt tags?)", file));
             break;
-        #endif
         default:
             break;
         }
