@@ -24,7 +24,7 @@
 #include "tags.h"
 #include "config.h"
 #include "filetyperesolver.h"
-#include "utils.h"
+#include "support/utils.h"
 #include <algorithm>
 #include <cctype>
 #include <cmath>
@@ -36,6 +36,13 @@
 #include <QString>
 #include <QStringList>
 #include <QTextCodec>
+#include <QDebug>
+#define TAGLIB_VERSION CANTATA_MAKE_VERSION(TAGLIB_MAJOR_VERSION, TAGLIB_MINOR_VERSION, TAGLIB_PATCH_VERSION)
+
+#include <taglib/taglib.h>
+#if TAGLIB_VERSION >= CANTATA_MAKE_VERSION(1,8,0)
+#include <taglib/tpropertymap.h>
+#endif
 #include <taglib/fileref.h>
 #include <taglib/aifffile.h>
 #ifdef TAGLIB_ASF_FOUND
@@ -63,7 +70,6 @@
 #endif
 #include <taglib/apetag.h>
 #include <taglib/id3v2tag.h>
-#include <taglib/id3v1tag.h>
 #include <taglib/id3v1genres.h>
 #ifdef TAGLIB_MP4_FOUND
 #include <taglib/mp4tag.h>
@@ -91,6 +97,14 @@ static QMutex mutex;
 namespace Tags
 {
 
+static bool debugEnabled=false;
+#define DBUG if (debugEnabled) qWarning() << "Tags" << __FUNCTION__
+
+void enableDebug()
+{
+    debugEnabled=true;
+}
+
 static QString tString2QString(const TagLib::String &str)
 {
     static QTextCodec *codec = QTextCodec::codecForName("UTF-8");
@@ -169,7 +183,7 @@ static TagLib::FileRef getFileRef(const QString &path)
 {
     ensureFileTypeResolvers();
 
-    #ifdef Q_OS_WIN32
+    #ifdef Q_OS_WIN
     return TagLib::FileRef(reinterpret_cast<const wchar_t*>(path.constData()), true, TagLib::AudioProperties::Fast);
     #else
     return TagLib::FileRef(QFile::encodeName(path).constData(), true, TagLib::AudioProperties::Fast);
@@ -274,7 +288,7 @@ static void readID3v2Tags(TagLib::ID3v2::Tag *tag, Song *song, ReplayGain *rg, Q
         const TagLib::ID3v2::FrameList &composer = tag->frameListMap()["TCOM"];
 
         if (!composer.isEmpty()) {
-            song->composer=tString2QString(composer.front()->toString());
+            song->setComposer(tString2QString(composer.front()->toString()));
         }
 
         const TagLib::ID3v2::FrameList &disc = tag->frameListMap()["TPOS"];
@@ -282,6 +296,42 @@ static void readID3v2Tags(TagLib::ID3v2::Tag *tag, Song *song, ReplayGain *rg, Q
         if (!disc.isEmpty()) {
             song->disc=splitDiscNumber(tString2QString(disc.front()->toString())).first;
         }
+
+        const TagLib::ID3v2::FrameList &tcon = tag->frameListMap()["TCON"];
+        if (!tcon.isEmpty()) {
+            TagLib::ID3v2::FrameList::ConstIterator it = tcon.begin();
+            TagLib::ID3v2::FrameList::ConstIterator end = tcon.end();
+            QSet<QString> genres;
+
+            for (; it!=end; ++it) {
+                TagLib::ID3v2::TextIdentificationFrame *f = static_cast<TagLib::ID3v2::TextIdentificationFrame *>(*it);
+                TagLib::StringList fields = f->fieldList();
+                TagLib::StringList::ConstIterator fIt = fields.begin();
+                TagLib::StringList::ConstIterator fEnd = fields.end();
+
+                for (; fIt!=fEnd; ++fIt) {
+                    if ((*fIt).isEmpty()) {
+                        continue;
+                    }
+
+                    bool ok;
+                    int number = (*fIt).toInt(&ok);
+                    QString genre;
+                    if (ok && number >= 0 && number <= 255) {
+                        genre = tString2QString(TagLib::ID3v1::genre(number));
+                    } else {
+                        genre = tString2QString(*fIt);
+                    }
+                    if (!genres.contains(genre)) {
+                        genres.insert(genre);
+                        if (!song->genre.isEmpty()) {
+                            song->genre+=QLatin1Char(';');
+                        }
+                        song->genre+=genre;
+                    }
+                }
+            }
+        }
     }
 
     if (rg) {
@@ -390,12 +440,30 @@ static bool writeID3v2Tags(TagLib::ID3v2::Tag *tag, const Song &from, const Song
         if (from.albumartist!=to.albumartist && updateID3v2Tag(tag, "TPE2", to.albumartist)) {
             changed=true;
         }
-        if (from.composer!=to.composer && updateID3v2Tag(tag, "TCOM", to.composer)) {
+        if (from.composer()!=to.composer() && updateID3v2Tag(tag, "TCOM", to.composer())) {
             changed=true;
         }
         if (from.disc!=to.disc&& updateID3v2Tag(tag, "TPOS", 0==to.disc ? QString() : QString::number(to.disc))) {
             changed=true;
         }
+        DBUG << "genres" << from.genre << to.genre;
+        if (from.genre!=to.genre) {
+            tag->removeFrames("TCON");
+            QStringList genres=to.genres();
+            DBUG << "num genres:" << genres.count();
+            if (genres.count()<2) {
+                DBUG << "set genre" << (genres.isEmpty() ? QString() : genres.first().trimmed());
+                tag->setGenre(qString2TString(genres.isEmpty() ? QString() : genres.first().trimmed()));
+            } else {
+                foreach (const QString &genre, genres) {
+                    TagLib::ID3v2::TextIdentificationFrame *frame = new TagLib::ID3v2::TextIdentificationFrame("TCON");
+                    tag->addFrame(frame);
+                    DBUG << "add genre" << genre.trimmed();
+                    frame->setText(qString2TString(genre.trimmed()));
+                }
+            }
+            changed=true;
+        }
     }
 
     if (!rg.null) {
@@ -434,7 +502,7 @@ static bool writeID3v2Tags(TagLib::ID3v2::Tag *tag, const Song &from, const Song
     return changed;
 }
 
-static void readAPETags(TagLib::APE::Tag *tag, Song *song, ReplayGain *rg)
+static void readAPETags(TagLib::APE::Tag *tag, Song *song, ReplayGain *rg, QImage *img)
 {
     const TagLib::APE::ItemListMap &map = tag->itemListMap();
 
@@ -443,11 +511,19 @@ static void readAPETags(TagLib::APE::Tag *tag, Song *song, ReplayGain *rg)
             song->albumartist=tString2QString(map["Album Artist"].toString());
         }
         if (map.contains("Composer")) {
-            song->composer=tString2QString(map["Composer"].toString());
+            song->setComposer(tString2QString(map["Composer"].toString()));
         }
         if (map.contains("Disc")) {
             song->disc=splitDiscNumber(tString2QString(map["Disc"].toString())).first;
         }
+        if (map.contains("GENRE")) {
+            TagLib::StringList genres=map["GENRE"].values();
+            TagLib::StringList::ConstIterator it=genres.begin();
+            TagLib::StringList::ConstIterator end=genres.end();
+            for (; it!=end; ++it) {
+                song->addGenre(tString2QString(*it));
+            }
+        }
     }
 
     if (rg) {
@@ -464,6 +540,21 @@ static void readAPETags(TagLib::APE::Tag *tag, Song *song, ReplayGain *rg)
             rg->albumPeak=parseRgString(map["replaygain_album_peak"].toString());
         }
     }
+
+    if (img) {
+        if (map.contains("COVER ART (FRONT)")) {
+            const TagLib::ByteVector nullStringTerminator(1, 0);
+
+            TagLib::ByteVector item = map["COVER ART (FRONT)"].value();
+            int pos = item.find(nullStringTerminator);   // Skip the filename
+
+            if (++pos > 0) {
+                const TagLib::ByteVector &bytes=item.mid(pos);
+                QByteArray data(bytes.data(), bytes.size());
+                img->loadFromData(data);
+            }
+        }
+    }
 }
 
 static bool updateAPETag(TagLib::APE::Tag *tag, const char *tagName, const QString &value)
@@ -490,12 +581,24 @@ static bool writeAPETags(TagLib::APE::Tag *tag, const Song &from, const Song &to
         if (from.albumartist!=to.albumartist && updateAPETag(tag, "Album Artist", to.albumartist)) {
             changed=true;
         }
-        if (from.composer!=to.composer && updateAPETag(tag, "Composer", to.composer)) {
+        if (from.composer()!=to.composer() && updateAPETag(tag, "Composer", to.composer())) {
             changed=true;
         }
         if (from.disc!=to.disc && updateAPETag(tag, "Disc", 0==to.disc ? QString() : QString::number(to.disc))) {
             changed=true;
         }
+        if (from.genre!=to.genre) {
+            tag->removeItem("GENRE");
+            QStringList genres=to.genres();
+            if (genres.count()<2) {
+                tag->setGenre(qString2TString(genres.isEmpty() ? QString() : genres.first().trimmed()));
+            } else {
+                foreach (const QString &genre, genres) {
+                    tag->addValue("GENRE", qString2TString(genre.trimmed()), false);
+                }
+            }
+            changed=true;
+        }
     }
 
     if (!rg.null) {
@@ -518,7 +621,6 @@ static TagLib::String readVorbisTag(TagLib::Ogg::XiphComment *tag, const char *f
 {
     if (tag->contains(field)) {
         const TagLib::StringList &list = tag->fieldListMap()[field];
-
         if (!list.isEmpty()) {
             return list.front();
         }
@@ -536,12 +638,22 @@ static void readVorbisCommentTags(TagLib::Ogg::XiphComment *tag, Song *song, Rep
         }
         str=readVorbisTag(tag, "COMPOSER");
         if (str.isEmpty()) {
-            song->composer=tString2QString(str);
+            song->setComposer(tString2QString(str));
         }
         str=readVorbisTag(tag, "DISCNUMBER");
         if (str.isEmpty()) {
             song->disc=splitDiscNumber(tString2QString(str)).first;
         }
+        if (tag->contains("GENRE")) {
+            const TagLib::StringList &genres = tag->fieldListMap()["GENRE"];
+            if (!genres.isEmpty()) {
+                TagLib::StringList::ConstIterator it=genres.begin();
+                TagLib::StringList::ConstIterator end=genres.end();
+                for (; it!=end; ++it) {
+                    song->addGenre(tString2QString(*it));
+                }
+            }
+        }
     }
 
     if (rg) {
@@ -553,9 +665,30 @@ static void readVorbisCommentTags(TagLib::Ogg::XiphComment *tag, Song *song, Rep
 
     if (img) {
         TagLib::Ogg::FieldListMap map = tag->fieldListMap();
-        // Ogg lacks a definitive standard for embedding cover art, but it seems
-        // b64 encoding a field called COVERART is the general convention
-        if (map.contains("COVERART")) {
+        #if TAGLIB_VERSION >= CANTATA_MAKE_VERSION(1,7,0)
+        // METADATA_BLOCK_PICTURE is the Ogg standard way of encoding a covers.
+        // https://wiki.xiph.org/index.php/VorbisComment#Cover_art
+        if (map.contains("METADATA_BLOCK_PICTURE")) {
+            TagLib::StringList block=map["METADATA_BLOCK_PICTURE"];
+            if (!block.isEmpty()) {
+                TagLib::StringList::ConstIterator i = block.begin();
+                TagLib::StringList::ConstIterator end = block.end();
+
+                for (; i != end; ++i ) {
+                    QByteArray data(QByteArray::fromBase64(i->to8Bit().c_str()));
+                    TagLib::ByteVector bytes(data.data(), data.size());
+                    TagLib::FLAC::Picture pic;
+
+                    if (pic.parse(bytes)) {
+                        img->loadFromData(QByteArray(pic.data().data(), pic.data().size()));
+                    }
+                }
+            }
+        }
+        #endif
+
+        // COVERART is an older (now deprecated) way of storing covers...
+        if (img->isNull() && map.contains("COVERART")) {
             QByteArray data=map["COVERART"].toString().toCString();
             img->loadFromData(QByteArray::fromBase64(data));
             if (img->isNull()) {
@@ -572,7 +705,7 @@ static void readVorbisCommentTags(TagLib::Ogg::XiphComment *tag, Song *song, Rep
     }
 }
 
-#if (TAGLIB_MAJOR_VERSION > 1) || (TAGLIB_MAJOR_VERSION == 1 && TAGLIB_MINOR_VERSION >= 7)
+#if TAGLIB_VERSION >= CANTATA_MAKE_VERSION(1,7,0)
 static void readFlacPicture(const TagLib::List<TagLib::FLAC::Picture*> &pics, QImage *img)
 {
     if (!pics.isEmpty() && 1==pics.size()) {
@@ -606,12 +739,24 @@ static bool writeVorbisCommentTags(TagLib::Ogg::XiphComment *tag, const Song &fr
         if (from.albumartist!=to.albumartist && updateVorbisCommentTag(tag, "ALBUMARTIST", to.albumartist)) {
             changed=true;
         }
-        if (from.composer!=to.composer && updateVorbisCommentTag(tag, "COMPOSER", to.composer)) {
+        if (from.composer()!=to.composer() && updateVorbisCommentTag(tag, "COMPOSER", to.composer())) {
             changed=true;
         }
         if (from.disc!=to.disc && updateVorbisCommentTag(tag, "DISCNUMBER", 0==to.disc ? QString() : QString::number(to.disc))) {
             changed=true;
         }
+        if (from.genre!=to.genre) {
+            tag->removeField("GENRE");
+            QStringList genres=to.genres();
+            if (genres.count()<2) {
+                tag->setGenre(qString2TString(genres.isEmpty() ? QString() : genres.first().trimmed()));
+            } else {
+                foreach (const QString &genre, genres) {
+                    tag->addField("GENRE", qString2TString(genre.trimmed()), false);
+                }
+            }
+            changed=true;
+        }
     }
 
     if (!rg.null) {
@@ -649,11 +794,19 @@ static void readMP4Tags(TagLib::MP4::Tag *tag, Song *song, ReplayGain *rg, QImag
             song->albumartist=tString2QString(map["aART"].toStringList().front());
         }
         if (map.contains("\xA9wrt") && !map["\xA9wrt"].toStringList().isEmpty()) {
-            song->composer=tString2QString(map["\xA9wrt"].toStringList().front());
+            song->setComposer(tString2QString(map["\xA9wrt"].toStringList().front()));
         }
         if (map.contains("disk")) {
             song->disc=map["disk"].toIntPair().first;
         }
+        if (map.contains("\251gen")) {
+            TagLib::StringList genres=map["\251gen"].toStringList();
+            TagLib::StringList::ConstIterator it=genres.begin();
+            TagLib::StringList::ConstIterator end=genres.end();
+            for (; it!=end; ++it) {
+                song->addGenre(tString2QString(*it));
+            }
+        }
     }
     if (rg) {
         if (map.contains("----:com.apple.iTunes:replaygain_track_gain")) {
@@ -691,7 +844,7 @@ static bool updateMP4Tag(TagLib::MP4::Tag *tag, const char *tagName, const QStri
             return true;
         }
     } else {
-        map.insert("aART", TagLib::StringList(qString2TString(value)));
+        map.insert(tagName, TagLib::StringList(qString2TString(value)));
         return true;
     }
     return false;
@@ -705,12 +858,25 @@ static bool writeMP4Tags(TagLib::MP4::Tag *tag, const Song &from, const Song &to
         if (from.albumartist!=to.albumartist && updateMP4Tag(tag, "aART", to.albumartist)) {
             changed=true;
         }
-        if (from.composer!=to.composer && updateMP4Tag(tag, "\xA9wrt", to.composer)) {
+        if (from.composer()!=to.composer() && updateMP4Tag(tag, "\xA9wrt", to.composer())) {
             changed=true;
         }
         if (from.disc!=to.disc && updateMP4Tag(tag, "disk", 0==to.disc ? QString() : QString::number(to.disc))) {
             changed=true;
         }
+        if (from.genre!=to.genre) {
+            QStringList genres=to.genres();
+            if (genres.count()<2) {
+                tag->setGenre(qString2TString(genres.isEmpty() ? QString() : genres.first().trimmed()));
+            } else {
+                TagLib::StringList tagGenres;
+                foreach (const QString &genre, genres) {
+                    tagGenres.append(qString2TString(genre.trimmed()));
+                }
+                tag->itemListMap()["\251gen"]=tagGenres;
+            }
+            changed=true;
+        }
     }
 
     if (!rg.null) {
@@ -754,11 +920,19 @@ static void readASFTags(TagLib::ASF::Tag *tag, Song *song, ReplayGain *rg)
             song->albumartist=tString2QString(map["WM/AlbumTitle"].front().toString());
         }
         if (map.contains("WM/Composer") && !map["WM/Composer"].isEmpty()) {
-            song->composer=tString2QString(map["WM/Composer"].front().toString());
+            song->setComposer(tString2QString(map["WM/Composer"].front().toString()));
         }
-        if (map.contains("WM/PartOfSet")) {
+        if (map.contains("WM/PartOfSet") && !map["WM/PartOfSet"].isEmpty()) {
             song->albumartist=map["WM/PartOfSet"].front().toUInt();
         }
+        if (map.contains("WM/Genre") && !map["WM/Genre"].isEmpty()) {
+            const TagLib::ASF::AttributeList &genres=map["WM/Genre"];
+            TagLib::ASF::AttributeList::ConstIterator it=genres.begin();
+            TagLib::ASF::AttributeList::ConstIterator end=genres.end();
+            for (; it!=end; ++it) {
+                song->addGenre(tString2QString((*it).toString()));
+            }
+        }
     }
 }
 
@@ -787,12 +961,24 @@ static bool writeASFTags(TagLib::ASF::Tag *tag, const Song &from, const Song &to
         if (from.albumartist!=to.albumartist && updateASFTag(tag, "WM/AlbumTitle", to.albumartist)) {
             changed=true;
         }
-        if (from.composer!=to.composer && updateASFTag(tag, "WM/Composer", to.composer)) {
+        if (from.composer()!=to.composer() && updateASFTag(tag, "WM/Composer", to.composer())) {
             changed=true;
         }
         if (from.disc!=to.disc && updateASFTag(tag, "WM/PartOfSet", 0==to.disc ? QString() : QString::number(to.disc))) {
             changed=true;
         }
+        if (from.genre!=to.genre) {
+            tag->removeItem("WM/Genre");
+            QStringList genres=to.genres();
+            if (genres.count()<2) {
+                tag->setGenre(qString2TString(genres.isEmpty() ? QString() : genres.first().trimmed()));
+            } else {
+                foreach (const QString &genre, genres) {
+                    tag->addAttribute("WM/Genre", qString2TString(genre.trimmed()));
+                }
+            }
+            changed=true;
+        }
     }
     return changed;
 }
@@ -805,7 +991,7 @@ static void readTags(const TagLib::FileRef fileref, Song *song, ReplayGain *rg,
         song->title=tString2QString(tag->title());
         song->artist=tString2QString(tag->artist());
         song->album=tString2QString(tag->album());
-        song->genre=tString2QString(tag->genre());
+//        song->genre=tString2QString(tag->genre());
         song->track=tag->track();
         song->year=tag->year();
     }
@@ -814,7 +1000,7 @@ static void readTags(const TagLib::FileRef fileref, Song *song, ReplayGain *rg,
         if (file->ID3v2Tag() && !file->ID3v2Tag()->isEmpty()) {
             readID3v2Tags(file->ID3v2Tag(), song, rg, img, lyrics);
         } else if (file->APETag()) {
-            readAPETags(file->APETag(), song, rg);
+            readAPETags(file->APETag(), song, rg, img);
 //         } else if (file->ID3v1Tag()) {
 //             readID3v1Tags(fileref, song, rg);
         }
@@ -839,7 +1025,7 @@ static void readTags(const TagLib::FileRef fileref, Song *song, ReplayGain *rg,
     } else if (TagLib::FLAC::File *file = dynamic_cast< TagLib::FLAC::File * >(fileref.file())) {
         if (file->xiphComment()) {
             readVorbisCommentTags(file->xiphComment(), song, rg, img, lyrics);
-            #if (TAGLIB_MAJOR_VERSION > 1) || (TAGLIB_MAJOR_VERSION == 1 && TAGLIB_MINOR_VERSION >= 7)
+            #if TAGLIB_VERSION >= CANTATA_MAKE_VERSION(1,7,0)
             if (img && img->isNull()) {
                 readFlacPicture(file->pictureList(), img);
             }
@@ -858,7 +1044,7 @@ static void readTags(const TagLib::FileRef fileref, Song *song, ReplayGain *rg,
     #endif
     } else if (TagLib::MPC::File *file = dynamic_cast< TagLib::MPC::File * >(fileref.file())) {
         if (file->APETag()) {
-            readAPETags(file->APETag(), song, rg);
+            readAPETags(file->APETag(), song, rg, img);
 //         } else if (file->ID3v1Tag()) {
 //             readID3v1Tags(fileref, song, rg);
         }
@@ -885,14 +1071,17 @@ static void readTags(const TagLib::FileRef fileref, Song *song, ReplayGain *rg,
         }
     } else if (TagLib::WavPack::File *file = dynamic_cast< TagLib::WavPack::File * >(fileref.file())) {
         if (file->APETag()) {
-            readAPETags(file->APETag(), song, rg);
+            readAPETags(file->APETag(), song, rg, img);
 //         } else if (file->ID3v1Tag()) {
 //             readID3v1Tags(fileref, song, rg);
         }
     }
+    if (song && song->genre.isEmpty()) {
+        song->genre=tString2QString(tag->genre());
+    }
 }
 
-static bool writeTags(const TagLib::FileRef fileref, const Song &from, const Song &to, const RgTags &rg, const QByteArray &img)
+static bool writeTags(const TagLib::FileRef fileref, const Song &from, const Song &to, const RgTags &rg, const QByteArray &img, bool saveComment)
 {
     bool changed=false;
     TagLib::Tag *tag=fileref.tag();
@@ -909,10 +1098,10 @@ static bool writeTags(const TagLib::FileRef fileref, const Song &from, const Son
             tag->setAlbum(qString2TString(to.album));
             changed=true;
         }
-        if (from.genre!=to.genre) {
-            tag->setGenre(qString2TString(to.genre));
-            changed=true;
-        }
+//        if (from.genre!=to.genre) {
+//            tag->setGenre(qString2TString(to.genre));
+//            changed=true;
+//        }
         if (from.track!=to.track) {
             tag->setTrack(to.track);
             changed=true;
@@ -921,6 +1110,10 @@ static bool writeTags(const TagLib::FileRef fileref, const Song &from, const Son
             tag->setYear(to.year);
             changed=true;
         }
+        if (saveComment && from.comment()!=to.comment()) {
+            tag->setComment(qString2TString(to.comment()));
+            changed=true;
+        }
     }
     if (TagLib::MPEG::File *file = dynamic_cast< TagLib::MPEG::File * >(fileref.file())) {
         if (file->ID3v2Tag() && !file->ID3v2Tag()->isEmpty()) {
@@ -1046,21 +1239,27 @@ QString readLyrics(const QString &fileName)
     return lyrics;
 }
 
-static Update update(const TagLib::FileRef fileref, const Song &from, const Song &to, const RgTags &rg, const QByteArray &img, int id3Ver=-1)
+QString readComment(const QString &fileName)
 {
-    if (writeTags(fileref, from, to, rg, img)) {
+    LOCK_MUTEX
+    TagLib::FileRef fileref = getFileRef(fileName);
+    return fileref.isNull() ? QString() : tString2QString(fileref.tag()->comment());
+}
+
+static Update update(const TagLib::FileRef fileref, const Song &from, const Song &to, const RgTags &rg, const QByteArray &img, int id3Ver=-1, bool saveComment=false)
+{
+    if (writeTags(fileref, from, to, rg, img, saveComment)) {
         TagLib::MPEG::File *mpeg=dynamic_cast<TagLib::MPEG::File *>(fileref.file());
         if (mpeg) {
-            TagLib::ID3v1::Tag *v1=mpeg->ID3v1Tag(false);
-            bool haveV1=v1 && (!v1->title().isEmpty() || !v1->artist().isEmpty() || !v1->album().isEmpty());
             #ifdef TAGLIB_CAN_SAVE_ID3VER
             TagLib::ID3v2::Tag *v2=mpeg->ID3v2Tag(false);
             bool isID3v24=v2 && isId3V24(v2->header());
             int ver=id3Ver==3 ? 3 : (id3Ver==4 ? 4 : (isID3v24 ? 4 : 3));
-            return mpeg->save((haveV1 ? TagLib::MPEG::File::ID3v1 : 0)|TagLib::MPEG::File::ID3v2, true, ver) ? Update_Modified : Update_Failed;
+            DBUG << "isID3v24" << isID3v24 << "reqVer:" << id3Ver << "use:" << ver;
+            return mpeg->save(TagLib::MPEG::File::ID3v2, true, ver) ? Update_Modified : Update_Failed;
             #else
             Q_UNUSED(id3Ver)
-            return mpeg->save((haveV1 ? TagLib::MPEG::File::ID3v1 : 0)|TagLib::MPEG::File::ID3v2, true) ? Update_Modified : Update_Failed;
+            return mpeg->save(TagLib::MPEG::File::ID3v2, true) ? Update_Modified : Update_Failed;
             #endif
         }
         return fileref.file()->save() ? Update_Modified : Update_Failed;
@@ -1082,24 +1281,23 @@ Update updateArtistAndTitle(const QString &fileName, const Song &song)
     tag->setArtist(qString2TString(song.artist));
 
     if (mpeg) {
-        TagLib::ID3v1::Tag *v1=mpeg->ID3v1Tag(false);
-        bool haveV1=v1 && (!v1->title().isEmpty() || !v1->artist().isEmpty() || !v1->album().isEmpty());
         #ifdef TAGLIB_CAN_SAVE_ID3VER
         TagLib::ID3v2::Tag *v2=mpeg->ID3v2Tag(false);
         int ver=v2 && isId3V24(v2->header()) ? 4 : 3;
-        return mpeg->save((haveV1 ? TagLib::MPEG::File::ID3v1 : 0)|TagLib::MPEG::File::ID3v2, true, ver) ? Update_Modified : Update_Failed;
+        DBUG << "useId3ver:" << ver;
+        return mpeg->save(TagLib::MPEG::File::ID3v2, true, ver) ? Update_Modified : Update_Failed;
         #else
-        return mpeg->save((haveV1 ? TagLib::MPEG::File::ID3v1 : 0)|TagLib::MPEG::File::ID3v2, true) ? Update_Modified : Update_Failed;
+        return mpeg->save(TagLib::MPEG::File::ID3v2, true) ? Update_Modified : Update_Failed;
         #endif
     }
     return fileref.file()->save() ? Update_Modified : Update_Failed;
 }
 
-Update update(const QString &fileName, const Song &from, const Song &to, int id3Ver)
+Update update(const QString &fileName, const Song &from, const Song &to, int id3Ver, bool saveComment)
 {
     LOCK_MUTEX
     TagLib::FileRef fileref = getFileRef(fileName);
-    return fileref.isNull() ? Update_Failed : update(fileref, from, to, RgTags(), QByteArray(), id3Ver);
+    return fileref.isNull() ? Update_Failed : update(fileref, from, to, RgTags(), QByteArray(), id3Ver, saveComment);
 }
 
 ReplayGain readReplaygain(const QString &fileName)
@@ -1132,7 +1330,7 @@ Update embedImage(const QString &fileName, const QByteArray &cover)
 QString oggMimeType(const QString &fileName)
 {
     LOCK_MUTEX
-    #ifdef Q_OS_WIN32
+    #ifdef Q_OS_WIN
     const wchar_t*encodedName=reinterpret_cast<const wchar_t*>(fileName.constData());
     #else
     const char *encodedName=QFile::encodeName(fileName).constData();
@@ -1162,6 +1360,32 @@ QString oggMimeType(const QString &fileName)
     return QLatin1String("audio/ogg");
 }
 
+QMap<QString, QString> readAll(const QString &fileName)
+{
+    QMap<QString, QString> allTags;
+    TagLib::FileRef fileref = getFileRef(fileName);
+    if (fileref.isNull()) {
+        return allTags;
+    }
+
+    #if TAGLIB_VERSION >= CANTATA_MAKE_VERSION(1,8,0)
+    TagLib::PropertyMap properties=fileref.file()->properties();
+    TagLib::PropertyMap::ConstIterator it = properties.begin();
+    TagLib::PropertyMap::ConstIterator end = properties.end();
+    for (; it!=end; ++it) {
+        allTags.insert(tString2QString(it->first.upper()), tString2QString(it->second.toString(", ")));
+    }
+
+    if (fileref.audioProperties()) {
+        TagLib::AudioProperties *properties = fileref.audioProperties();
+        allTags.insert(QLatin1String("X-AUDIO:BITRATE"), QString("%1 kb/s").arg(properties->bitrate()));
+        allTags.insert(QLatin1String("X-AUDIO:SAMPLERATE"), QString("%1 Hz").arg(properties->sampleRate()));
+        allTags.insert(QLatin1String("X-AUDIO:CHANNELS"), QString::number(properties->channels()));
+    }
+    #endif
+    return allTags;
+}
+
 QString id3Genre(int id)
 {
     // Clementine: In theory, genre 0 is "blues"; in practice it's invalid.
