@@ -22,24 +22,24 @@
  */
 
 #include "trackorganiser.h"
-#include "filenameschemedialog.h"
+#include "devices/filenameschemedialog.h"
 #ifdef ENABLE_DEVICES_SUPPORT
-#include "devicesmodel.h"
+#include "models/devicesmodel.h"
 #endif
-#include "device.h"
-#include "musiclibrarymodel.h"
-#include "dirviewmodel.h"
-#include "settings.h"
-#include "mpdconnection.h"
-#include "utils.h"
-#include "songview.h"
-#include "localize.h"
-#include "messagebox.h"
-#include "icons.h"
-#include "basicitemdelegate.h"
-#include "cuefile.h"
-#include "covers.h"
-#include "contextwidget.h"
+#include "devices/device.h"
+#include "models/musiclibrarymodel.h"
+#include "models/dirviewmodel.h"
+#include "gui/settings.h"
+#include "mpd/mpdconnection.h"
+#include "support/utils.h"
+#include "context/songview.h"
+#include "support/localize.h"
+#include "support/messagebox.h"
+#include "widgets/icons.h"
+#include "widgets/basicitemdelegate.h"
+#include "mpd/cuefile.h"
+#include "gui/covers.h"
+#include "context/contextwidget.h"
 #include <QTimer>
 #include <QFile>
 #include <QDir>
@@ -144,9 +144,10 @@ void TrackOrganiser::show(const QList<Song> &songs, const QString &udi, bool for
 
     connect(configFilename, SIGNAL(clicked()), SLOT(configureFilenameScheme()));
     connect(filenameScheme, SIGNAL(textChanged(const QString &)), this, SLOT(updateView()));
-    connect(vfatSafe, SIGNAL(stateChanged(int)), this, SLOT(updateView()));
-    connect(asciiOnly, SIGNAL(stateChanged(int)), this, SLOT(updateView()));
-    connect(ignoreThe, SIGNAL(stateChanged(int)), this, SLOT(updateView()));
+    connect(vfatSafe, SIGNAL(toggled(bool)), this, SLOT(updateView()));
+    connect(asciiOnly, SIGNAL(toggled(bool)), this, SLOT(updateView()));
+    connect(ignoreThe, SIGNAL(toggled(bool)), this, SLOT(updateView()));
+    connect(replaceSpaces, SIGNAL(toggled(bool)), this, SLOT(updateView()));
 
     if (!songsOk(origSongs, musicFolder, udi.isEmpty())) {
         return;
@@ -429,7 +430,7 @@ void TrackOrganiser::renameFile()
 
             if (deviceUdi.isEmpty()) {
                 MusicLibraryModel::self()->updateSongFile(s, to);
-                DirViewModel::self()->removeFileFromList(s.file);                
+                DirViewModel::self()->removeFileFromList(s.file);
                 DirViewModel::self()->addFileToList(origPath.isEmpty() ? to.file : origPath,
                                                     origPath.isEmpty() ? QString() : to.file);
             }
