@@ -22,8 +22,11 @@
  */
 
 #include "icons.h"
+#include "treeview.h"
 #include "config.h"
-#include "settings.h"
+#include "gui/settings.h"
+#include "support/globalstatic.h"
+#include "support/utils.cpp"
 #include <QApplication>
 #include <QPixmap>
 #include <QFont>
@@ -32,28 +35,14 @@
 #include <QDir>
 #include <QFile>
 #include <math.h>
-#if !defined Q_OS_WIN && !defined Q_OS_MAC
-#include "gtkstyle.h"
+#if !defined Q_OS_WIN && !defined Q_OS_MAC && !defined ENABLE_UBUNTU
+#include "support/gtkstyle.h"
 #endif
 
-#ifdef ENABLE_KDE_SUPPORT
-K_GLOBAL_STATIC(Icons, instance)
-#endif
-
-Icons * Icons::self()
-{
-    #ifdef ENABLE_KDE_SUPPORT
-    return instance;
-    #else
-    static Icons *instance=0;
-    if(!instance) {
-        instance=new Icons;
-    }
-    return instance;
-    #endif
-}
+GLOBAL_STATIC(Icons, instance)
 
 static QList<int> constStdSizes=QList<int>() << 16 << 22 << 32 << 48;
+static QList<int> constMonoSvgSizes=QList<int>() << constStdSizes << 64;
 
 static const int constDarkLimit=80;
 static const int constDarkValue=64;
@@ -135,19 +124,6 @@ static QPixmap createConsumeIconPixmap(int size, const QColor &col, double opaci
 }
 
 #ifndef USE_SYSTEM_MENU_ICON
-static QPainterPath buildPath(const QRectF &r, double radius)
-{
-    QPainterPath path;
-    double diameter(radius*2);
-
-    path.moveTo(r.x()+r.width(), r.y()+r.height()-radius);
-    path.arcTo(r.x()+r.width()-diameter, r.y(), diameter, diameter, 0, 90);
-    path.arcTo(r.x(), r.y(), diameter, diameter, 90, 90);
-    path.arcTo(r.x(), r.y()+r.height()-diameter, diameter, diameter, 180, 90);
-    path.arcTo(r.x()+r.width()-diameter, r.y()+r.height()-diameter, diameter, diameter, 270, 90);
-    return path;
-}
-
 static QPixmap createMenuIconPixmap(int size, QColor col, double opacity=1.0)
 {
     static const int constShadeFactor=75;
@@ -171,6 +147,7 @@ static QPixmap createMenuIconPixmap(int size, QColor col, double opacity=1.0)
         space=6;
         borderX=4;
     }
+
     int borderY=((size-((3*lineWidth)+(2*space)))/2.0)+0.5;
 
     p.setOpacity(opacity);
@@ -189,7 +166,7 @@ static QPixmap createMenuIconPixmap(int size, QColor col, double opacity=1.0)
         grad.setColorAt(0, col);
         col.setAlphaF(light ? 1.0 : 0.5);
         grad.setColorAt(1, col);
-        p.fillPath(buildPath(rect, lineWidth/2.0), grad);
+        p.fillPath(Utils::buildPath(rect, lineWidth/2.0), grad);
     }
     p.end();
     return pix;
@@ -319,9 +296,8 @@ static void updateMonoSvgIcon(Icon &i, const QString &type, const QString &name,
         Icon std;
         std.addFile(":"+type+"-"+name+"-dark", QSize(), mode);
         // Now recolour the icon!
-        QList<int> sizes=QList<int>() << 16 << 22 << 32 << 48 << 64;
         QColor col=clampColor(color, constDarkLimit, darkValue, constLightLimit, lightValue);
-        foreach (int s, sizes) {
+        foreach (int s, constMonoSvgSizes) {
             QImage img=std.pixmap(s, s, mode).toImage().convertToFormat(QImage::Format_ARGB32);
             recolourPix(img, col);
             i.addPixmap(QPixmap::fromImage(img), mode);
@@ -344,7 +320,7 @@ static Icon loadSidebarIcon(const QString &name, const QColor &normal, const QCo
     return loadMonoSvgIcon(QLatin1String("sidebar"), name, normal, selected);
 }
 
-#if !defined Q_OS_WIN && !defined Q_OS_MAC
+#if !defined Q_OS_WIN && !defined Q_OS_MAC && !defined ENABLE_UBUNTU
 static void setDisabledOpacity(Icon &icon)
 {
     static const double constDisabledOpacity=0.5;
@@ -353,18 +329,11 @@ static void setDisabledOpacity(Icon &icon)
         QIcon::State state=(QIcon::State)i;
         for (int j=0; j<4; ++j) {
             QIcon::Mode mode=(QIcon::Mode)j;
-            QList<int> sizes=constStdSizes;
-            foreach (const int sz, sizes) {
+            foreach (const int sz, constMonoSvgSizes) {
                 if (QIcon::Disabled==mode) {
                     QPixmap pix=icon.pixmap(QSize(sz, sz), QIcon::Normal, state);
                     if (!pix.isNull()) {
-                        QPixmap dis(sz, sz);
-                        dis.fill(Qt::transparent);
-                        QPainter p(&dis);
-                        p.setOpacity(constDisabledOpacity);
-                        p.drawPixmap(0, 0, pix);
-                        p.end();
-                        copy.addPixmap(dis, mode, state);
+                        copy.addPixmap(QPixmap::fromImage(TreeView::setOpacity(pix.toImage(), constDisabledOpacity)), mode, state);
                     }
                 } else {
                     copy.addPixmap(icon.pixmap(QSize(sz, sz), mode, state), mode, state);
@@ -398,11 +367,7 @@ Icons::Icons()
     streamCategoryIcon=Icon(QLatin1String("oxygen")==Icon::currentTheme().toLower() ? "inode-directory" : "folder-music");
     #endif
 
-    #ifdef Q_OS_WIN
-    QString iconFile=QCoreApplication::applicationDirPath()+"/icons/stream.png";
-    #else
-    QString iconFile=QString(INSTALL_PREFIX"/share/")+QCoreApplication::applicationName()+"/icons/stream.png";
-    #endif
+    QString iconFile=QString(CANTATA_SYS_ICONS_DIR+"stream.png");
     if (QFile::exists(iconFile)) {
         streamIcon.addFile(iconFile);
     }
@@ -434,16 +399,17 @@ Icons::Icons()
     if (importIcon.isNull()) {
         importIcon=Icon("down");
     }
-    libraryIcon=Icon::create("lib", constStdSizes);
     radioStreamIcon=Icon::create("radio", constStdSizes);
     addRadioStreamIcon=Icon::create("addradio", constStdSizes);
-    variousArtistsIcon=Icon::create("va", QList<int>() << 16 << 22 << 32 << 48 << 64 << 128);
-    artistIcon=Icon::create("artist", QList<int>() << 16 << 22 << 32 << 48 << 64 << 128);
+    variousArtistsIcon=Icon::create("va", QList<int>() << 16 << 22 << 32 << 48 << 64 << 128, true);
+    artistIcon=Icon::create("artist", QList<int>() << 16 << 22 << 32 << 48 << 64 << 128, true);
+    lastFmIcon.addFile(":lastfm.svg");
     #ifndef ENABLE_KDE_SUPPORT
-    appIcon.addFile(":cantata.svg");
-
+    #ifndef ENABLE_UBUNTU
+    appIcon=Icon("cantata");
+    #endif // ENABLE_UBUNTU
     shortcutsIcon=Icon("preferences-desktop-keyboard");
-    #if !defined Q_OS_WIN && !defined Q_OS_MAC
+    #if !defined Q_OS_WIN && !defined Q_OS_MAC && !defined ENABLE_UBUNTU
     if (shortcutsIcon.isNull()) {
         shortcutsIcon=Icon("keyboard");
     }
@@ -480,7 +446,7 @@ Icons::Icons()
         editIcon=Icon("gtk-edit");
     }
     if (clearListIcon.isNull()) {
-        clearListIcon=Icon("edit-clear");
+        clearListIcon=Icon("gtk-delete");
     }
     if (filesIcon.isNull()) {
         filesIcon=Icon("empty");
@@ -493,7 +459,7 @@ Icons::Icons()
 
     #ifdef ENABLE_STREAMS
     if (streamCategoryIcon.isNull()) {
-        streamCategoryIcon=libraryIcon;
+        streamCategoryIcon=folderIcon;
     }
     #endif
 }
@@ -564,9 +530,9 @@ void Icons::initToolbarIcons(const QColor &toolbarText)
     Q_UNUSED(toolbarText)
     #endif
     QColor stdColor=calcIconColor();
-    #if !defined Q_OS_WIN && !defined Q_OS_MAC
+    #if !defined Q_OS_WIN && !defined Q_OS_MAC && !defined ENABLE_UBUNTU
     if (GtkStyle::useSymbolicIcons()) {
-        bool rtl=Qt::RightToLeft==QApplication::layoutDirection();
+        bool rtl=QApplication::isRightToLeft();
         QColor col=GtkStyle::symbolicColor();
         toolbarPrevIcon=loadMediaIcon(QLatin1String(rtl ? "prev-rtl" : "prev"), col, col);
         toolbarPlayIcon=loadMediaIcon(QLatin1String(rtl ? "play-rtl" : "play"), col, col);
