@@ -23,27 +23,30 @@
 
 #include "itemview.h"
 #include "groupedview.h"
-#include "mainwindow.h"
-#include "covers.h"
-#include "proxymodel.h"
+#include "tableview.h"
+#include "models/roles.h"
+#include "gui/covers.h"
+#include "models/proxymodel.h"
 #include "actionitemdelegate.h"
 #include "basicitemdelegate.h"
-#include "actionmodel.h"
-#include "localize.h"
-#include "icon.h"
+#include "models/actionmodel.h"
+#include "support/localize.h"
+#include "support/icon.h"
 #include "config.h"
-#include "gtkstyle.h"
-#include "spinner.h"
+#include "support/gtkstyle.h"
+#include "support/spinner.h"
 #include "messageoverlay.h"
+#include "support/action.h"
+#include "support/actioncollection.h"
+#include "support/configuration.h"
 #include <QToolButton>
 #include <QStyle>
 #include <QStyleOptionViewItem>
 #include <QPainter>
 #include <QAction>
 #include <QTimer>
-#ifdef ENABLE_KDE_SUPPORT
-#include <KDE/KGlobalSettings>
-#endif
+#include <QKeyEvent>
+#include <QProxyStyle>
 
 static int listDecorationSize=22;
 static int treeDecorationSize=16;
@@ -61,14 +64,39 @@ void ItemView::setup()
     }
 }
 
-ViewEventHandler::ViewEventHandler(ActionItemDelegate *d, QAbstractItemView *v)
+KeyEventHandler::KeyEventHandler(QAbstractItemView *v, QAction *a)
     : QObject(v)
-    , delegate(d)
     , view(v)
+    , deleteAct(a)
     , interceptBackspace(qobject_cast<ListView *>(view))
 {
 }
 
+bool KeyEventHandler::eventFilter(QObject *obj, QEvent *event)
+{
+    if (view->hasFocus()) {
+        if (QEvent::KeyRelease==event->type()) {
+            QKeyEvent *keyEvent=static_cast<QKeyEvent *>(event);
+            if (deleteAct && Qt::Key_Delete==keyEvent->key() && Qt::NoModifier==keyEvent->modifiers()) {
+                deleteAct->trigger();
+                return true;
+            }
+        } else if (QEvent::KeyPress==event->type()) {
+            QKeyEvent *keyEvent=static_cast<QKeyEvent *>(event);
+            if (interceptBackspace && Qt::Key_Backspace==keyEvent->key() && Qt::NoModifier==keyEvent->modifiers()) {
+                emit backspacePressed();
+            }
+        }
+    }
+    return QObject::eventFilter(obj, event);
+}
+
+ViewEventHandler::ViewEventHandler(ActionItemDelegate *d, QAbstractItemView *v)
+    : KeyEventHandler(v, 0)
+    , delegate(d)
+{
+}
+
 // HACK time. For some reason, IconView is not always re-drawn when mouse leaves the view.
 // We sometimes get an item that is left in the mouse-over state. So, work-around this by
 // keeping track of when mouse is over listview.
@@ -84,16 +112,7 @@ bool ViewEventHandler::eventFilter(QObject *obj, QEvent *event)
         }
     }
 
-    if (interceptBackspace && view->hasFocus()) {
-        if (QEvent::KeyPress==event->type()) {
-            QKeyEvent *keyEvent=static_cast<QKeyEvent *>(event);
-            if (Qt::Key_Backspace==keyEvent->key() && Qt::NoModifier==keyEvent->modifiers()) {
-                emit escPressed();
-            }
-        }
-    }
-
-    return QObject::eventFilter(obj, event);
+    return KeyEventHandler::eventFilter(obj, event);
 }
 
 static const int constImageSize=22;
@@ -101,7 +120,7 @@ static const int constDevImageSize=32;
 
 static inline double subTextAlpha(bool selected)
 {
-    return selected ? 0.7 : 0.6;
+    return selected ? 0.7 : 0.5;
 }
 
 class ListDelegate : public ActionItemDelegate
@@ -120,7 +139,7 @@ public:
     QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
     {
         Q_UNUSED(option)
-        int imageSize = index.data(ItemView::Role_ImageSize).toInt();
+        int imageSize = index.data(Cantata::Role_ImageSize).toInt();
 
         if (imageSize<0) {
             imageSize=constImageSize;
@@ -130,17 +149,17 @@ public:
             // Use same calculation as in LibraryPage/AlbumsPage...
             return QSize(imageSize+8, imageSize+(QApplication::fontMetrics().height()*2.5));
         } else {
-            // TODO: Any poit to checking one-line here? All models return sub-text...
-            //       Thisngs will be quicker if we dont call SubText here...
-            bool oneLine = false ; // index.data(ItemView::Role_SubText).toString().isEmpty();
-            bool showCapacity = !index.data(ItemView::Role_CapacityText).toString().isEmpty();
+            // TODO: Any point to checking one-line here? All models return sub-text...
+            //       Things will be quicker if we dont call SubText here...
+            bool oneLine = false ; // index.data(Cantata::Role_SubText).toString().isEmpty();
+            bool showCapacity = !index.data(Cantata::Role_CapacityText).toString().isEmpty();
             int textHeight = QApplication::fontMetrics().height()*(oneLine ? 1 : 2);
 
             if (showCapacity) {
                 imageSize=constDevImageSize;
             }
             return QSize(qMax(64, imageSize) + (constBorder * 2),
-                         qMax(textHeight, imageSize) + (constBorder*2) + (int)((showCapacity ? textHeight*0.8 : 0)+0.5));
+                         qMax(textHeight, imageSize) + (constBorder*2) + (int)((showCapacity ? textHeight*Utils::smallFontFactor(QApplication::font()) : 0)+0.5));
         }
     }
 
@@ -154,6 +173,7 @@ public:
         bool selected=option.state&QStyle::State_Selected;
         bool active=option.state&QStyle::State_Active;
         bool drawBgnd=true;
+        bool iconMode = view && QListView::IconMode==view->viewMode();
         QStyleOptionViewItemV4 opt(option);
         opt.showDecorationSelected=true;
 
@@ -172,16 +192,19 @@ public:
             }
         }
 
-        QString capacityText=index.data(ItemView::Role_CapacityText).toString();
+        QString capacityText=index.data(Cantata::Role_CapacityText).toString();
         bool showCapacity = !capacityText.isEmpty();
-        QString text = index.data(ItemView::Role_MainText).toString();
+        QString text = iconMode ? index.data(Cantata::Role_BriefMainText).toString() : QString();
+        if (text.isEmpty()) {
+            text=index.data(Cantata::Role_MainText).toString();
+        }
         if (text.isEmpty()) {
             text=index.data(Qt::DisplayRole).toString();
         }
         QRect r(option.rect);
         QRect r2(r);
-        QString childText = index.data(ItemView::Role_SubText).toString();
-        QVariant image = index.data(ItemView::Role_Image);
+        QString childText = index.data(Cantata::Role_SubText).toString();
+        QVariant image = index.data(Cantata::Role_Image);
         if (image.isNull()) {
             image = index.data(Qt::DecorationRole);
         }
@@ -193,9 +216,8 @@ public:
                                                                               Qt::KeepAspectRatio, Qt::SmoothTransformation))
                             : image.value<QIcon>().pixmap(listDecorationSize, listDecorationSize);
         bool oneLine = childText.isEmpty();
-        bool iconMode = view && QListView::IconMode==view->viewMode();
         ActionPos actionPos = iconMode ? AP_VTop : AP_HMiddle;
-        bool rtl = Qt::RightToLeft==QApplication::layoutDirection();
+        bool rtl = QApplication::isRightToLeft();
 
         if (childText==QLatin1String("-")) {
             childText.clear();
@@ -213,7 +235,7 @@ public:
         }
 
         if (AP_VTop==actionPos) {
-            r.adjust(constBorder, constBorder, -constBorder, -constBorder);
+            r.adjust(constBorder, constBorder*4, -constBorder, -constBorder);
             r2=r;
         } else {
             r.adjust(constBorder, 0, -constBorder, 0);
@@ -263,10 +285,6 @@ public:
             }
         }
 
-//        if (!mouseOver) {
-//            drawIcons(painter, AP_VTop==actionPos ? r2 : r, false, rtl, actionPos, index);
-//        }
-
         QRect textRect;
         QColor color(option.palette.color(active ? QPalette::Active : QPalette::Inactive, selected ? QPalette::HighlightedText : QPalette::Text));
         QTextOption textOpt(AP_VTop==actionPos ? Qt::AlignHCenter|Qt::AlignVCenter : Qt::AlignVCenter);
@@ -279,15 +297,15 @@ public:
             painter->setFont(textFont);
             painter->drawText(textRect, text, textOpt);
         } else {
-            QFont childFont(QApplication::font());
-            childFont.setPointSizeF(childFont.pointSizeF()*0.8);
+            QFont childFont(Utils::smallFont(textFont));
             QFontMetrics childMetrics(childFont);
             QRect childRect;
 
             int childHeight=childMetrics.height();
             int totalHeight=textHeight+childHeight;
             textRect=QRect(r.x(), r.y()+((r.height()-totalHeight)/2), r.width(), textHeight);
-            childRect=QRect(r.x(), r.y()+textHeight+((r.height()-totalHeight)/2), r.width(), textHeight);
+            childRect=QRect(r.x(), r.y()+textHeight+((r.height()-totalHeight)/2), r.width(),
+                            (iconMode ? childHeight-(2*constBorder) : childHeight));
 
             text = textMetrics.elidedText(text, Qt::ElideRight, textRect.width(), QPalette::WindowText);
             painter->setPen(color);
@@ -305,7 +323,7 @@ public:
 
         if (showCapacity) {
             QStyleOptionProgressBar opt;
-            double capacity=index.data(ItemView::Role_Capacity).toDouble();
+            double capacity=index.data(Cantata::Role_Capacity).toDouble();
 
             if (capacity<0.0) {
                 capacity=0.0;
@@ -326,8 +344,8 @@ public:
             QApplication::style()->drawControl(QStyle::CE_ProgressBar, &opt, painter, 0L);
         }
 
-        if (drawBgnd && mouseOver) {
-            drawIcons(painter, AP_VTop==actionPos ? r2 : r, true, rtl, actionPos, index);
+        if ((drawBgnd && mouseOver) || Utils::touchFriendly()) {
+            drawIcons(painter, AP_VTop==actionPos ? option.rect : r, mouseOver || (selected && Utils::touchFriendly()), rtl, actionPos, index);
         }
         if (!iconMode) {
             BasicItemDelegate::drawLine(painter, option.rect, color);
@@ -354,12 +372,18 @@ public:
 
     QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
     {
-        if (!simpleStyle || !index.data(ItemView::Role_CapacityText).toString().isEmpty()) {
+        if (!simpleStyle || !index.data(Cantata::Role_CapacityText).toString().isEmpty()) {
             return ListDelegate::sizeHint(option, index);
         }
 
         QSize sz(QStyledItemDelegate::sizeHint(option, index));
-
+        if (Utils::touchFriendly()) {
+            int minH=option.fontMetrics.height()*2;
+            if (sz.height()<minH) {
+                sz.setHeight(minH);
+                return sz;
+            }
+        }
         return QSize(sz.width(), sz.height()+2);
     }
 
@@ -369,14 +393,14 @@ public:
             return;
         }
 
-        if (!simpleStyle || !index.data(ItemView::Role_CapacityText).toString().isEmpty()) {
+        if (!simpleStyle || !index.data(Cantata::Role_CapacityText).toString().isEmpty()) {
             ListDelegate::paint(painter, option, index);
             return;
         }
 
         QStringList text=index.data(Qt::DisplayRole).toString().split("\n");
         bool gtk=GtkStyle::isActive();
-        bool rtl = Qt::RightToLeft==QApplication::layoutDirection();
+        bool rtl = QApplication::isRightToLeft();
         bool selected=option.state&QStyle::State_Selected;
         bool active=option.state&QStyle::State_Active;
         bool mouseOver=underMouse && option.state&QStyle::State_MouseOver;
@@ -393,7 +417,7 @@ public:
             r.adjust(4, 0, -4, 0);
             QPixmap pix=noIcons ? QPixmap() : index.data(Qt::DecorationRole).value<QIcon>().pixmap(treeDecorationSize, treeDecorationSize);
             if (gtk && pix.isNull()) {
-                QVariant image = index.data(ItemView::Role_Image);
+                QVariant image = index.data(Cantata::Role_Image);
                 if (!image.isNull()) {
                     pix = QVariant::Pixmap==image.type() ? image.value<QPixmap>() : noIcons ? QPixmap() : image.value<QIcon>().pixmap(listDecorationSize, listDecorationSize);
                 }
@@ -439,8 +463,8 @@ public:
             }
         }
 
-        if (mouseOver) {
-            drawIcons(painter, option.rect, true, rtl, AP_HMiddle, index);
+        if (mouseOver || Utils::touchFriendly()) {
+            drawIcons(painter, option.rect, mouseOver || (selected && Utils::touchFriendly()), rtl, AP_HMiddle, index);
         }
         BasicItemDelegate::drawLine(painter, option.rect, option.palette.color(active ? QPalette::Active : QPalette::Inactive,
                                                                                selected ? QPalette::HighlightedText : QPalette::Text));
@@ -500,7 +524,7 @@ public:
                 BasicItemDelegate::drawLine(painter, option.rect, col, true, true);
             }
             if (drawActions && underMouse && option.state&QStyle::State_MouseOver) {
-                drawIcons(painter, option.rect, true, Qt::RightToLeft==QApplication::layoutDirection(), AP_HMiddle, index);
+                drawIcons(painter, option.rect, true, QApplication::isRightToLeft(), AP_HMiddle, index);
             }
         } else {
             BasicItemDelegate::drawLine(painter, option.rect, col, false, false);
@@ -543,6 +567,36 @@ QString ItemView::modeStr(Mode m)
 }
 
 static const char *constPageProp="page";
+static Action *backAction=0;
+
+// ALL this just becasue styles do not elide the text of toolbuttons!!!!! :-(
+class BackButtonProxyStyle : public QProxyStyle
+{
+public:
+    BackButtonProxyStyle()
+        : QProxyStyle()
+    {
+        setBaseStyle(qApp->style());
+    }
+    
+    void drawComplexControl(ComplexControl control, const QStyleOptionComplex *option, QPainter *painter, const QWidget *widget) const
+    {
+        if (CC_ToolButton==control) {
+            if (const QStyleOptionToolButton *toolbutton = qstyleoption_cast<const QStyleOptionToolButton *>(option)) {
+                QStyleOptionToolButton copy=*toolbutton;
+                int iconWidth = copy.iconSize.isValid() ? copy.iconSize.width() : baseStyle()->pixelMetric(PM_ToolBarIconSize, option, widget);
+                int frameWidth=baseStyle()->pixelMetric(PM_DefaultFrameWidth, option, widget);
+                QFontMetrics fm(painter->fontMetrics());
+                copy.text=fm.elidedText(toolbutton->text, Qt::ElideRight, toolbutton->rect.width()-(iconWidth+8+(2*frameWidth)), QPalette::WindowText);
+                baseStyle()->drawComplexControl(control, &copy, painter, widget);
+                return;
+            }
+        }
+        baseStyle()->drawComplexControl(control, option, painter, widget);
+    }
+};
+
+const QLatin1String ItemView::constSearchActiveKey("searchActive");
 
 ItemView::ItemView(QWidget *p)
     : QWidget(p)
@@ -558,27 +612,24 @@ ItemView::ItemView(QWidget *p)
     , searchResetLevel(0)
 {
     setupUi(this);
-    backAction = new QAction(i18n("Go Back"), this);
-    backAction->setIcon(Icon("go-previous"));
+    if (!backAction) {
+        backAction=ActionCollection::get()->createAction("itemview-goback", i18n("Go Back"), Icon("go-previous"));
+        backAction->setShortcut(Qt::AltModifier+(Qt::LeftToRight==layoutDirection() ? Qt::Key_Left : Qt::Key_Right));
+    }
     backButton->setDefaultAction(backAction);
-    backAction->setShortcut(Qt::AltModifier+(Qt::LeftToRight==layoutDirection() ? Qt::Key_Left : Qt::Key_Right));
+    backButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
+    Icon::init(backButton);
+    backButton->setAutoRaise(true);
+    #ifdef Q_OS_MAC
+    backButton->setStyleSheet("QToolButton {border: 0}");
+    #endif
+    backButton->setFocusPolicy(Qt::NoFocus);
+    backButton->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
+    backButton->setStyle(new BackButtonProxyStyle());
     Action::updateToolTip(backAction);
-    homeAction = new QAction(i18n("Go Home"), this);
-    homeAction->setIcon(Icon("go-home"));
-    homeButton->setDefaultAction(homeAction);
-    homeAction->setShortcut(Qt::AltModifier+Qt::Key_Up);
-    Action::updateToolTip(homeAction);
-    listView->addAction(backAction);
-    listView->addAction(homeAction);
-    listView->addDefaultAction(backAction);
-    listView->addDefaultAction(homeAction);
     QAction *sep=new QAction(this);
     sep->setSeparator(true);
     listView->addAction(sep);
-    Icon::init(backButton);
-    Icon::init(homeButton);
-    title->installEventFilter(this);
-    title->setAttribute(Qt::WA_Hover, true);
     treeView->setPageDefaults();
     // Some styles, eg Cleanlooks/Plastique require that we explicitly set mouse tracking on the treeview.
     treeView->setAttribute(Qt::WA_MouseTracking);
@@ -587,9 +638,12 @@ ItemView::ItemView(QWidget *p)
     TreeDelegate *td=new TreeDelegate(treeView);
     listView->setItemDelegate(ld);
     treeView->setItemDelegate(td);
+    listView->setProperty(GtkStyle::constHideFrameProp, true);
+    treeView->setProperty(GtkStyle::constHideFrameProp, true);
     ViewEventHandler *listViewEventHandler=new ViewEventHandler(ld, listView);
-    listView->installEventFilter(listViewEventHandler);
-    treeView->installEventFilter(new ViewEventHandler(td, treeView));
+    ViewEventHandler *treeViewEventHandler=new ViewEventHandler(td, treeView);
+    listView->installFilter(listViewEventHandler);
+    treeView->installFilter(treeViewEventHandler);
     connect(searchWidget, SIGNAL(returnPressed()), this, SLOT(delaySearchItems()));
     connect(searchWidget, SIGNAL(textChanged(const QString)), this, SLOT(delaySearchItems()));
     connect(searchWidget, SIGNAL(active(bool)), this, SLOT(searchActive(bool)));
@@ -599,11 +653,10 @@ ItemView::ItemView(QWidget *p)
     connect(treeView, SIGNAL(clicked(const QModelIndex &)),  this, SLOT(itemClicked(const QModelIndex &)));
     connect(listView, SIGNAL(itemsSelected(bool)), this, SIGNAL(itemsSelected(bool)));
     connect(listView, SIGNAL(activated(const QModelIndex &)), this, SLOT(activateItem(const QModelIndex &)));
-    connect(listView, SIGNAL(doubleClicked(const QModelIndex &)), this, SIGNAL(doubleClicked(const QModelIndex &)));
+    connect(listView, SIGNAL(itemDoubleClicked(const QModelIndex &)), this, SIGNAL(doubleClicked(const QModelIndex &)));
     connect(listView, SIGNAL(clicked(const QModelIndex &)),  this, SLOT(itemClicked(const QModelIndex &)));
     connect(backAction, SIGNAL(triggered(bool)), this, SLOT(backActivated()));
-    connect(homeAction, SIGNAL(triggered(bool)), this, SLOT(homeActivated()));
-    connect(listViewEventHandler, SIGNAL(escPressed()), this, SLOT(backActivated()));
+    connect(listViewEventHandler, SIGNAL(backspacePressed()), this, SLOT(backActivated()));
     searchWidget->setVisible(false);
 }
 
@@ -611,6 +664,18 @@ ItemView::~ItemView()
 {
 }
 
+void ItemView::load(const QString &group)
+{
+    if (Configuration(group).get(ItemView::constSearchActiveKey, false)) {
+        focusSearch();
+    }
+}
+
+void ItemView::save(const QString &group)
+{
+    Configuration(group).set(ItemView::constSearchActiveKey, searchWidget->isActive());
+}
+
 void ItemView::allowGroupedView()
 {
     if (!groupedView) {
@@ -619,13 +684,15 @@ void ItemView::allowGroupedView()
         groupedView->setProperty(constPageProp, stackedWidget->count()-1);
         // Some styles, eg Cleanlooks/Plastique require that we explicitly set mouse tracking on the treeview.
         groupedView->setAttribute(Qt::WA_MouseTracking, true);
-        groupedView->installEventFilter(new ViewEventHandler(qobject_cast<ActionItemDelegate *>(groupedView->itemDelegate()), groupedView));
+        ViewEventHandler *viewHandler=new ViewEventHandler(qobject_cast<ActionItemDelegate *>(groupedView->itemDelegate()), groupedView);
+        groupedView->installFilter(viewHandler);
         groupedView->setAutoExpand(false);
         groupedView->setMultiLevel(true);
         connect(groupedView, SIGNAL(itemsSelected(bool)), this, SIGNAL(itemsSelected(bool)));
         connect(groupedView, SIGNAL(itemActivated(const QModelIndex &)), this, SLOT(itemActivated(const QModelIndex &)));
         connect(groupedView, SIGNAL(doubleClicked(const QModelIndex &)), this, SIGNAL(doubleClicked(const QModelIndex &)));
         connect(groupedView, SIGNAL(clicked(const QModelIndex &)), this, SLOT(itemClicked(const QModelIndex &)));
+        groupedView->setProperty(GtkStyle::constHideFrameProp, true);
     }
 }
 
@@ -636,12 +703,13 @@ void ItemView::allowTableView(TableView *v)
         tableView->setParent(stackedWidget);
         stackedWidget->addWidget(tableView);
         tableView->setProperty(constPageProp, stackedWidget->count()-1);
-        // Some styles, eg Cleanlooks/Plastique require that we explicitly set mouse tracking on the treeview.
-//        tableView->installEventFilter(new ViewEventHandler(0, tableView));
+        ViewEventHandler *viewHandler=new ViewEventHandler(0, tableView);
+        tableView->installFilter(viewHandler);
         connect(tableView, SIGNAL(itemsSelected(bool)), this, SIGNAL(itemsSelected(bool)));
         connect(tableView, SIGNAL(itemActivated(const QModelIndex &)), this, SLOT(itemActivated(const QModelIndex &)));
         connect(tableView, SIGNAL(doubleClicked(const QModelIndex &)), this, SIGNAL(doubleClicked(const QModelIndex &)));
         connect(tableView, SIGNAL(clicked(const QModelIndex &)), this, SLOT(itemClicked(const QModelIndex &)));
+        tableView->setProperty(GtkStyle::constHideFrameProp, true);
     }
 }
 
@@ -667,13 +735,13 @@ void ItemView::setMode(Mode m)
         return;
     }
 
+    searchWidget->setText(QString());
     QIcon oldBgndIcon=bgndIcon;
     if (!bgndIcon.isNull()) {
         setBackgroundImage(QIcon());
     }
 
     mode=m;
-    searchWidget->setText(QString());
     int stackIndex=0;
     if (usingTreeView()) {
         listView->setModel(0);
@@ -777,9 +845,9 @@ void ItemView::setLevel(int l, bool haveChildren)
 {
     int prev=currentLevel;
     currentLevel=l;
-    backAction->setEnabled(0!=l);
-    homeAction->setEnabled(l>1);
-    homeAction->setVisible(l>1);
+    if (isVisible()) {
+        backAction->setEnabled(0!=currentLevel);
+    }
 
     if (Mode_IconTop==mode) {
         if (0==currentLevel || haveChildren) {
@@ -790,7 +858,7 @@ void ItemView::setLevel(int l, bool haveChildren)
 //                listView->setAlternatingRowColors(false);
                 listView->setWordWrap(true);
                 listView->setDragDropMode(QAbstractItemView::DragOnly);
-                static_cast<ActionItemDelegate *>(listView->itemDelegate())->setLargeIcons(iconGridSize.width()>(ActionItemDelegate::constLargeActionIconSize*6));
+                static_cast<ActionItemDelegate *>(listView->itemDelegate())->setLargeIcons(iconGridSize.width()>(ActionItemDelegate::constLargeActionIconSize*4));
             }
         } else if(QListView::ListMode!=listView->viewMode()) {
             listView->setGridSize(listGridSize);
@@ -808,14 +876,13 @@ void ItemView::setLevel(int l, bool haveChildren)
     }
 
     backButton->setVisible(currentLevel>0);
-    title->setVisible(currentLevel>0);
-    homeButton->setVisible(currentLevel>1);
 
     if (currentLevel>0) {
         if (prev>currentLevel) {
-            title->setText(prevText[currentLevel]);
+            currentText=prevText[currentLevel];
+            backButton->setText(prevText[currentLevel]);
         } else {
-            prevText.insert(prev, title->text());
+            prevText.insert(prev, backButton->text());
         }
     }
 }
@@ -844,6 +911,11 @@ void ItemView::setModel(ProxyModel *m)
     view()->setModel(m);
 }
 
+bool ItemView::searchVisible() const
+{
+    return searchWidget->isVisible();
+}
+
 QString ItemView::searchText() const
 {
     return searchWidget->isVisible() ? searchWidget->text() : QString();
@@ -916,13 +988,29 @@ void ItemView::update()
 
 void ItemView::setDeleteAction(QAction *act)
 {
-    listView->installEventFilter(new DeleteKeyEventHandler(listView, act));
-    treeView->installEventFilter(new DeleteKeyEventHandler(treeView, act));
+    if (!listView->filter() || !qobject_cast<KeyEventHandler *>(listView->filter())) {
+        listView->installFilter(new KeyEventHandler(listView, act));
+    } else {
+        static_cast<KeyEventHandler *>(listView->filter())->setDeleteAction(act);
+    }
+    if (!treeView->filter() || !qobject_cast<KeyEventHandler *>(treeView->filter())) {
+        treeView->installEventFilter(new KeyEventHandler(treeView, act));
+    } else {
+        static_cast<KeyEventHandler *>(treeView->filter())->setDeleteAction(act);
+    }
     if (groupedView) {
-        groupedView->installEventFilter(new DeleteKeyEventHandler(groupedView, act));
+        if (!groupedView->filter() || !qobject_cast<KeyEventHandler *>(groupedView->filter())) {
+            groupedView->installEventFilter(new KeyEventHandler(groupedView, act));
+        } else {
+            static_cast<KeyEventHandler *>(groupedView->filter())->setDeleteAction(act);
+        }
     }
     if (tableView) {
-        tableView->installEventFilter(new DeleteKeyEventHandler(tableView, act));
+        if (!tableView->filter() || !qobject_cast<KeyEventHandler *>(tableView->filter())) {
+            tableView->installEventFilter(new KeyEventHandler(tableView, act));
+        } else {
+            static_cast<KeyEventHandler *>(tableView->filter())->setDeleteAction(act);
+        }
     }
 }
 
@@ -967,10 +1055,12 @@ void ItemView::showIndex(const QModelIndex &idx, bool scrollTo)
     }
 }
 
-void ItemView::focusSearch()
+void ItemView::focusSearch(const QString &text)
 {
-    performedSearch=false;
-    searchWidget->activate();
+    if (isEnabled()) {
+        performedSearch=false;
+        searchWidget->activate(searchWidget->text().isEmpty() ? text : QString());
+    }
 }
 
 void ItemView::focusView()
@@ -1111,41 +1201,14 @@ void ItemView::setSearchCategory(const QString &id)
     searchWidget->setCategory(id);
 }
 
-void ItemView::hideBackAction()
+void ItemView::showEvent(QShowEvent *ev)
 {
-    backAction->setEnabled(false);
-    backAction->setVisible(false);
-}
-
-bool ItemView::eventFilter(QObject *o, QEvent *e)
-{
-    if (o==title) {
-        static const char * constPressesProperty="pressed";
-        switch (e->type()) {
-        case QEvent::MouseButtonPress:
-            if (Qt::NoModifier==static_cast<QMouseEvent *>(e)->modifiers() && Qt::LeftButton==static_cast<QMouseEvent *>(e)->button()) {
-                title->setProperty(constPressesProperty, true);
-            }
-            break;
-        case QEvent::MouseButtonRelease:
-            if (title->property(constPressesProperty).toBool()) {
-                backActivated();
-            }
-            title->setProperty(constPressesProperty, false);
-            break;
-        case QEvent::HoverEnter: {
-            QColor bgnd=palette().highlight().color();
-            title->setStyleSheet(QString("QLabel{background:rgb(%1,%2,%3,38);}")
-                                 .arg(bgnd.red()).arg(bgnd.green()).arg(bgnd.blue()));
-            break;
-        }
-        case QEvent::HoverLeave:
-            title->setStyleSheet(QString());
-        default:
-            break;
-        }
+    QWidget::showEvent(ev);
+    backAction->setEnabled(0!=currentLevel);
+    if (0!=currentLevel) {
+        // For some reason this gets overridden with the action text!
+        backButton->setText(currentText);
     }
-    return QWidget::eventFilter(o, e);
 }
 
 void ItemView::showSpinner(bool v)
@@ -1195,17 +1258,6 @@ void ItemView::backActivated()
     }
 }
 
-void ItemView::homeActivated()
-{
-    if (!usingListView() || 0==currentLevel || !isVisible()) {
-        return;
-    }
-    setLevel(0);
-    itemModel->setRootIndex(QModelIndex());
-    listView->setRootIndex(QModelIndex());
-    emit rootIndexSet(QModelIndex());
-}
-
 void ItemView::setExpanded(const QModelIndex &idx, bool exp)
 {
     if (usingTreeView()) {
@@ -1262,13 +1314,12 @@ void ItemView::activateItem(const QModelIndex &index, bool emitRootSet)
             prevTopIndex=static_cast<QSortFilterProxyModel *>(listView->model())->mapToSource(prevTopIndex);
         }
         setLevel(currentLevel+1, index.child(0, 0).child(0, 0).isValid());
-        QString text=index.data(Role_TitleText).toString();
+        QString text=index.data(Cantata::Role_TitleText).toString();
         if (text.isEmpty()) {
             text=index.data(Qt::DisplayRole).toString();
         }
-        // Add padding so that text is not right on border of label widget.
-        // This spacing makes the mouse-over background change look nicer.
-        title->setText(QLatin1String("  ")+text);
+        currentText=text;
+        backButton->setText(text);
         listView->setRootIndex(index);
         itemModel->setRootIndex(index);
         if (emitRootSet) {
