@@ -22,31 +22,21 @@
  */
 
 #include "sizewidget.h"
-#include "genrecombo.h"
 #include "toolbutton.h"
-#include <QIcon>
-
-static int stdHeight=-1;
-
-int SizeWidget::standardHeight()
-{
-    if (-1==stdHeight)
-    {
-        GenreCombo combo(0);
-        combo.ensurePolished();
-        ToolButton tb(0);
-        tb.setToolButtonStyle(Qt::ToolButtonIconOnly);
-        tb.setIcon(QIcon::fromTheme("ok"));
-        tb.ensurePolished();
-        stdHeight=qMax(tb.sizeHint().height(), combo.sizeHint().height());
-    }
-
-    return stdHeight;
-}
+#include "icons.h"
 
 SizeWidget::SizeWidget(QWidget *parent)
-    : QWidget(parent)
+    : ComboBox(parent)
 {
-    setFixedHeight(stdHeight);
+    #ifdef Q_OS_MAC
+    // For some reason, using 0 pixles on Mac causes genre combo to overlap
+    setFixedWidth(12);
+    #else
     setFixedWidth(0);
+    #endif
+}
+
+void SizeWidget::paintEvent(QPaintEvent *e)
+{
+    Q_UNUSED(e);
 }
