@@ -31,11 +31,6 @@ class TableView : public TreeView
     Q_OBJECT
 
 public:
-    enum Roles {
-        Role_Width = Qt::UserRole+400,
-        Role_Hideable
-    };
-
     TableView(const QString &cfgName, QWidget *parent=0, bool menuAlwaysAllowed=false);
     virtual ~TableView() { }
     void initHeader();
